@@ -15,6 +15,7 @@
 
 #include "dive.h"
 #include "file.h"
+#include "subsurface-time.h"
 #include "units.h"
 #include "sha1.h"
 #include "gettext.h"
@@ -675,20 +676,24 @@ static void cochran_parse_dive(const unsigned char *decode, unsigned mod,
 	case TYPE_GEMINI:
 	case TYPE_COMMANDER:
 		if (config.type == TYPE_GEMINI) {
+			cylinder_t cyl = empty_cylinder;
 			dc->model = "Gemini";
 			dc->deviceid = buf[0x18c] * 256 + buf[0x18d];	// serial no
-			fill_default_cylinder(dive, 0);
-			dive->cylinder[0].gasmix.o2.permille = (log[CMD_O2_PERCENT] / 256
+			fill_default_cylinder(dive, &cyl);
+			cyl.gasmix.o2.permille = (log[CMD_O2_PERCENT] / 256
 				+ log[CMD_O2_PERCENT + 1]) * 10;
-			dive->cylinder[0].gasmix.he.permille = 0;
+			cyl.gasmix.he.permille = 0;
+			add_cylinder(&dive->cylinders, 0, cyl);
 		} else {
 			dc->model = "Commander";
 			dc->deviceid = array_uint32_le(buf + 0x31e);	// serial no
 			for (g = 0; g < 2; g++) {
-				fill_default_cylinder(dive, g);
-				dive->cylinder[g].gasmix.o2.permille = (log[CMD_O2_PERCENT + g * 2] / 256
+				cylinder_t cyl = empty_cylinder;
+				fill_default_cylinder(dive, &cyl);
+				cyl.gasmix.o2.permille = (log[CMD_O2_PERCENT + g * 2] / 256
 					+ log[CMD_O2_PERCENT + g * 2 + 1]) * 10;
-				dive->cylinder[g].gasmix.he.permille = 0;
+				cyl.gasmix.he.permille = 0;
+				add_cylinder(&dive->cylinders, g, cyl);
 			}
 		}
 
@@ -727,13 +732,15 @@ static void cochran_parse_dive(const unsigned char *decode, unsigned mod,
 		dc->model = "EMC";
 		dc->deviceid = array_uint32_le(buf + 0x31e);	// serial no
 		for (g = 0; g < 4; g++) {
-			fill_default_cylinder(dive , g);
-			dive->cylinder[g].gasmix.o2.permille =
+			cylinder_t cyl = empty_cylinder;
+			fill_default_cylinder(dive, &cyl);
+			cyl.gasmix.o2.permille =
 				(log[EMC_O2_PERCENT + g * 2] / 256
 				+ log[EMC_O2_PERCENT + g * 2 + 1]) * 10;
-			dive->cylinder[g].gasmix.he.permille =
+			cyl.gasmix.he.permille =
 				(log[EMC_HE_PERCENT + g * 2] / 256
 				+ log[EMC_HE_PERCENT + g * 2 + 1]) * 10;
+			add_cylinder(&dive->cylinders, g, cyl);
 		}
 
 		tm.tm_year = log[EMC_YEAR];
