@@ -128,31 +128,25 @@ struct divecomputer {
 	struct divecomputer *next;
 };
 
-typedef struct dive_table {
-	int nr, allocated;
-	struct dive **dives;
-} dive_table_t;
-
 struct picture;
 struct dive_site;
 struct dive_site_table;
+struct dive_table;
 struct dive_trip;
 struct trip_table;
+struct full_text_cache;
 struct dive {
-	int number;
-	bool notrip; /* Don't autogroup this dive to a trip */
 	struct dive_trip *divetrip;
-	bool selected;
-	bool hidden_by_filter;
 	timestamp_t when;
 	struct dive_site *dive_site;
 	char *notes;
 	char *divemaster, *buddy;
-	int rating;
-	int visibility; /* 0 - 5 star rating */
-	cylinder_t cylinder[MAX_CYLINDERS];
+	struct cylinder_table cylinders;
 	struct weightsystem_table weightsystems;
 	char *suit;
+	int number;
+	int rating;
+	int wavesize, current, visibility, surge, chill; /* 0 - 5 star ratings */
 	int sac, otu, cns, maxcns;
 
 	/* Calculated based on dive computer data */
@@ -161,12 +155,18 @@ struct dive {
 	pressure_t surface_pressure;
 	duration_t duration;
 	int salinity; // kg per 10000 l
+	int user_salinity; // water density reflecting a user-specified type
 
 	struct tag_entry *tag_list;
 	struct divecomputer dc;
 	int id; // unique ID for this dive
 	struct picture *picture_list;
 	unsigned char git_id[20];
+	bool notrip; /* Don't autogroup this dive to a trip */
+	bool selected;
+	bool hidden_by_filter;
+	struct full_text_cache *full_text; /* word cache for full text search */
+	bool invalid;
 };
 
 /* For the top-level list: an entry is either a dive or a trip */
@@ -180,7 +180,7 @@ extern bool dive_cache_is_valid(const struct dive *dive);
 
 extern int get_cylinder_idx_by_use(const struct dive *dive, enum cylinderuse cylinder_use_type);
 extern void cylinder_renumber(struct dive *dive, int mapping[]);
-extern int same_gasmix_cylinder(cylinder_t *cyl, int cylid, struct dive *dive, bool check_unused);
+extern int same_gasmix_cylinder(const cylinder_t *cyl, int cylid, const struct dive *dive, bool check_unused);
 
 /* when selectively copying dive information, which parts should be copied? */
 struct dive_components {
@@ -191,6 +191,10 @@ struct dive_components {
 	unsigned int suit : 1;
 	unsigned int rating : 1;
 	unsigned int visibility : 1;
+	unsigned int wavesize : 1;
+	unsigned int current : 1;
+	unsigned int surge : 1;
+	unsigned int chill : 1;
 	unsigned int tags : 1;
 	unsigned int cylinders : 1;
 	unsigned int weights : 1;
@@ -201,32 +205,20 @@ extern struct event *get_next_divemodechange(const struct event **evd, bool upda
 extern enum divemode_t get_divemode_at_time(const struct divecomputer *dc, int dtime, const struct event **ev_dmc);
 
 /* picture list and methods related to dive picture handling */
-struct picture {
-	char *filename;
-	offset_t offset;
-	location_t location;
-	struct picture *next;
-};
-
 #define FOR_EACH_PICTURE(_dive) \
 	if (_dive)              \
 		for (struct picture *picture = (_dive)->picture_list; picture; picture = picture->next)
-
-extern struct picture *alloc_picture();
-extern void free_picture(struct picture *picture);
 extern void create_picture(const char *filename, int shift_time, bool match_all);
 extern void dive_add_picture(struct dive *d, struct picture *newpic);
 extern bool dive_remove_picture(struct dive *d, const char *filename);
-extern unsigned int dive_get_picture_count(struct dive *d);
 extern bool picture_check_valid_time(timestamp_t timestamp, int shift_time);
-extern void picture_free(struct picture *picture);
 
 extern bool has_gaschange_event(const struct dive *dive, const struct divecomputer *dc, int idx);
 extern int explicit_first_cylinder(const struct dive *dive, const struct divecomputer *dc);
 extern int get_depth_at_time(const struct divecomputer *dc, unsigned int time);
 
 extern fraction_t best_o2(depth_t depth, const struct dive *dive);
-extern fraction_t best_he(depth_t depth, const struct dive *dive);
+extern fraction_t best_he(depth_t depth, const struct dive *dive, bool o2narcotic, fraction_t fo2);
 
 extern int get_surface_pressure_in_mbar(const struct dive *dive, bool non_null);
 extern int calculate_depth_to_mbar(int depth, pressure_t surface_pressure, int salinity);
@@ -245,9 +237,11 @@ extern bool autogroup;
 struct dive *unregister_dive(int idx);
 extern void delete_single_dive(int idx);
 
-extern int run_survey, quit, force_root;
+extern int quit, force_root, ignore_bt;
+#ifdef SUBSURFACE_MOBILE_DESKTOP
+extern char *testqml;
+#endif
 
-extern struct dive_table dive_table;
 extern struct dive displayed_dive;
 extern unsigned int dc_number;
 extern struct dive *current_dive;
@@ -255,7 +249,7 @@ extern struct dive *current_dive;
 #define displayed_dc (get_dive_dc(&displayed_dive, dc_number))
 
 extern struct dive *get_dive(int nr);
-extern struct dive *get_dive_from_table(int nr, struct dive_table *dt);
+extern struct dive *get_dive_from_table(int nr, const struct dive_table *dt);
 extern struct dive_site *get_dive_site_for_dive(const struct dive *dive);
 extern const char *get_dive_country(const struct dive *dive);
 extern const char *get_dive_location(const struct dive *dive);
@@ -302,32 +296,23 @@ extern void set_filename(const char *filename);
 extern int save_dives(const char *filename);
 extern int save_dives_logic(const char *filename, bool select_only, bool anonymize);
 extern int save_dive(FILE *f, struct dive *dive, bool anonymize);
-extern int export_dives_xslt(const char *filename, const bool selected, const int units, const char *export_xslt, bool anonymize);
+extern int export_dives_xslt(const char *filename, bool selected, const int units, const char *export_xslt, bool anonymize);
 
-extern int save_dive_sites_logic(const char *filename, bool select_only, bool anonymize);
+extern int save_dive_sites_logic(const char *filename, const struct dive_site *sites[], int nr_sites, bool anonymize);
 
 struct membuffer;
 extern void save_one_dive_to_mb(struct membuffer *b, struct dive *dive, bool anonymize);
 
-struct user_info {
-	char *name;
-	char *email;
-};
-
-extern void subsurface_user_info(struct user_info *);
 extern void subsurface_console_init(void);
 extern void subsurface_console_exit(void);
 extern bool subsurface_user_is_root(void);
 
 extern timestamp_t get_times();
 
-extern timestamp_t utc_mktime(struct tm *tm);
-extern void utc_mkdate(timestamp_t, struct tm *tm);
-
 extern struct dive *alloc_dive(void);
 extern void free_dive(struct dive *);
+extern void free_dive_dcs(struct divecomputer *dc);
 extern void record_dive_to_table(struct dive *dive, struct dive_table *table);
-extern void record_dive(struct dive *dive);
 extern void clear_dive(struct dive *dive);
 extern void copy_dive(const struct dive *s, struct dive *d);
 extern void selective_copy_dive(const struct dive *s, struct dive *d, struct dive_components what, bool clear);
@@ -357,15 +342,22 @@ extern struct dive *merge_dives(const struct dive *a, const struct dive *b, int
 extern struct dive *try_to_merge(struct dive *a, struct dive *b, bool prefer_downloaded);
 extern struct event *clone_event(const struct event *src_ev);
 extern void copy_events(const struct divecomputer *s, struct divecomputer *d);
+extern void copy_events_until(const struct dive *sd, struct dive *dd, int time);
 extern void free_events(struct event *ev);
-extern void copy_cylinders(const struct dive *s, struct dive *d, bool used_only);
+extern void copy_cylinders(const struct cylinder_table *s, struct cylinder_table *d);
+extern void copy_used_cylinders(const struct dive *s, struct dive *d, bool used_only);
 extern void copy_samples(const struct divecomputer *s, struct divecomputer *d);
 extern bool is_cylinder_used(const struct dive *dive, int idx);
 extern bool is_cylinder_prot(const struct dive *dive, int idx);
-extern void fill_default_cylinder(struct dive *dive, int idx);
 extern void add_gas_switch_event(struct dive *dive, struct divecomputer *dc, int time, int idx);
+extern struct event *create_event(unsigned int time, int type, int flags, int value, const char *name);
+extern struct event *create_gas_switch_event(struct dive *dive, struct divecomputer *dc, int seconds, int idx);
+extern struct event *clone_event_rename(const struct event *ev, const char *name);
+extern void add_event_to_dc(struct divecomputer *dc, struct event *ev);
+extern void swap_event(struct divecomputer *dc, struct event *from, struct event *to);
+extern bool same_event(const struct event *a, const struct event *b);
 extern struct event *add_event(struct divecomputer *dc, unsigned int time, int type, int flags, int value, const char *name);
-extern void remove_event(struct event *event);
+extern void remove_event_from_dc(struct divecomputer *dc, struct event *event);
 extern void update_event_name(struct dive *d, struct event *event, const char *name);
 extern void add_extra_data(struct divecomputer *dc, const char *key, const char *value);
 extern void per_cylinder_mean_depth(const struct dive *dive, struct divecomputer *dc, int *mean, int *duration);
@@ -376,15 +368,9 @@ extern int nr_weightsystems(const struct dive *dive);
 
 /* UI related protopypes */
 
-// extern void report_error(GError* error);
-
 extern void remember_event(const char *eventname);
 extern void invalidate_dive_cache(struct dive *dc);
 
-#if WE_DONT_USE_THIS /* this is a missing feature in Qt - selecting which events to display */
-extern int evn_foreach(void (*callback)(const char *, bool *, void *), void *data);
-#endif /* WE_DONT_USE_THIS */
-
 extern void clear_events(void);
 
 extern void set_dc_nickname(struct dive *dive);
@@ -400,10 +386,6 @@ extern const char *existing_filename;
 extern void subsurface_command_line_init(int *, char ***);
 extern void subsurface_command_line_exit(int *, char ***);
 
-#define FRACTION(n, x) ((unsigned)(n) / (x)), ((unsigned)(n) % (x))
-
-#define DECOTIMESTEP 60 /* seconds. Unit of deco stop times */
-
 extern bool is_dc_planner(const struct divecomputer *dc);
 extern bool has_planned(const struct dive *dive, bool planned);
 
@@ -436,15 +418,11 @@ extern void update_setpoint_events(const struct dive *dive, struct divecomputer
 #ifdef __cplusplus
 }
 
-/* Make pointers to dive, dive_trip, dive_table, trip_table and dive_site_table
- * "Qt metatypes" so that they can be passed through QVariants and through QML.
- * Note: we have to use the typedef "dive_table_t" instead of "struct dive_table",
- * because MOC removes the "struct", but dive_table is already the name of a global
- * variable, leading to compilation errors. Likewise for "struct trip_table" and
- * "struct dive_site_table" (defined in "dive.h" and "divesite.h"). */
+/* Make pointers to dive and dive_trip "Qt metatypes" so that they can be passed through
+ * QVariants and through QML.
+ */
 #include <QObject>
 Q_DECLARE_METATYPE(struct dive *);
-Q_DECLARE_METATYPE(dive_table_t *);
 
 #endif
 
