@@ -1,24 +1,18 @@
 // SPDX-License-Identifier: GPL-2.0
 /* divelist.c */
-#include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
-#include <math.h>
-#include "gettext.h"
-#include <assert.h>
-#include <zip.h>
-#include <libxslt/transform.h>
 
+#include "divelist.h"
 #include "subsurface-string.h"
 #include "deco.h"
+#include "device.h"
 #include "divesite.h"
-#include "divelist.h"
-#include "display.h"
+#include "dive.h"
+#include "fulltext.h"
 #include "planner.h"
 #include "qthelper.h"
+#include "gettext.h"
 #include "git-access.h"
+#include "selection.h"
 #include "table.h"
 #include "trip.h"
 
@@ -28,23 +22,6 @@ static bool dive_list_changed = false;
 
 bool autogroup = false;
 
-unsigned int amount_selected;
-
-#if DEBUG_SELECTION_TRACKING
-void dump_selection(void)
-{
-	int i;
-	struct dive *dive;
-
-	printf("currently selected are %u dives:", amount_selected);
-	for_each_dive(i, dive) {
-		if (dive->selected)
-			printf(" %d", i);
-	}
-	printf("\n");
-}
-#endif
-
 void set_autogroup(bool value)
 {
 	/* if we keep the UI paradigm, this needs to toggle
@@ -64,16 +41,13 @@ void get_dive_gas(const struct dive *dive, int *o2_p, int *he_p, int *o2max_p)
 	int i;
 	int maxo2 = -1, maxhe = -1, mino2 = 1000;
 
-
-	for (i = 0; i < MAX_CYLINDERS; i++) {
-		const cylinder_t *cyl = dive->cylinder + i;
+	for (i = 0; i < dive->cylinders.nr; i++) {
+		const cylinder_t *cyl = get_cylinder(dive, i);
 		int o2 = get_o2(cyl->gasmix);
 		int he = get_he(cyl->gasmix);
 
 		if (!is_cylinder_used(dive, i))
 			continue;
-		if (cylinder_none(cyl))
-			continue;
 		if (o2 > maxo2)
 			maxo2 = o2;
 		if (he > maxhe)
@@ -161,7 +135,6 @@ static int calculate_otu(const struct dive *dive)
 	return lrint(otu);
 }
 
-
 /* Calculate the CNS for a single dive  - this only takes the first divecomputer into account.
    The CNS contributions are summed for dive segments defined by samples. The maximum O2 exposure duration for each
    segment is calculated based on the mean depth of the two samples (start & end) that define each segment. The CNS
@@ -349,9 +322,9 @@ static double calculate_airuse(const struct dive *dive)
 	int airuse = 0;
 	int i;
 
-	for (i = 0; i < MAX_CYLINDERS; i++) {
+	for (i = 0; i < dive->cylinders.nr; i++) {
 		pressure_t start, end;
-		const cylinder_t *cyl = dive->cylinder + i;
+		const cylinder_t *cyl = get_cylinder(dive, i);
 
 		start = cyl->start.mbar ? cyl->start : cyl->sample_start;
 		end = cyl->end.mbar ? cyl->end : cyl->sample_end;
@@ -650,30 +623,6 @@ char *get_dive_gas_string(const struct dive *dive)
 	return buffer;
 }
 
-struct dive *first_selected_dive()
-{
-	int idx;
-	struct dive *d;
-
-	for_each_dive (idx, d) {
-		if (d->selected)
-			return d;
-	}
-	return NULL;
-}
-
-struct dive *last_selected_dive()
-{
-	int idx;
-	struct dive *d, *ret = NULL;
-
-	for_each_dive (idx, d) {
-		if (d->selected)
-			ret = d;
-	}
-	return ret;
-}
-
 /* Like strcmp(), but don't crash on null-pointers */
 static int safe_strcmp(const char *s1, const char *s2)
 {
@@ -753,6 +702,7 @@ static MAKE_GET_IDX(dive_table, struct dive *, dives)
 MAKE_SORT(dive_table, struct dive *, dives, comp_dives)
 MAKE_REMOVE(dive_table, struct dive *, dive)
 MAKE_CLEAR_TABLE(dive_table, dives, dive)
+MAKE_MOVE_TABLE(dive_table, dives)
 
 void insert_dive(struct dive_table *table, struct dive *d)
 {
@@ -796,6 +746,13 @@ void delete_dive_from_table(struct dive_table *table, int idx)
 	remove_from_dive_table(table, idx);
 }
 
+struct dive *get_dive_from_table(int nr, const struct dive_table *dt)
+{
+	if (nr >= dt->nr || nr < 0)
+		return NULL;
+	return dt->dives[nr];
+}
+
 /* This removes a dive from the global dive table but doesn't free the
  * resources associated with the dive. The caller must removed the dive
  * from the trip-list. Returns a pointer to the unregistered dive.
@@ -805,6 +762,9 @@ struct dive *unregister_dive(int idx)
 	struct dive *dive = get_dive(idx);
 	if (!dive)
 		return NULL; /* this should never happen */
+	/* When removing a dive from the global dive table,
+	 * we also have to unregister its fulltext cache. */
+	fulltext_unregister(dive);
 	remove_from_dive_table(&dive_table, idx);
 	if (dive->selected)
 		amount_selected--;
@@ -826,87 +786,20 @@ void delete_single_dive(int idx)
 	delete_dive_from_table(&dive_table, idx);
 }
 
-/* add a dive at the end of the global dive table and keep track
- * of the number of selected dives. */
-void append_dive(struct dive *dive)
-{
-	add_to_dive_table(&dive_table, dive_table.nr, dive);
-	if (dive->selected)
-		amount_selected++;
-}
-
-bool consecutive_selected()
-{
-	struct dive *d;
-	int i;
-	bool consecutive = true;
-	bool firstfound = false;
-	bool lastfound = false;
-
-	if (amount_selected == 0 || amount_selected == 1)
-		return true;
-
-	for_each_dive(i, d) {
-		if (d->selected) {
-			if (!firstfound)
-				firstfound = true;
-			else if (lastfound)
-				consecutive = false;
-		} else if (firstfound) {
-			lastfound = true;
-		}
-	}
-	return consecutive;
-}
-
-void select_dive(struct dive *dive)
-{
-	if (!dive)
-		return;
-	if (!dive->selected) {
-		dive->selected = 1;
-		amount_selected++;
-	}
-	current_dive = dive;
-}
-
-void deselect_dive(struct dive *dive)
-{
-	int idx;
-	if (dive && dive->selected) {
-		dive->selected = 0;
-		if (amount_selected)
-			amount_selected--;
-		if (current_dive == dive && amount_selected > 0) {
-			/* pick a different dive as selected */
-			int selected_dive = idx = get_divenr(dive);
-			while (--selected_dive >= 0) {
-				dive = get_dive(selected_dive);
-				if (dive && dive->selected) {
-					current_dive = dive;
-					return;
-				}
-			}
-			selected_dive = idx;
-			while (++selected_dive < dive_table.nr) {
-				dive = get_dive(selected_dive);
-				if (dive && dive->selected) {
-					current_dive = dive;
-					return;
-				}
-			}
-		}
-		current_dive = NULL;
-	}
-}
-
-void filter_dive(struct dive *d, bool shown)
+int shown_dives = 0;
+bool filter_dive(struct dive *d, bool shown)
 {
+	bool old_shown, changed;
 	if (!d)
-		return;
+		return false;
+	old_shown = !d->hidden_by_filter;
 	d->hidden_by_filter = !shown;
 	if (!shown && d->selected)
 		deselect_dive(d);
+	changed = old_shown != shown;
+	if (changed)
+		shown_dives += shown - old_shown;
+	return changed;
 }
 
 void mark_divelist_changed(bool changed)
@@ -927,15 +820,21 @@ void process_loaded_dives()
 	int i;
 	struct dive *dive;
 
-	/* Register dive computer nick names */
-	for_each_dive(i, dive)
+	/* Register dive computer nick names and count shown dives. */
+	shown_dives = 0;
+	for_each_dive(i, dive) {
+		if (!dive->hidden_by_filter)
+			shown_dives++;
 		set_dc_nickname(dive);
+	}
 
 	sort_dive_table(&dive_table);
 	sort_trip_table(&trip_table);
 
 	/* Autogroup dives if desired by user. */
 	autogroup_dives(&dive_table, &trip_table);
+
+	fulltext_populate();
 }
 
 /*
@@ -1099,10 +998,10 @@ static bool merge_dive_tables(struct dive_table *dives_from, struct dive_table *
 void add_imported_dives(struct dive_table *import_table, struct trip_table *import_trip_table, struct dive_site_table *import_sites_table, int flags)
 {
 	int i, idx;
-	struct dive_table dives_to_add = { 0 };
-	struct dive_table dives_to_remove = { 0 };
-	struct trip_table trips_to_add = { 0 };
-	struct dive_site_table dive_sites_to_add = { 0 };
+	struct dive_table dives_to_add = empty_dive_table;
+	struct dive_table dives_to_remove = empty_dive_table;
+	struct trip_table trips_to_add = empty_trip_table;
+	struct dive_site_table dive_sites_to_add = empty_dive_site_table;
 
 	/* Process imported dives and generate lists of dives
 	 * to-be-added and to-be-removed */
@@ -1228,7 +1127,7 @@ void process_imported_dives(struct dive_table *import_table, struct trip_table *
 	/* If the caller didn't pass an import_trip_table because all
 	 * dives are tripless, provide a local table. This may be
 	 * necessary if the trips are autogrouped */
-	struct trip_table local_trip_table = { 0 };
+	struct trip_table local_trip_table = empty_trip_table;
 	if (!import_trip_table)
 		import_trip_table = &local_trip_table;
 
@@ -1372,6 +1271,16 @@ void process_imported_dives(struct dive_table *import_table, struct trip_table *
 	}
 }
 
+static struct dive *get_last_valid_dive()
+{
+	int i;
+	for (i = dive_table.nr - 1; i >= 0; i--) {
+		if (!dive_table.dives[i]->invalid)
+			return dive_table.dives[i];
+	}
+	return NULL;
+}
+
 /* return the number a dive gets when inserted at the given index.
  * this function is supposed to be called *before* a dive was added.
  * this returns:
@@ -1381,13 +1290,12 @@ void process_imported_dives(struct dive_table *import_table, struct trip_table *
  */
 int get_dive_nr_at_idx(int idx)
 {
-	if (dive_table.nr == 0)
+	if (idx < dive_table.nr)
+		return 0;
+	struct dive *last_dive = get_last_valid_dive();
+	if (!last_dive)
 		return 1;
-	if (idx >= dive_table.nr) {
-		struct dive *last_dive = get_dive(dive_table.nr - 1);
-		return last_dive->number ? last_dive->number + 1 : 0;
-	}
-	return 0;
+	return last_dive->number ? last_dive->number + 1 : 0;
 }
 
 void set_dive_nr_for_current_dive()
@@ -1445,8 +1353,12 @@ int get_dive_id_closest_to(timestamp_t when)
 
 void clear_dive_file_data()
 {
+	fulltext_unregister_all();
+
 	while (dive_table.nr)
 		delete_single_dive(0);
+	current_dive = NULL;
+	shown_dives = 0;
 	while (dive_site_table.nr)
 		delete_dive_site(get_dive_site(0, &dive_site_table), &dive_site_table);
 	if (trip_table.nr != 0) {
@@ -1455,6 +1367,8 @@ void clear_dive_file_data()
 	}
 
 	clear_dive(&displayed_dive);
+	clear_device_nodes();
+	clear_events();
 
 	reset_min_datafile_version();
 	clear_git_id();
@@ -1470,13 +1384,25 @@ static int comp_dive_to_trip(struct dive *a, struct dive_trip *b)
 {
 	/* This should never happen, nevertheless don't crash on trips
 	 * with no (or worse a negative number of) dives. */
-	if (b->dives.nr <= 0)
+	if (!b || b->dives.nr <= 0)
 		return -1;
 	return comp_dives(a, b->dives.dives[0]);
 }
 
 static int comp_dive_or_trip(struct dive_or_trip a, struct dive_or_trip b)
 {
+	/* we should only be called with both a and b having exactly one of
+	 * dive or trip not NULL. But in an abundance of caution, make sure
+	 * we still give a consistent answer even when called with invalid
+	 * arguments, as otherwise we might be hunting down crashes at a later
+	 * time...
+	 */
+	if (!a.dive && !a.trip && !b.dive && !b.trip)
+		return 0;
+	if (!a.dive && !a.trip)
+		return -1;
+	if (!b.dive && !b.trip)
+		return 1;
 	if (a.dive && b.dive)
 		return comp_dives(a.dive, b.dive);
 	if (a.trip && b.trip)
