@@ -76,11 +76,6 @@ void DownloadThread::run()
 		qDebug() << "No download possible when DC type is unknown";
 		return;
 	}
-#if defined(Q_OS_ANDROID)
-	// on Android we either use BT, a USB device, or we download via FTDI cable
-	if (!internalData->bluetooth_mode && (same_string(internalData->devname, "FTDI") || same_string(internalData->devname, "")))
-		internalData->devname = "ftdi";
-#endif
 	qDebug() << "Starting download from " << (internalData->bluetooth_mode ? "BT" : internalData->devname);
 	qDebug() << "downloading" << (internalData->force_download ? "all" : "only new") << "dives";
 	clear_dive_table(&downloadTable);
@@ -110,50 +105,6 @@ void DownloadThread::run()
 	updateRememberedDCs();
 }
 
-static void fill_supported_mobile_list()
-{
-#if defined(Q_OS_ANDROID)
-	/* USB or FTDI devices that are supported on Android - this does NOT include the BLE or BT only devices */
-	mobileProductList["Aeris"] =
-		QStringList({{"500 AI"}, {"A300"}, {"A300 AI"}, {"A300CS"}, {"Atmos 2"}, {"Atmos AI"}, {"Atmos AI 2"}, {"Compumask"}, {"Elite"}, {"Elite T3"}, {"Epic"}, {"F10"}, {"F11"}, {"Manta"}, {"XR-1 NX"}, {"XR-2"}});
-	mobileProductList["Aqualung"] =
-		QStringList({{"i200"}, {"i300"}, {"i450T"}, {"i550"}, {"i750TC"}});
-	mobileProductList["Beuchat"] =
-		QStringList({{"Mundial 2"}, {"Mundial 3"}, {"Voyager 2G"}});
-	mobileProductList["Cochran"] =
-		QStringList({{"Commander I"}, {"Commander II"}, {"Commander TM"}, {"EMC-14"}, {"EMC-16"}, {"EMC-20H"}});
-	mobileProductList["Cressi"] =
-		QStringList({{"Leonardo"}, {"Giotto"}, {"Newton"}, {"Drake"}, {"Cartesio"}, {"Goa"}});
-	mobileProductList["Genesis"] =
-		QStringList({{"React Pro"}, {"React Pro White"}});
-	mobileProductList["Heinrichs Weikamp"] =
-		QStringList({{"Frog"}, {"OSTC"}, {"OSTC 2"}, {"OSTC 2C"}, {"OSTC 2N"}, {"OSTC 3"}, {"OSTC 3+"}, {"OSTC 4"}, {"OSTC Mk2"}, {"OSTC Plus"}, {"OSTC Sport"}, {"OSTC cR"}, {"OSTC 2 TR"}});
-	mobileProductList["Hollis"] =
-		QStringList({{"DG02"}, {"DG03"}, {"TX1"}});
-	mobileProductList["Mares"] =
-		QStringList({{"Puck Pro"}, {"Smart"}, {"Quad"}});
-	mobileProductList["Oceanic"] =
-		QStringList({{"Atom 1.0"}, {"Atom 2.0"}, {"Atom 3.0"}, {"Atom 3.1"}, {"Datamask"}, {"F10"}, {"F11"}, {"Geo"}, {"Geo 2.0"}, {"OC1"}, {"OCS"}, {"OCi"}, {"Pro Plus 2"}, {"Pro Plus 2.1"}, {"Pro Plus 3"}, {"VT 4.1"}, {"VT Pro"}, {"VT3"}, {"VT4"}, {"VTX"}, {"Veo 1.0"}, {"Veo 180"}, {"Veo 2.0"}, {"Veo 200"}, {"Veo 250"}, {"Veo 3.0"}, {"Versa Pro"}});
-	mobileProductList["Scubapro"] =
-		QStringList({{"Aladin Square"}, {"G2"}});
-	mobileProductList["Seemann"] =
-		QStringList({{"XP5"}});
-	mobileProductList["Sherwood"] =
-		QStringList({{"Amphos"}, {"Amphos Air"}, {"Insight"}, {"Insight 2"}, {"Vision"}, {"Wisdom"}, {"Wisdom 2"}, {"Wisdom 3"}});
-	mobileProductList["Subgear"] =
-		QStringList({{"XP-Air"}});
-	mobileProductList["Suunto"] =
-		QStringList({{"Cobra"}, {"Cobra 2"}, {"Cobra 3"}, {"D3"}, {"D4"}, {"D4f"}, {"D4i"}, {"D6"}, {"D6i"}, {"D9"}, {"D9tx"}, {"DX"}, {"EON Core"}, {"EON Steel"}, {"Eon"}, {"Gekko"}, {"HelO2"}, {"Mosquito"}, {"Solution"}, {"Solution Alpha"}, {"Solution Nitrox"}, {"Spyder"}, {"Stinger"}, {"Vyper"}, {"Vyper 2"}, {"Vyper Air"}, {"Vyper Novo"}, {"Vytec"}, {"Zoop"}, {"Zoop Novo"}});
-	mobileProductList["Tusa"] =
-		QStringList({{"Element II (IQ-750)"}, {"Zen (IQ-900)"}, {"Zen Air (IQ-950)"}});
-	mobileProductList["Uwatec"] =
-		QStringList({{"Aladin Air Twin"}, {"Aladin Air Z"}, {"Aladin Air Z Nitrox"}, {"Aladin Air Z O2"}, {"Aladin Pro"}, {"Aladin Pro Ultra"}, {"Aladin Sport Plus"}, {"Memomouse"}});
-	mobileProductList["Atomic Aquatics"] =
-		QStringList({{"Cobalt"}, {"Cobalt 2"}});
-
-#endif
-}
-
 void fill_computer_list()
 {
 	dc_iterator_t *iterator = NULL;
@@ -161,8 +112,6 @@ void fill_computer_list()
 
 	unsigned int transportMask = get_supported_transports(NULL);
 
-	fill_supported_mobile_list();
-
 	dc_descriptor_iterator(&iterator);
 	while (dc_iterator_next(iterator, &descriptor) == DC_STATUS_SUCCESS) {
 		// mask out the transports that aren't supported
@@ -173,13 +122,6 @@ void fill_computer_list()
 
 		const char *vendor = dc_descriptor_get_vendor(descriptor);
 		const char *product = dc_descriptor_get_product(descriptor);
-#if defined(Q_OS_ANDROID)
-		if ((transports & ~(DC_TRANSPORT_SERIAL | DC_TRANSPORT_USB | DC_TRANSPORT_USBHID)) == 0)
-			// if the only available transports are serial/USB, then check against
-			// the ones that we explicitly support on Android
-			if (!mobileProductList.contains(vendor) || !mobileProductList[vendor].contains(product))
-				continue;
-#endif
 		if (!vendorList.contains(vendor))
 			vendorList.append(vendor);
 		if (!productList[vendor].contains(product))
@@ -278,6 +220,9 @@ DCDeviceData::DCDeviceData()
 #else
 	data.libdc_log = false;
 #endif
+#if defined(Q_OS_ANDROID)
+	data.androidUsbDeviceDescriptor = nullptr;
+#endif
 }
 
 DCDeviceData *DCDeviceData::instance()
@@ -302,16 +247,6 @@ DCDeviceData *DownloadThread::data()
 	return m_data;
 }
 
-struct dive_table *DownloadThread::table()
-{
-	return &downloadTable;
-}
-
-struct dive_site_table *DownloadThread::sites()
-{
-	return &diveSiteTable;
-}
-
 QString DCDeviceData::vendor() const
 {
 	return data.vendor;
@@ -388,6 +323,14 @@ void DCDeviceData::setDevName(const QString &devName)
 	data.devname = copy_qstring(devName);
 }
 
+#if defined(Q_OS_ANDROID)
+void DCDeviceData::setUsbDevice(const android_usb_serial_device_descriptor &usbDescriptor)
+{
+	androidUsbDescriptor = usbDescriptor;
+	data.androidUsbDeviceDescriptor = &androidUsbDescriptor;
+}
+#endif
+
 void DCDeviceData::setDevBluetoothName(const QString &name)
 {
 	m_devBluetoothName = name;
@@ -433,7 +376,6 @@ bool DCDeviceData::saveLog() const
 	return data.libdc_log;
 }
 
-
 device_data_t *DCDeviceData::internalData()
 {
 	return &data;
