@@ -19,9 +19,20 @@
 #include "subsurface-string.h"
 #include "table.h"
 
-static void free_weightsystem(weightsystem_t w)
+/* Warning: this has strange semantics for C-code! Not the weightsystem object
+ * is freed, but the data it references. The object itself is passed in by value.
+ * This is due to the fact how the table macros work.
+ */
+void free_weightsystem(weightsystem_t ws)
 {
-	free((void *)w.description);
+	free((void *)ws.description);
+	ws.description = NULL;
+}
+
+void free_cylinder(cylinder_t c)
+{
+	free((void *)c.type.description);
+	c.type.description = NULL;
 }
 
 void copy_weights(const struct weightsystem_table *s, struct weightsystem_table *d)
@@ -41,6 +52,16 @@ MAKE_REMOVE_FROM(weightsystem_table, weightsystems)
 //MAKE_REMOVE(weightsystem_table, weightsystem_t, weightsystem)
 MAKE_CLEAR_TABLE(weightsystem_table, weightsystems, weightsystem)
 
+/* cylinder table functions */
+//static MAKE_GET_IDX(cylinder_table, cylinder_t, cylinders)
+static MAKE_GROW_TABLE(cylinder_table, cylinder_t, cylinders)
+//static MAKE_GET_INSERTION_INDEX(cylinder_table, cylinder_t, cylinders, cylinder_less_than)
+static MAKE_ADD_TO(cylinder_table, cylinder_t, cylinders)
+MAKE_REMOVE_FROM(cylinder_table, cylinders)
+//MAKE_SORT(cylinder_table, cylinder_t, cylinders, comp_cylinders)
+//MAKE_REMOVE(cylinder_table, cylinder_t, cylinder)
+MAKE_CLEAR_TABLE(cylinder_table, cylinders, cylinder)
+
 const char *cylinderuse_text[NUM_GAS_USE] = {
 	QT_TRANSLATE_NOOP("gettextFromC", "OC-gas"), QT_TRANSLATE_NOOP("gettextFromC", "diluent"), QT_TRANSLATE_NOOP("gettextFromC", "oxygen"), QT_TRANSLATE_NOOP("gettextFromC", "not used")
 };
@@ -96,12 +117,50 @@ void add_weightsystem_description(const weightsystem_t *weightsystem)
 	}
 }
 
+weightsystem_t clone_weightsystem(weightsystem_t ws)
+{
+	weightsystem_t res = { ws.weight, copy_string(ws.description) };
+	return res;
+}
+
 /* Add a clone of a weightsystem to the end of a weightsystem table.
  * Cloned in means that the description-string is copied. */
 void add_cloned_weightsystem(struct weightsystem_table *t, weightsystem_t ws)
 {
-	weightsystem_t w_clone = { ws.weight, copy_string(ws.description) };
-	add_to_weightsystem_table(t, t->nr, w_clone);
+	add_to_weightsystem_table(t, t->nr, clone_weightsystem(ws));
+}
+
+/* Add a clone of a weightsystem to the end of a weightsystem table.
+ * Cloned in means that the description-string is copied. */
+void add_cloned_weightsystem_at(struct weightsystem_table *t, weightsystem_t ws)
+{
+	add_to_weightsystem_table(t, t->nr, clone_weightsystem(ws));
+}
+
+cylinder_t clone_cylinder(cylinder_t cyl)
+{
+	cylinder_t res = cyl;
+	res.type.description = copy_string(res.type.description);
+	return res;
+}
+
+void add_cylinder(struct cylinder_table *t, int idx, cylinder_t cyl)
+{
+	add_to_cylinder_table(t, idx, cyl);
+	/* FIXME: This is a horrible hack: we make sure that at the end of
+	 * every single cylinder table there is an empty cylinder that can
+	 * be used by the planner as "surface air" cylinder. Fix this.
+	 */
+	add_to_cylinder_table(t, t->nr, empty_cylinder);
+	t->nr--;
+	t->cylinders[t->nr].cylinder_use = NOT_USED;
+}
+
+/* Add a clone of a cylinder to the end of a cylinder table.
+ * Cloned in means that the description-string is copied. */
+void add_cloned_cylinder(struct cylinder_table *t, cylinder_t cyl)
+{
+	add_cylinder(t, t->nr, clone_cylinder(cyl));
 }
 
 bool same_weightsystem(weightsystem_t w1, weightsystem_t w2)
@@ -110,30 +169,6 @@ bool same_weightsystem(weightsystem_t w1, weightsystem_t w2)
 	       same_string(w1.description, w2.description);
 }
 
-bool cylinder_nodata(const cylinder_t *cyl)
-{
-	return !cyl->type.size.mliter &&
-	       !cyl->type.workingpressure.mbar &&
-	       !cyl->type.description &&
-	       !cyl->gasmix.o2.permille &&
-	       !cyl->gasmix.he.permille &&
-	       !cyl->start.mbar &&
-	       !cyl->end.mbar &&
-	       !cyl->gas_used.mliter &&
-	       !cyl->deco_gas_used.mliter;
-}
-
-static bool cylinder_nosamples(const cylinder_t *cyl)
-{
-	return !cyl->sample_start.mbar &&
-	       !cyl->sample_end.mbar;
-}
-
-bool cylinder_none(const cylinder_t *cyl)
-{
-	return cylinder_nodata(cyl) && cylinder_nosamples(cyl);
-}
-
 void get_gas_string(struct gasmix gasmix, char *text, int len)
 {
 	if (gasmix_is_air(gasmix))
@@ -161,18 +196,16 @@ int gas_volume(const cylinder_t *cyl, pressure_t p)
 	return lrint(cyl->type.size.mliter * bar_to_atm(bar) / z_factor);
 }
 
-int find_best_gasmix_match(struct gasmix mix, const cylinder_t array[])
+int find_best_gasmix_match(struct gasmix mix, const struct cylinder_table *cylinders)
 {
 	int i;
 	int best = -1, score = INT_MAX;
 
-	for (i = 0; i < MAX_CYLINDERS; i++) {
+	for (i = 0; i < cylinders->nr; i++) {
 		const cylinder_t *match;
 		int distance;
 
-		match = array + i;
-		if (cylinder_nodata(match))
-			continue;
+		match = cylinders->cylinders + i;
 		distance = gasmix_distance(mix, match->gasmix);
 		if (distance >= score)
 			continue;
@@ -258,10 +291,7 @@ struct ws_info_t ws_info[MAX_WS_INFO] = {
 
 void remove_cylinder(struct dive *dive, int idx)
 {
-	cylinder_t *cyl = dive->cylinder + idx;
-	int nr = MAX_CYLINDERS - idx - 1;
-	memmove(cyl, cyl + 1, nr * sizeof(*cyl));
-	memset(cyl + nr, 0, sizeof(*cyl));
+	remove_from_cylinder_table(&dive->cylinders, idx);
 }
 
 void remove_weightsystem(struct dive *dive, int idx)
@@ -269,16 +299,23 @@ void remove_weightsystem(struct dive *dive, int idx)
 	remove_from_weightsystem_table(&dive->weightsystems, idx);
 }
 
+// ws is cloned.
+void set_weightsystem(struct dive *dive, int idx, weightsystem_t ws)
+{
+	if (idx < 0 || idx >= dive->weightsystems.nr)
+		return;
+	free_weightsystem(dive->weightsystems.weightsystems[idx]);
+	dive->weightsystems.weightsystems[idx] = clone_weightsystem(ws);
+}
+
 /* when planning a dive we need to make sure that all cylinders have a sane depth assigned
  * and if we are tracking gas consumption the pressures need to be reset to start = end = workingpressure */
 void reset_cylinders(struct dive *dive, bool track_gas)
 {
 	pressure_t decopo2 = {.mbar = prefs.decopo2};
 
-	for (int i = 0; i < MAX_CYLINDERS; i++) {
-		cylinder_t *cyl = &dive->cylinder[i];
-		if (cylinder_none(cyl))
-			continue;
+	for (int i = 0; i < dive->cylinders.nr; i++) {
+		cylinder_t *cyl = get_cylinder(dive, i);
 		if (cyl->depth.mm == 0) /* if the gas doesn't give a mod, calculate based on prefs */
 			cyl->depth = gas_mod(cyl->gasmix, decopo2, dive, M_OR_FT(3,10));
 		if (track_gas)
@@ -288,12 +325,116 @@ void reset_cylinders(struct dive *dive, bool track_gas)
 	}
 }
 
+static void copy_cylinder_type(const cylinder_t *s, cylinder_t *d)
+{
+	free_cylinder(*d);
+	d->type = s->type;
+	d->type.description = s->type.description ? strdup(s->type.description) : NULL;
+	d->gasmix = s->gasmix;
+	d->depth = s->depth;
+	d->cylinder_use = s->cylinder_use;
+	d->manually_added = true;
+}
+
+/* copy the equipment data part of the cylinders but keep pressures */
+void copy_cylinder_types(const struct dive *s, struct dive *d)
+{
+	int i;
+	if (!s || !d)
+		return;
+
+	for (i = 0; i < s->cylinders.nr && i < d->cylinders.nr; i++)
+		copy_cylinder_type(get_cylinder(s, i), get_cylinder(d, i));
+
+	for ( ; i < s->cylinders.nr; i++)
+		add_cloned_cylinder(&d->cylinders, *get_cylinder(s, i));
+}
+
+cylinder_t *add_empty_cylinder(struct cylinder_table *t)
+{
+	cylinder_t cyl = empty_cylinder;
+	cyl.type.description = strdup("");
+	add_cylinder(t, t->nr, cyl);
+	return &t->cylinders[t->nr - 1];
+}
+
+/* access to cylinders is controlled by two functions:
+ * - get_cylinder() returns the cylinder of a dive and supposes that
+ *   the cylinder with the given index exists. If it doesn't, an error
+ *   message is printed and NULL returned.
+ * - get_or_create_cylinder() creates an empty cylinder if it doesn't exist.
+ *   Multiple cylinders might be created if the index is bigger than the
+ *   number of existing cylinders
+ */
+cylinder_t *get_cylinder(const struct dive *d, int idx)
+{
+	/* FIXME: The planner uses a dummy cylinder one past the official number of cylinders
+	 * in the table to mark no-cylinder surface interavals. This is horrendous. Fix ASAP. */
+	// if (idx < 0 || idx >= d->cylinders.nr) {
+	if (idx < 0 || idx >= d->cylinders.nr + 1 || idx >= d->cylinders.allocated) {
+		fprintf(stderr, "Warning: accessing invalid cylinder %d (%d existing)\n", idx, d->cylinders.nr);
+		return NULL;
+	}
+	return &d->cylinders.cylinders[idx];
+}
+
+cylinder_t *get_or_create_cylinder(struct dive *d, int idx)
+{
+	if (idx < 0) {
+		fprintf(stderr, "Warning: accessing invalid cylinder %d\n", idx);
+		return NULL;
+	}
+	while (idx >= d->cylinders.nr)
+		add_empty_cylinder(&d->cylinders);
+	return &d->cylinders.cylinders[idx];
+}
+
+/* if a default cylinder is set, use that */
+void fill_default_cylinder(const struct dive *dive, cylinder_t *cyl)
+{
+	const char *cyl_name = prefs.default_cylinder;
+	struct tank_info_t *ti = tank_info;
+	pressure_t pO2 = {.mbar = lrint(prefs.modpO2 * 1000.0)};
+
+	if (!cyl_name)
+		return;
+	while (ti->name != NULL && ti < tank_info + MAX_TANK_INFO) {
+		if (strcmp(ti->name, cyl_name) == 0)
+			break;
+		ti++;
+	}
+	if (ti->name == NULL)
+		/* didn't find it */
+		return;
+	cyl->type.description = strdup(ti->name);
+	if (ti->ml) {
+		cyl->type.size.mliter = ti->ml;
+		cyl->type.workingpressure.mbar = ti->bar * 1000;
+	} else {
+		cyl->type.workingpressure.mbar = psi_to_mbar(ti->psi);
+		if (ti->psi)
+			cyl->type.size.mliter = lrint(cuft_to_l(ti->cuft) * 1000 / bar_to_atm(psi_to_bar(ti->psi)));
+	}
+	// MOD of air
+	cyl->depth = gas_mod(cyl->gasmix, pO2, dive, 1);
+}
+
+cylinder_t create_new_cylinder(const struct dive *d)
+{
+	cylinder_t cyl = empty_cylinder;
+	fill_default_cylinder(d, &cyl);
+	cyl.start = cyl.type.workingpressure;
+	cyl.manually_added = true;
+	cyl.cylinder_use = OC_GAS;
+	return cyl;
+}
+
 #ifdef DEBUG_CYL
 void dump_cylinders(struct dive *dive, bool verbose)
 {
 	printf("Cylinder list:\n");
-	for (int i = 0; i < MAX_CYLINDERS; i++) {
-		cylinder_t *cyl = &dive->cylinder[i];
+	for (int i = 0; i < dive->cylinders; i++) {
+		cylinder_t *cyl = get_cylinder(dive, i);
 
 		printf("%02d: Type     %s, %3.1fl, %3.0fbar\n", i, cyl->type.description, cyl->type.size.mliter / 1000.0, cyl->type.workingpressure.mbar / 1000.0);
 		printf("    Gasmix   O2 %2.0f%% He %2.0f%%\n", cyl->gasmix.o2.permille / 10.0, cyl->gasmix.he.permille / 10.0);
