@@ -238,7 +238,7 @@ static void fill_missing_tank_pressures(struct dive *dive, struct plot_info *pi,
 	if (!track_pr)
 		return;
 
-	if (dive->cylinder[cyl].cylinder_use == OC_GAS)
+	if (get_cylinder(dive, cyl)->cylinder_use == OC_GAS)
 		strategy = SAC;
 	else
 		strategy = TIME;
@@ -263,13 +263,10 @@ static void fill_missing_tank_pressures(struct dive *dive, struct plot_info *pi,
 		double magic;
 		pr_track_t *segment;
 		int pressure;
-		int *save_pressure, *save_interpolated;
 
 		entry = pi->entry + i;
 
-		save_pressure = &(entry->pressure[cyl][SENSOR_PR]);
-		save_interpolated = &(entry->pressure[cyl][INTERPOLATED_PR]);
-		pressure = *save_pressure ? *save_pressure : *save_interpolated;
+		pressure = get_plot_pressure(pi, i, cyl);
 
 		if (pressure) {			// If there is a valid pressure value,
 			last_segment = NULL;	// get rid of interpolation data,
@@ -291,7 +288,8 @@ static void fill_missing_tank_pressures(struct dive *dive, struct plot_info *pi,
 			continue;
 
 		if (!segment->pressure_time) {		// Empty segment?
-			*save_pressure = cur_pr;	// Just use our current pressure
+			set_plot_pressure_data(pi, i, SENSOR_PR, cyl, cur_pr);
+							// Just use our current pressure
 			continue;			// and skip to next point.
 		}
 
@@ -304,7 +302,7 @@ static void fill_missing_tank_pressures(struct dive *dive, struct plot_info *pi,
 			last_segment = segment;
 		}
 
-		if(dive->cylinder[cyl].cylinder_use == OC_GAS) {
+		if(get_cylinder(dive, cyl)->cylinder_use == OC_GAS) {
 
 			/* if this segment has pressure_time, then calculate a new interpolated pressure */
 			if (interpolate.pressure_time) {
@@ -318,7 +316,7 @@ static void fill_missing_tank_pressures(struct dive *dive, struct plot_info *pi,
 			magic = (interpolate.end - interpolate.start) /  (segment->t_end - segment->t_start);
 			cur_pr = lrint(segment->start + magic * (entry->sec - segment->t_start));
 		}
-		*save_interpolated = cur_pr; // and store the interpolated data in plot_info
+		set_plot_pressure_data(pi, i, INTERPOLATED_PR, cyl, cur_pr); // and store the interpolated data in plot_info
 	}
 }
 
@@ -350,10 +348,8 @@ static inline int calc_pressure_time(struct dive *dive, struct plot_data *a, str
 static void debug_print_pressures(struct plot_info *pi)
 {
 	int i;
-	for (i = 0; i < pi->nr; i++) {
-		struct plot_data *entry = pi->entry + i;
-		printf("%5d |%9d | %9d |\n", i, get_plot_sensor_pressure(entry), get_plot_interpolated_pressure(entry));
-	}
+	for (i = 0; i < pi->nr; i++)
+		printf("%5d |%9d | %9d |\n", i, get_plot_sensor_pressure(pi, i), get_plot_interpolated_pressure(pi, i));
 }
 #endif
 
@@ -370,7 +366,7 @@ void populate_pressure_information(struct dive *dive, struct divecomputer *dc, s
 {
 	UNUSED(dc);
 	int first, last, cyl;
-	cylinder_t *cylinder = dive->cylinder + sensor;
+	cylinder_t *cylinder = get_cylinder(dive, sensor);
 	pr_track_t *track = NULL;
 	pr_track_t *current = NULL;
 	const struct event *ev, *b_ev;
@@ -378,6 +374,9 @@ void populate_pressure_information(struct dive *dive, struct divecomputer *dc, s
 	enum divemode_t dmode = dc->divemode;
 	const double gasfactor[5] = {1.0, 0.0, prefs.pscr_ratio/1000.0, 1.0, 1.0 };
 
+	if (sensor < 0 || sensor >= dive->cylinders.nr)
+		return;
+
 	/* if we have no pressure data whatsoever, this is pointless, so let's just return */
 	if (!cylinder->start.mbar && !cylinder->end.mbar &&
 	    !cylinder->sample_start.mbar && !cylinder->sample_end.mbar)
@@ -386,8 +385,7 @@ void populate_pressure_information(struct dive *dive, struct divecomputer *dc, s
 	/* Get a rough range of where we have any pressures at all */
 	first = last = -1;
 	for (int i = 0; i < pi->nr; i++) {
-		struct plot_data *entry = pi->entry + i;
-		int pressure = get_plot_sensor_pressure(entry, sensor);
+		int pressure = get_plot_sensor_pressure(pi, i, sensor);
 
 		if (!pressure)
 			continue;
@@ -416,7 +414,7 @@ void populate_pressure_information(struct dive *dive, struct divecomputer *dc, s
 
 	for (int i = first; i <= last; i++) {
 		struct plot_data *entry = pi->entry + i;
-		int pressure = get_plot_sensor_pressure(entry, sensor);
+		int pressure = get_plot_sensor_pressure(pi, i, sensor);
 		int time = entry->sec;
 
 		while (ev && ev->time.seconds <= time) {   // Find 1st gaschange event after 
@@ -445,7 +443,7 @@ void populate_pressure_information(struct dive *dive, struct divecomputer *dc, s
 		// until we get back to this cylinder.
 		if (cyl != sensor) {
 			current = NULL;
-			set_plot_pressure_data(entry, SENSOR_PR, sensor, 0);
+			set_plot_pressure_data(pi, i, SENSOR_PR, sensor, 0);
 			continue;
 		}
 
