@@ -8,6 +8,7 @@
 #include "divelist.h"
 #include "file.h"
 #include "parse.h"
+#include "subsurface-time.h"
 #include "divelist.h"
 #include "gettext.h"
 #include "import-csv.h"
@@ -351,16 +352,26 @@ int parse_csv_file(const char *filename, char **params, int pnr, const char *csv
 static int try_to_xslt_open_csv(const char *filename, struct memblock *mem, const char *tag)
 {
 	char *buf;
+	size_t i, amp = 0, rest = 0;
 
 	if (mem->size == 0 && readfile(filename, mem) < 0)
 		return report_error(translate("gettextFromC", "Failed to read '%s'"), filename);
 
+	/* Count ampersand characters */
+	for (i = 0; i < mem->size; ++i) {
+		if (((char *)mem->buffer)[i] == '&') {
+			++amp;
+		}
+	}
+
+
 	/* Surround the CSV file content with XML tags to enable XSLT
 	 * parsing
 	 *
 	 * Tag markers take: strlen("<></>") = 5
+	 * Reserve also room for encoding ampersands "&" => "&amp;"
 	 */
-	buf = realloc(mem->buffer, mem->size + 7 + strlen(tag) * 2);
+	buf = realloc(mem->buffer, mem->size + 7 + strlen(tag) * 2 + amp * 4);
 	if (buf != NULL) {
 		char *starttag = NULL;
 		char *endtag = NULL;
@@ -389,6 +400,16 @@ static int try_to_xslt_open_csv(const char *filename, struct memblock *mem, cons
 
 		free(starttag);
 		free(endtag);
+
+		/* Expand ampersands to encoded version */
+		for (i = mem->size, rest = 0; i > 0; --i, ++rest) {
+			if (((char *)mem->buffer)[i] == '&') {
+				memmove(((char *)mem->buffer) + i + 4 + 1, ((char *)mem->buffer) + i + 1, rest);
+				memcpy(((char *)mem->buffer) + i + 1, "amp;", 4);
+				rest += 4;
+				mem->size += 4;
+			}
+		}
 	} else {
 		free(mem->buffer);
 		return report_error("realloc failed in %s", __func__);
@@ -509,8 +530,8 @@ int parse_txt_file(const char *filename, const char *csv, struct dive_table *tab
 		int prev_depth = 0, cur_sampletime = 0, prev_setpoint = -1, prev_ndl = -1;
 		bool has_depth = false, has_setpoint = false, has_ndl = false;
 		char *lineptr, *key, *value;
-		int cur_cylinder_index = 0;
 		unsigned int prev_time = 0;
+		cylinder_t cyl;
 
 		struct dive *dive;
 		struct divecomputer *dc;
@@ -538,25 +559,26 @@ int parse_txt_file(const char *filename, const char *csv, struct dive_table *tab
 		dive->dc.divemode = CCR;
 		dive->dc.no_o2sensors = 2;
 
-		dive->cylinder[cur_cylinder_index].cylinder_use = OXYGEN;
-		dive->cylinder[cur_cylinder_index].type.size.mliter = 3000;
-		dive->cylinder[cur_cylinder_index].type.workingpressure.mbar = 200000;
-		dive->cylinder[cur_cylinder_index].type.description = strdup("3l Mk6");
-		dive->cylinder[cur_cylinder_index].gasmix.o2.permille = 1000;
-		cur_cylinder_index++;
+		cyl.cylinder_use = OXYGEN;
+		cyl.type.size.mliter = 3000;
+		cyl.type.workingpressure.mbar = 200000;
+		cyl.type.description = "3l Mk6";
+		cyl.gasmix.o2.permille = 1000;
+		cyl.manually_added = true;
+		add_cloned_cylinder(&dive->cylinders, cyl);
 
-		dive->cylinder[cur_cylinder_index].cylinder_use = DILUENT;
-		dive->cylinder[cur_cylinder_index].type.size.mliter = 3000;
-		dive->cylinder[cur_cylinder_index].type.workingpressure.mbar = 200000;
-		dive->cylinder[cur_cylinder_index].type.description = strdup("3l Mk6");
+		cyl.cylinder_use = DILUENT;
+		cyl.type.size.mliter = 3000;
+		cyl.type.workingpressure.mbar = 200000;
+		cyl.type.description = "3l Mk6";
 		value = parse_mkvi_value(memtxt.buffer, "Helium percentage");
 		he = atoi(value);
 		free(value);
 		value = parse_mkvi_value(memtxt.buffer, "Nitrogen percentage");
-		dive->cylinder[cur_cylinder_index].gasmix.o2.permille = (100 - atoi(value) - he) * 10;
+		cyl.gasmix.o2.permille = (100 - atoi(value) - he) * 10;
 		free(value);
-		dive->cylinder[cur_cylinder_index].gasmix.he.permille = he * 10;
-		cur_cylinder_index++;
+		cyl.gasmix.he.permille = he * 10;
+		add_cloned_cylinder(&dive->cylinders, cyl);
 
 		lineptr = strstr(memtxt.buffer, "Dive started at");
 		while (!empty_string(lineptr) && (lineptr = strchr(lineptr, '\n'))) {
@@ -719,7 +741,7 @@ int parse_txt_file(const char *filename, const char *csv, struct dive_table *tab
 						add_sample_data(sample, POSEIDON_NDL, value);
 						break;
 					case 39:
-						// Water Temperature in Celcius
+						// Water Temperature in Celsius
 						add_sample_data(sample, POSEIDON_TEMP, value);
 						break;
 					case 85:
