@@ -12,15 +12,16 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <time.h>
 #include "gettext.h"
 #include "divesite.h"
 #include "subsurface-string.h"
 #include "device.h"
-#include "divelist.h"
+#include "dive.h"
 #include "display.h"
 #include "errorhelper.h"
 #include "sha1.h"
+#include "subsurface-time.h"
+#include "timer.h"
 
 #include <libdivecomputer/version.h>
 #include <libdivecomputer/usbhid.h>
@@ -112,7 +113,9 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 	}
 	bool no_volume = true;
 
-	for (i = 0; i < MAX_CYLINDERS && (i < ngases || i < ntanks); i++) {
+	clear_cylinder_table(&dive->cylinders);
+	for (i = 0; i < ngases || i < ntanks; i++) {
+		cylinder_t cyl = empty_cylinder;
 		if (i < ngases) {
 			dc_gasmix_t gasmix = { 0 };
 			int o2, he;
@@ -139,27 +142,22 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 				}
 				he = 0;
 			}
-			dive->cylinder[i].gasmix.o2.permille = o2;
-			dive->cylinder[i].gasmix.he.permille = he;
-		} else {
-			dive->cylinder[i].gasmix.o2.permille = 0;
-			dive->cylinder[i].gasmix.he.permille = 0;
+			cyl.gasmix.o2.permille = o2;
+			cyl.gasmix.he.permille = he;
 		}
 
 		if (i < ntanks) {
 			dc_tank_t tank = { 0 };
 			rc = dc_parser_get_field(parser, DC_FIELD_TANK, i, &tank);
 			if (rc == DC_STATUS_SUCCESS) {
-				cylinder_t *cyl = dive->cylinder + i;
+				cyl.type.size.mliter = lrint(tank.volume * 1000);
+				cyl.type.workingpressure.mbar = lrint(tank.workpressure * 1000);
 
-				cyl->type.size.mliter = lrint(tank.volume * 1000);
-				cyl->type.workingpressure.mbar = lrint(tank.workpressure * 1000);
-
-				cyl->cylinder_use = OC_GAS;
+				cyl.cylinder_use = OC_GAS;
 				if (tank.type & DC_TANKINFO_CC_O2)
-					cyl->cylinder_use = OXYGEN;
+					cyl.cylinder_use = OXYGEN;
 				if (tank.type & DC_TANKINFO_CC_DILUENT)
-					cyl->cylinder_use = DILUENT;
+					cyl.cylinder_use = DILUENT;
 
 				if (tank.type & DC_TANKINFO_IMPERIAL) {
 					if (same_string(devdata->model, "Suunto EON Steel")) {
@@ -169,13 +167,13 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 						 * First, the pressures are off by a constant factor. WTF?
 						 * Then we can round the wet sizes so we get to multiples of 10
 						 * for cuft sizes (as that's all that you can enter) */
-						dive->cylinder[i].type.workingpressure.mbar = lrint(
-							dive->cylinder[i].type.workingpressure.mbar * 206.843 / 206.7 );
+						cyl.type.workingpressure.mbar = lrint(
+							cyl.type.workingpressure.mbar * 206.843 / 206.7 );
 						char name_buffer[17];
-						int rounded_size = lrint(ml_to_cuft(gas_volume(&dive->cylinder[i],
-							dive->cylinder[i].type.workingpressure)));
+						int rounded_size = lrint(ml_to_cuft(gas_volume(&cyl,
+							cyl.type.workingpressure)));
 						rounded_size = (int)((rounded_size + 5) / 10) * 10;
-						switch (dive->cylinder[i].type.workingpressure.mbar) {
+						switch (cyl.type.workingpressure.mbar) {
 						case 206843:
 							snprintf(name_buffer, sizeof(name_buffer), "AL%d", rounded_size);
 							break;
@@ -192,9 +190,9 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 							snprintf(name_buffer, sizeof(name_buffer), "%d cuft", rounded_size);
 							break;
 						}
-						dive->cylinder[i].type.description = copy_string(name_buffer);
-						dive->cylinder[i].type.size.mliter = lrint(cuft_to_l(rounded_size) * 1000 /
-											mbar_to_atm(dive->cylinder[i].type.workingpressure.mbar));
+						cyl.type.description = copy_string(name_buffer);
+						cyl.type.size.mliter = lrint(cuft_to_l(rounded_size) * 1000 /
+											mbar_to_atm(cyl.type.workingpressure.mbar));
 					}
 				}
 				if (tank.gasmix != i) { // we don't handle this, yet
@@ -213,22 +211,24 @@ static int parse_gasmixes(device_data_t *devdata, struct dive *dive, dc_parser_t
 			// rest of the code treats this as if they were valid values
 			if (!IS_FP_SAME(tank.beginpressure, 0.0)) {
 				if (!IS_FP_SAME(tank.endpressure, 0.0)) {
-					dive->cylinder[i].start.mbar = lrint(tank.beginpressure * 1000);
-					dive->cylinder[i].end.mbar = lrint(tank.endpressure * 1000);
+					cyl.start.mbar = lrint(tank.beginpressure * 1000);
+					cyl.end.mbar = lrint(tank.endpressure * 1000);
 				} else if (same_string(devdata->vendor, "Uwatec")) {
-					dive->cylinder[i].start.mbar = lrint(tank.beginpressure * 1000 + 30000);
-					dive->cylinder[i].end.mbar = 30000;
+					cyl.start.mbar = lrint(tank.beginpressure * 1000 + 30000);
+					cyl.end.mbar = 30000;
 				}
 			}
 		}
 		if (no_volume) {
 			/* for the first tank, if there is no tanksize available from the
 			 * dive computer, fill in the default tank information (if set) */
-			fill_default_cylinder(dive, i);
+			fill_default_cylinder(dive, &cyl);
 		}
 		/* whatever happens, make sure there is a name for the cylinder */
-		if (empty_string(dive->cylinder[i].type.description))
-			dive->cylinder[i].type.description = strdup(translate("gettextFromC", "unknown"));
+		if (empty_string(cyl.type.description))
+			cyl.type.description = strdup(translate("gettextFromC", "unknown"));
+
+		add_cylinder(&dive->cylinders, dive->cylinders.nr, cyl);
 	}
 	return DC_STATUS_SUCCESS;
 }
@@ -294,7 +294,8 @@ static void handle_event(struct divecomputer *dc, struct sample *sample, dc_samp
 
 static void handle_gasmix(struct divecomputer *dc, struct sample *sample, int idx)
 {
-	if (idx < 0 || idx >= MAX_CYLINDERS)
+	/* TODO: Verify that index is not higher than the number of cylinders */
+	if (idx < 0)
 		return;
 	add_event(dc, sample->time.seconds, SAMPLE_EVENT_GASCHANGE2, idx+1, 0, "gaschange");
 	current_gas_index = idx;
@@ -434,6 +435,17 @@ static void dev_info(device_data_t *devdata, const char *fmt, ...)
 
 static int import_dive_number = 0;
 
+static void download_error(const char *fmt, ...)
+{
+	static char buffer[1024];
+	va_list ap;
+
+	va_start(ap, fmt);
+	vsnprintf(buffer, sizeof(buffer), fmt, ap);
+	va_end(ap);
+	report_error("Dive %d: %s", import_dive_number, buffer);
+}
+
 static int parse_samples(device_data_t *devdata, struct divecomputer *dc, dc_parser_t *parser)
 {
 	UNUSED(devdata);
@@ -611,7 +623,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 
 	rc = dc_parser_get_datetime(parser, &dt);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error parsing the datetime"));
+		download_error(translate("gettextFromC", "Error parsing the datetime"));
 		return rc;
 	}
 
@@ -635,7 +647,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 	unsigned int divetime = 0;
 	rc = dc_parser_get_field(parser, DC_FIELD_DIVETIME, 0, &divetime);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error parsing the divetime"));
+		download_error(translate("gettextFromC", "Error parsing the divetime"));
 		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
@@ -645,7 +657,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 	double maxdepth = 0.0;
 	rc = dc_parser_get_field(parser, DC_FIELD_MAXDEPTH, 0, &maxdepth);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error parsing the maxdepth"));
+		download_error(translate("gettextFromC", "Error parsing the maxdepth"));
 		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
@@ -659,7 +671,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 	for (int i = 0; i < 3; i++) {
 		rc = dc_parser_get_field(parser, temp_fields[i], 0, &temperature);
 		if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-			dev_info(devdata, translate("gettextFromC", "Error parsing temperature"));
+			download_error(translate("gettextFromC", "Error parsing temperature"));
 			return rc;
 		}
 		if (rc == DC_STATUS_SUCCESS)
@@ -678,7 +690,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 	unsigned int ngases = 0;
 	rc = dc_parser_get_field(parser, DC_FIELD_GASMIX_COUNT, 0, &ngases);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error parsing the gas mix count"));
+		download_error(translate("gettextFromC", "Error parsing the gas mix count"));
 		return rc;
 	}
 
@@ -689,7 +701,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 	};
 	rc = dc_parser_get_field(parser, DC_FIELD_SALINITY, 0, &salinity);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error obtaining water salinity"));
+		download_error(translate("gettextFromC", "Error obtaining water salinity"));
 		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
@@ -698,7 +710,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 	double surface_pressure = 0;
 	rc = dc_parser_get_field(parser, DC_FIELD_ATMOSPHERIC, 0, &surface_pressure);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error obtaining surface pressure"));
+		download_error(translate("gettextFromC", "Error obtaining surface pressure"));
 		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
@@ -721,7 +733,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 	dc_divemode_t divemode;
 	rc = dc_parser_get_field(parser, DC_FIELD_DIVEMODE, 0, &divemode);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error obtaining dive mode"));
+		download_error(translate("gettextFromC", "Error obtaining dive mode"));
 		return rc;
 	}
 	if (rc == DC_STATUS_SUCCESS)
@@ -743,7 +755,7 @@ static dc_status_t libdc_header_parser(dc_parser_t *parser, device_data_t *devda
 
 	rc = parse_gasmixes(devdata, dive, parser, ngases);
 	if (rc != DC_STATUS_SUCCESS && rc != DC_STATUS_UNSUPPORTED) {
-		dev_info(devdata, translate("gettextFromC", "Error parsing the gas mix"));
+		download_error(translate("gettextFromC", "Error parsing the gas mix"));
 		return rc;
 	}
 
@@ -767,19 +779,20 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	in_deco = false;
 	current_gas_index = -1;
 
+	import_dive_number++;
+
 	rc = create_parser(devdata, &parser);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC", "Unable to create parser for %s %s"), devdata->vendor, devdata->product);
-		return false;
+		download_error(translate("gettextFromC", "Unable to create parser for %s %s"), devdata->vendor, devdata->product);
+		return true;
 	}
 
 	rc = dc_parser_set_data(parser, data, size);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC", "Error registering the data"));
+		download_error(translate("gettextFromC", "Error registering the data"));
 		goto error_exit;
 	}
 
-	import_dive_number++;
 	dive = alloc_dive();
 
 	// Fill in basic fields
@@ -799,27 +812,28 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 	// Parse the dive's header data
 	rc = libdc_header_parser (parser, devdata, dive);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("getextFromC", "Error parsing the header"));
+		download_error(translate("getextFromC", "Error parsing the header"));
 		goto error_exit;
 	}
 
 	// Initialize the sample data.
 	rc = parse_samples(devdata, &dive->dc, parser);
 	if (rc != DC_STATUS_SUCCESS) {
-		dev_info(devdata, translate("gettextFromC", "Error parsing the samples"));
+		download_error(translate("gettextFromC", "Error parsing the samples"));
 		goto error_exit;
 	}
 
+	dc_parser_destroy(parser);
+
 	/* If we already saw this dive, abort. */
 	if (!devdata->force_download && find_dive(&dive->dc)) {
 		char *date_string = get_dive_date_c_string(dive->when);
 		dev_info(devdata, translate("gettextFromC", "Already downloaded dive at %s"), date_string);
 		free(date_string);
-		goto error_exit;
+		free(dive);
+		return false;
 	}
 
-	dc_parser_destroy(parser);
-
 	/* Various libdivecomputer interface fixups */
 	if (dive->dc.airtemp.mkelvin == 0 && first_temp_is_air && dive->dc.samples) {
 		dive->dc.airtemp = dive->dc.sample[0].temperature;
@@ -832,7 +846,7 @@ static int dive_cb(const unsigned char *data, unsigned int size,
 error_exit:
 	dc_parser_destroy(parser);
 	free(dive);
-	return false;
+	return true;
 
 }
 
@@ -1175,17 +1189,29 @@ static const char *do_device_import(device_data_t *data)
 	return NULL;
 }
 
+static dc_timer_t *logfunc_timer = NULL;
 void logfunc(dc_context_t *context, dc_loglevel_t loglevel, const char *file, unsigned int line, const char *function, const char *msg, void *userdata)
 {
 	UNUSED(context);
 	const char *loglevels[] = { "NONE", "ERROR", "WARNING", "INFO", "DEBUG", "ALL" };
 
+	if (logfunc_timer == NULL)
+		dc_timer_new(&logfunc_timer);
+
 	FILE *fp = (FILE *)userdata;
 
+	dc_usecs_t now = 0;
+	dc_timer_now(logfunc_timer, &now);
+
+	unsigned long seconds = now / 1000000;
+	unsigned long microseconds = now % 1000000;
+
 	if (loglevel == DC_LOGLEVEL_ERROR || loglevel == DC_LOGLEVEL_WARNING) {
-		fprintf(fp, "%s: %s [in %s:%d (%s)]\n", loglevels[loglevel], msg, file, line, function);
+		fprintf(fp, "[%li.%06li] %s: %s [in %s:%d (%s)]\n",
+			seconds, microseconds,
+			loglevels[loglevel], msg, file, line, function);
 	} else {
-		fprintf(fp, "%s: %s\n", loglevels[loglevel], msg);
+		fprintf(fp, "[%li.%06li] %s: %s\n", seconds, microseconds, loglevels[loglevel], msg);
 	}
 }
 
@@ -1222,7 +1248,10 @@ unsigned int get_supported_transports(device_data_t *data)
 #if defined(BLE_SUPPORT)
 	supported |= DC_TRANSPORT_BLE;
 #endif
-
+#if defined(Q_OS_ANDROID)
+	// we cannot support transports that need libusb, hid, filesystem access, or IRDA on Android
+	supported &= ~(DC_TRANSPORT_USB | DC_TRANSPORT_USBHID | DC_TRANSPORT_IRDA | DC_TRANSPORT_USBSTORAGE);
+#endif
 	if (data) {
 		/*
 		 * If we have device data available, we can refine this:
@@ -1258,6 +1287,7 @@ dc_status_t divecomputer_device_open(device_data_t *data)
 
 #ifdef BT_SUPPORT
 	if (transports & DC_TRANSPORT_BLUETOOTH) {
+		dev_info(data, "Opening rfcomm stream %s", data->devname);
 		rc = rfcomm_stream_open(&data->iostream, context, data->devname);
 		if (rc == DC_STATUS_SUCCESS)
 			return rc;
@@ -1266,6 +1296,7 @@ dc_status_t divecomputer_device_open(device_data_t *data)
 
 #ifdef BLE_SUPPORT
 	if (transports & DC_TRANSPORT_BLE) {
+		dev_info(data, "Connecting to BLE device %s", data->devname);
 		rc = ble_packet_open(&data->iostream, context, data->devname, data);
 		if (rc == DC_STATUS_SUCCESS)
 			return rc;
@@ -1282,6 +1313,9 @@ dc_status_t divecomputer_device_open(device_data_t *data)
 		dc_iterator_free (iterator);
 
 		if (device) {
+			dev_info(data, "Opening USB HID device for %04x:%04x",
+				dc_usbhid_device_get_vid(device),
+				dc_usbhid_device_get_pid(device));
 			rc = dc_usbhid_open(&data->iostream, context, device);
 			dc_usbhid_device_free(device);
 			if (rc == DC_STATUS_SUCCESS)
@@ -1290,13 +1324,20 @@ dc_status_t divecomputer_device_open(device_data_t *data)
 	}
 
 	/* The dive computer backend does this all internally */
-	if (transports & DC_TRANSPORT_USB)
+	if (transports & DC_TRANSPORT_USB) {
+		dev_info(data, "Opening native USB device");
 		return DC_STATUS_SUCCESS;
+	}
 
 	if (transports & DC_TRANSPORT_SERIAL) {
+		dev_info(data, "Opening serial device %s", data->devname);
 #ifdef SERIAL_FTDI
 		if (!strcasecmp(data->devname, "ftdi"))
 			return ftdi_open(&data->iostream, context);
+#endif
+#ifdef __ANDROID__
+		if (data->androidUsbDeviceDescriptor)
+			return serial_usb_android_open(&data->iostream, context, data->androidUsbDeviceDescriptor);
 #endif
 		rc = dc_serial_open(&data->iostream, context, data->devname);
 		if (rc == DC_STATUS_SUCCESS)
@@ -1320,12 +1361,14 @@ dc_status_t divecomputer_device_open(device_data_t *data)
 		if (!address)
 			address = strtoul(data->devname, NULL, 0);
 
+		dev_info(data, "Opening IRDA address %u", address);
 		rc = dc_irda_open(&data->iostream, context, address, 1);
 		if (rc == DC_STATUS_SUCCESS)
 			return rc;
 	}
 
 	if (transports & DC_TRANSPORT_USBSTORAGE) {
+		dev_info(data, "Opening USB storage at %s", data->devname);
 		rc = dc_usb_storage_open(&data->iostream, context, data->devname);
 		if (rc == DC_STATUS_SUCCESS)
 			return rc;
@@ -1371,6 +1414,7 @@ const char *do_libdivecomputer_import(device_data_t *data)
 	if (rc != DC_STATUS_SUCCESS) {
 		report_error(errmsg(rc));
 	} else {
+		dev_info(data, "Connecting ...");
 		rc = dc_device_open(&data->device, data->context, data->descriptor, data->iostream);
 		INFO(0, "dc_device_open error value of %d", rc);
 		if (rc != DC_STATUS_SUCCESS && subsurface_access(data->devname, R_OK | W_OK) != 0)
@@ -1380,6 +1424,7 @@ const char *do_libdivecomputer_import(device_data_t *data)
 			err = translate("gettextFromC", "Error opening the device %s %s (%s).\nIn most cases, in order to debug this issue, a libdivecomputer logfile will be useful.\nYou can create this logfile by selecting the corresponding checkbox in the download dialog.");
 #endif
 		if (rc == DC_STATUS_SUCCESS) {
+			dev_info(data, "Starting import ...");
 			err = do_device_import(data);
 			/* TODO: Show the logfile to the user on error. */
 			dc_device_close(data->device);
