@@ -5,7 +5,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -15,6 +14,7 @@
 
 #include "gettext.h"
 
+#include "dive.h"
 #include "divesite.h"
 #include "errorhelper.h"
 #include "trip.h"
@@ -22,8 +22,10 @@
 #include "device.h"
 #include "membuffer.h"
 #include "git-access.h"
+#include "picture.h"
 #include "qthelper.h"
 #include "tag.h"
+#include "subsurface-time.h"
 
 const char *saved_git_id = NULL;
 
@@ -34,7 +36,9 @@ struct git_parser_state {
 	dive_trip_t *active_trip;
 	struct picture *active_pic;
 	struct dive_site *active_site;
-	int cylinder_index, weightsystem_index;
+	struct dive_table *table;
+	struct trip_table *trips;
+	struct dive_site_table *sites;
 	int o2pressure_sensor;
 };
 
@@ -46,21 +50,6 @@ struct keyword_action {
 
 static git_blob *git_tree_entry_blob(git_repository *repo, const git_tree_entry *entry);
 
-static char *get_utf8(struct membuffer *b)
-{
-	int len = b->len;
-	char *res;
-
-	if (!len)
-		return NULL;
-	res = malloc(len + 1);
-	if (res) {
-		memcpy(res, b->buffer, len);
-		res[len] = 0;
-	}
-	return res;
-}
-
 static temperature_t get_temperature(const char *line)
 {
 	temperature_t t;
@@ -175,7 +164,7 @@ static void parse_dive_gps(char *line, struct membuffer *str, struct git_parser_
 
 	parse_location(line, &location);
 	if (!ds) {
-		ds = get_dive_site_by_gps(&location, &dive_site_table);
+		ds = get_dive_site_by_gps(&location, state->sites);
 		if (!ds)
 			ds = create_dive_site_with_gps("", &location, &dive_site_table);
 		add_dive_to_dive_site(state->active_dive, ds);
@@ -194,7 +183,7 @@ static void parse_dive_gps(char *line, struct membuffer *str, struct git_parser_
 static void parse_dive_location(char *line, struct membuffer *str, struct git_parser_state *state)
 {
 	UNUSED(line);
-	char *name = get_utf8(str);
+	char *name = detach_cstring(str);
 	struct dive_site *ds = get_dive_site_for_dive(state->active_dive);
 	if (!ds) {
 		ds = get_dive_site_by_name(name, &dive_site_table);
@@ -215,16 +204,16 @@ static void parse_dive_location(char *line, struct membuffer *str, struct git_pa
 }
 
 static void parse_dive_divemaster(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_dive->divemaster = get_utf8(str); }
+{ UNUSED(line); state->active_dive->divemaster = detach_cstring(str); }
 
 static void parse_dive_buddy(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_dive->buddy = get_utf8(str); }
+{ UNUSED(line); state->active_dive->buddy = detach_cstring(str); }
 
 static void parse_dive_suit(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_dive->suit = get_utf8(str); }
+{ UNUSED(line); state->active_dive->suit = detach_cstring(str); }
 
 static void parse_dive_notes(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_dive->notes = get_utf8(str); }
+{ UNUSED(line); state->active_dive->notes = detach_cstring(str); }
 
 static void parse_dive_divesiteid(char *line, struct membuffer *str, struct git_parser_state *state)
 { UNUSED(str); add_dive_to_dive_site(state->active_dive, get_dive_site_by_uuid(get_hex(line), &dive_site_table)); }
@@ -274,6 +263,21 @@ static void parse_dive_rating(char *line, struct membuffer *str, struct git_pars
 static void parse_dive_visibility(char *line, struct membuffer *str, struct git_parser_state *state)
 { UNUSED(str); state->active_dive->visibility = get_index(line); }
 
+static void parse_dive_wavesize(char *line, struct membuffer *str, struct git_parser_state *state)
+{ UNUSED(str); state->active_dive->wavesize = get_index(line); }
+
+static void parse_dive_current(char *line, struct membuffer *str, struct git_parser_state *state)
+{ UNUSED(str); state->active_dive->current = get_index(line); }
+
+static void parse_dive_surge(char *line, struct membuffer *str, struct git_parser_state *state)
+{ UNUSED(str); state->active_dive->surge = get_index(line); }
+
+static void parse_dive_chill(char *line, struct membuffer *str, struct git_parser_state *state)
+{ UNUSED(str); state->active_dive->chill = get_index(line); }
+
+static void parse_dive_watersalinity(char *line, struct membuffer *str, struct git_parser_state *state)
+{ UNUSED(str); state->active_dive->user_salinity = get_salinity(line); }
+
 static void parse_dive_notrip(char *line, struct membuffer *str, struct git_parser_state *state)
 {
 	UNUSED(str);
@@ -281,14 +285,21 @@ static void parse_dive_notrip(char *line, struct membuffer *str, struct git_pars
 	state->active_dive->notrip = true;
 }
 
+static void parse_dive_invalid(char *line, struct membuffer *str, struct git_parser_state *state)
+{
+	UNUSED(str);
+	UNUSED(line);
+	state->active_dive->invalid = true;
+}
+
 static void parse_site_description(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_site->description = strdup(mb_cstring(str)); }
+{ UNUSED(line); state->active_site->description = detach_cstring(str); }
 
 static void parse_site_name(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_site->name = strdup(mb_cstring(str)); }
+{ UNUSED(line); state->active_site->name = detach_cstring(str); }
 
 static void parse_site_notes(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_site->notes = strdup(mb_cstring(str)); }
+{ UNUSED(line); state->active_site->notes = detach_cstring(str); }
 
 static void parse_site_gps(char *line, struct membuffer *str, struct git_parser_state *state)
 {
@@ -303,14 +314,50 @@ static void parse_site_geo(char *line, struct membuffer *str, struct git_parser_
 	int nr = state->active_site->taxonomy.nr;
 	if (nr < TC_NR_CATEGORIES) {
 		struct taxonomy *t = &state->active_site->taxonomy.category[nr];
-		t->value = strdup(mb_cstring(str));
+		t->value = detach_cstring(str);
 		sscanf(line, "cat %d origin %d \"", &t->category, (int *)&t->origin);
 		state->active_site->taxonomy.nr++;
 	}
 }
 
+static char *remove_from_front(struct membuffer *str, int len)
+{
+	char *prefix;
+
+	if (len >= str->len)
+		return detach_cstring(str);
+
+	/* memdup() - oh well */
+	prefix = malloc(len);
+	if (!prefix) {
+		report_error("git-load: out of memory");
+		return NULL;
+	}
+	memcpy(prefix, str->buffer, len);
+
+	str->len -= len;
+	memmove(str->buffer, str->buffer+len, str->len);
+	return prefix;
+}
+
+static char *pop_cstring(struct membuffer *str, const char *err)
+{
+	int len;
+
+	if (!str) {
+		report_error("git-load: string marker without any strings ('%s')", err);
+		return strdup("");
+	}
+	if (!str->len) {
+		report_error("git-load: string marker after running out of strings ('%s')", err);
+		return strdup("");
+	}
+	len = strlen(mb_cstring(str)) + 1;
+	return remove_from_front(str, len);
+}
+
 /* Parse key=val parts of samples and cylinders etc */
-static char *parse_keyvalue_entry(void (*fn)(void *, const char *, const char *), void *fndata, char *line)
+static char *parse_keyvalue_entry(void (*fn)(void *, const char *, const char *), void *fndata, char *line, struct membuffer *str)
 {
 	char *key = line, *val, c;
 
@@ -324,6 +371,10 @@ static char *parse_keyvalue_entry(void (*fn)(void *, const char *, const char *)
 		*line++ = 0;
 	val = line;
 
+	/* Did we get a string? Take it from the list of strings */
+	if (*val == '"')
+		val = pop_cstring(str, key);
+
 	while ((c = *line) != 0) {
 		if (isspace(c))
 			break;
@@ -347,9 +398,10 @@ static void parse_cylinder_keyvalue(void *_cylinder, const char *key, const char
 		cylinder->type.workingpressure = get_pressure(value);
 		return;
 	}
-	/* This is handled by the "get_utf8()" */
-	if (!strcmp(key, "description"))
+	if (!strcmp(key, "description")) {
+		cylinder->type.description = value;
 		return;
+	}
 	if (!strcmp(key, "o2")) {
 		cylinder->gasmix.o2 = get_fraction(value);
 		return;
@@ -391,23 +443,20 @@ static void parse_cylinder_keyvalue(void *_cylinder, const char *key, const char
 
 static void parse_dive_cylinder(char *line, struct membuffer *str, struct git_parser_state *state)
 {
-	cylinder_t *cylinder = state->active_dive->cylinder + state->cylinder_index;
+	cylinder_t cylinder = empty_cylinder;
 
-	if (state->cylinder_index >= MAX_CYLINDERS)
-		return;
-
-	cylinder->type.description = get_utf8(str);
 	for (;;) {
 		char c;
 		while (isspace(c = *line))
 			line++;
 		if (!c)
 			break;
-		line = parse_keyvalue_entry(parse_cylinder_keyvalue, cylinder, line);
+		line = parse_keyvalue_entry(parse_cylinder_keyvalue, &cylinder, line, str);
 	}
-	if (cylinder->cylinder_use == OXYGEN)
-		state->o2pressure_sensor = state->cylinder_index;
-	state->cylinder_index++;
+	if (cylinder.cylinder_use == OXYGEN)
+		state->o2pressure_sensor = state->active_dive->cylinders.nr;
+
+	add_cylinder(&state->active_dive->cylinders, state->active_dive->cylinders.nr, cylinder);
 }
 
 static void parse_weightsystem_keyvalue(void *_ws, const char *key, const char *value)
@@ -417,24 +466,24 @@ static void parse_weightsystem_keyvalue(void *_ws, const char *key, const char *
 		ws->weight = get_weight(value);
 		return;
 	}
-	/* This is handled by the "get_utf8()" */
-	if (!strcmp(key, "description"))
+	if (!strcmp(key, "description")) {
+		ws->description = value;
 		return;
+	}
 	report_error("Unknown weightsystem key/value pair (%s/%s)", key, value);
 }
 
 static void parse_dive_weightsystem(char *line, struct membuffer *str, struct git_parser_state *state)
 {
-	weightsystem_t ws = { 0 };
+	weightsystem_t ws = empty_weightsystem;
 
-	ws.description = get_utf8(str);
 	for (;;) {
 		char c;
 		while (isspace(c = *line))
 			line++;
 		if (!c)
 			break;
-		line = parse_keyvalue_entry(parse_weightsystem_keyvalue, &ws, line);
+		line = parse_keyvalue_entry(parse_weightsystem_keyvalue, &ws, line, str);
 	}
 
 	add_to_weightsystem_table(&state->active_dive->weightsystems, state->active_dive->weightsystems.nr, ws);
@@ -641,7 +690,7 @@ static void sample_parser(char *line, struct git_parser_state *state)
 			break;
 		/* Less common sample entries have a name */
 		if (c >= 'a' && c <= 'z') {
-			line = parse_keyvalue_entry(parse_sample_keyvalue, sample, line);
+			line = parse_keyvalue_entry(parse_sample_keyvalue, sample, line, NULL);
 		} else {
 			const char *end;
 			double val = ascii_strtod(line, &end);
@@ -684,7 +733,7 @@ static void parse_dc_meandepth(char *line, struct membuffer *str, struct git_par
 { UNUSED(str); state->active_dc->meandepth = get_depth(line); }
 
 static void parse_dc_model(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_dc->model = get_utf8(str); }
+{ UNUSED(line); state->active_dc->model = detach_cstring(str); }
 
 static void parse_dc_numberofoxygensensors(char *line, struct membuffer *str, struct git_parser_state *state)
 { UNUSED(str); state->active_dc->no_o2sensors = get_index(line); }
@@ -713,28 +762,43 @@ static int get_divemode(const char *divemodestring) {
 	return 0;
 }
 
-static void parse_event_keyvalue(void *_event, const char *key, const char *value)
+/*
+ * A 'struct event' has a variable-sized name allocation at the
+ * end. So when we parse the event data, we can't fill in the
+ * event directly, because we don't know how to allocate one
+ * before we have the size of the name.
+ *
+ * Thus this initial 'parse_event' with a separate name pointer.
+ */
+struct parse_event {
+	const char *name;
+	int has_divemode;
+	struct event ev;
+};
+
+static void parse_event_keyvalue(void *_parse, const char *key, const char *value)
 {
-	struct event *event = _event;
+	struct parse_event *parse = _parse;
 	int val = atoi(value);
 
 	if (!strcmp(key, "type")) {
-		event->type = val;
+		parse->ev.type = val;
 	} else if (!strcmp(key, "flags")) {
-		event->flags = val;
+		parse->ev.flags = val;
 	} else if (!strcmp(key, "value")) {
-		event->value = val;
+		parse->ev.value = val;
 	} else if (!strcmp(key, "name")) {
-		/* We get the name from the string handling */
+		parse->name = value;
 	} else if (!strcmp(key,"divemode")) {
-		event->value = get_divemode(value);
+		parse->ev.value = get_divemode(value);
+		parse->has_divemode = 1;
 	} else if (!strcmp(key, "cylinder")) {
 		/* NOTE! We add one here as a marker that "yes, we got a cylinder index" */
-		event->gas.index = 1 + get_index(value);
+		parse->ev.gas.index = 1 + get_index(value);
 	} else if (!strcmp(key, "o2")) {
-		event->gas.mix.o2 = get_fraction(value);
+		parse->ev.gas.mix.o2 = get_fraction(value);
 	} else if (!strcmp(key, "he")) {
-		event->gas.mix.he = get_fraction(value);
+		parse->ev.gas.mix.he = get_fraction(value);
 	} else
 		report_error("Unexpected event key/value pair (%s/%s)", key, value);
 }
@@ -767,13 +831,13 @@ static void parse_dc_keyvalue(char *line, struct membuffer *str, struct git_pars
 static void parse_dc_event(char *line, struct membuffer *str, struct git_parser_state *state)
 {
 	int m, s = 0;
-	const char *name;
-	struct event event = { 0 }, *ev;
+	struct parse_event p = { NULL, };
+	struct event *ev;
 
 	m = strtol(line, &line, 10);
 	if (*line == ':')
 		s = strtol(line + 1, &line, 10);
-	event.time.seconds = m * 60 + s;
+	p.ev.time.seconds = m * 60 + s;
 
 	for (;;) {
 		char c;
@@ -781,20 +845,21 @@ static void parse_dc_event(char *line, struct membuffer *str, struct git_parser_
 			line++;
 		if (!c)
 			break;
-		line = parse_keyvalue_entry(parse_event_keyvalue, &event, line);
+		line = parse_keyvalue_entry(parse_event_keyvalue, &p, line, str);
 	}
 
-	name = "";
-	if (str->len)
-		name = mb_cstring(str);
-	ev = add_event(state->active_dc, event.time.seconds, event.type, event.flags, event.value, name);
+	/* Only modechange events should have a divemode - fix up any corrupted names */
+	if (p.has_divemode && strcmp(p.name, "modechange"))
+		p.name = "modechange";
+
+	ev = add_event(state->active_dc, p.ev.time.seconds, p.ev.type, p.ev.flags, p.ev.value, p.name);
 
 	/*
 	 * Older logs might mark the dive to be CCR by having an "SP change" event at time 0:00.
 	 * Better to mark them being CCR on import so no need for special treatments elsewhere on
 	 * the code.
 	 */
-	if (ev && event.time.seconds == 0 && event.type == SAMPLE_EVENT_PO2 && event.value && state->active_dc->divemode==OC)
+	if (ev && p.ev.time.seconds == 0 && p.ev.type == SAMPLE_EVENT_PO2 && p.ev.value && state->active_dc->divemode==OC)
 		state->active_dc->divemode = CCR;
 
 	if (ev && event_is_gaschange(ev)) {
@@ -803,9 +868,9 @@ static void parse_dc_event(char *line, struct membuffer *str, struct git_parser_
 		 * and the parsing will add one for actual cylinder
 		 * index data (see parse_event_keyvalue)
 		 */
-		ev->gas.index = event.gas.index-1;
-		if (event.gas.mix.o2.permille || event.gas.mix.he.permille)
-			ev->gas.mix = event.gas.mix;
+		ev->gas.index = p.ev.gas.index-1;
+		if (p.ev.gas.mix.o2.permille || p.ev.gas.mix.he.permille)
+			ev->gas.mix = p.ev.gas.mix;
 	}
 }
 
@@ -818,10 +883,10 @@ static void parse_trip_time(char *line, struct membuffer *str, struct git_parser
 { UNUSED(line); UNUSED(str); UNUSED(state); }
 
 static void parse_trip_location(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_trip->location = get_utf8(str); }
+{ UNUSED(line); state->active_trip->location = detach_cstring(str); }
 
 static void parse_trip_notes(char *line, struct membuffer *str, struct git_parser_state *state)
-{ UNUSED(line); state->active_trip->notes = get_utf8(str); }
+{ UNUSED(line); state->active_trip->notes = detach_cstring(str); }
 
 static void parse_settings_autogroup(char *line, struct membuffer *str, struct git_parser_state *_unused)
 {
@@ -887,7 +952,6 @@ struct divecomputerid {
 	const char *nickname;
 	const char *firmware;
 	const char *serial;
-	const char *cstr;
 	unsigned int deviceid;
 };
 
@@ -895,10 +959,6 @@ static void parse_divecomputerid_keyvalue(void *_cid, const char *key, const cha
 {
 	struct divecomputerid *cid = _cid;
 
-	if (*value == '"') {
-		value = cid->cstr;
-		cid->cstr += strlen(cid->cstr) + 1;
-	}
 	if (!strcmp(key, "deviceid")) {
 		cid->deviceid = get_hex(value);
 		return;
@@ -922,26 +982,23 @@ static void parse_divecomputerid_keyvalue(void *_cid, const char *key, const cha
  * The 'divecomputerid' is a bit harder to parse than some other things, because
  * it can have multiple strings (but see the tag parsing for another example of
  * that) in addition to the non-string entries.
- *
- * We keep the "next" string in "id.cstr" and update it as we use it.
  */
 static void parse_settings_divecomputerid(char *line, struct membuffer *str, struct git_parser_state *_unused)
 {
 	UNUSED(_unused);
-	struct divecomputerid id = { mb_cstring(str) };
-
-	id.cstr = id.model + strlen(id.model) + 1;
+	struct divecomputerid id = { pop_cstring(str, line) };
 
 	/* Skip the '"' that stood for the model string */
 	line++;
 
+	/* Parse the rest of the entries */
 	for (;;) {
 		char c;
 		while (isspace(c = *line))
 			line++;
 		if (!c)
 			break;
-		line = parse_keyvalue_entry(parse_divecomputerid_keyvalue, &id, line);
+		line = parse_keyvalue_entry(parse_divecomputerid_keyvalue, &id, line, str);
 	}
 	create_device_node(id.model, id.deviceid, id.serial, id.firmware, id.nickname);
 }
@@ -949,7 +1006,7 @@ static void parse_settings_divecomputerid(char *line, struct membuffer *str, str
 static void parse_picture_filename(char *line, struct membuffer *str, struct git_parser_state *state)
 {
 	UNUSED(line);
-	state->active_pic->filename = get_utf8(str);
+	state->active_pic->filename = detach_cstring(str);
 }
 
 static void parse_picture_gps(char *line, struct membuffer *str, struct git_parser_state *state)
@@ -990,9 +1047,9 @@ static void divecomputer_parser(char *line, struct membuffer *str, struct git_pa
 struct keyword_action dive_action[] = {
 #undef D
 #define D(x) { #x, parse_dive_ ## x }
-	D(airpressure), D(airtemp), D(buddy), D(cylinder), D(divemaster), D(divesiteid), D(duration),
-	D(gps), D(location), D(notes), D(notrip), D(rating), D(suit),
-	D(tags), D(visibility), D(watertemp), D(weightsystem)
+	D(airpressure), D(airtemp), D(buddy), D(chill), D(current), D(cylinder), D(divemaster), D(divesiteid), D(duration),
+	D(gps), D(invalid), D(location), D(notes), D(notrip), D(rating), D(suit), D(surge),
+	D(tags), D(visibility), D(watersalinity), D(watertemp), D(wavesize), D(weightsystem)
 };
 
 static void dive_parser(char *line, struct membuffer *str, struct git_parser_state *state)
@@ -1180,7 +1237,7 @@ static void finish_active_trip(struct git_parser_state *state)
 
 	if (trip) {
 		state->active_trip = NULL;
-		insert_trip(trip, &trip_table);
+		insert_trip(trip, state->trips);
 	}
 }
 
@@ -1190,7 +1247,7 @@ static void finish_active_dive(struct git_parser_state *state)
 
 	if (dive) {
 		state->active_dive = NULL;
-		record_dive(dive);
+		record_dive_to_table(dive, state->table);
 	}
 }
 
@@ -1493,7 +1550,6 @@ static int parse_dive_entry(struct git_parser_state *state, const git_tree_entry
 		return report_error("Unable to read dive file");
 	if (*suffix)
 		dive->number = atoi(suffix + 1);
-	state->cylinder_index = 0;
 	clear_weightsystem_table(&state->active_dive->weightsystems);
 	state->o2pressure_sensor = 1;
 	for_each_line(blob, dive_parser, state);
@@ -1575,7 +1631,7 @@ static int walk_tree_file(const char *root, const git_tree_entry *entry, struct
 	dive_trip_t *trip = state->active_trip;
 	const char *name = git_tree_entry_name(entry);
 	if (verbose > 1)
-		fprintf(stderr, "git load handling file %s\n", name);
+		SSRF_INFO("git load handling file %s\n", name);
 	switch (*name) {
 	case '-': case '+':
 		if (dive)
@@ -1623,15 +1679,17 @@ static int load_dives_from_tree(git_repository *repo, git_tree *tree, struct git
 
 void clear_git_id(void)
 {
+	free((void *)saved_git_id);
 	saved_git_id = NULL;
 }
 
 void set_git_id(const struct git_oid *id)
 {
-	static char git_id_buffer[GIT_OID_HEXSZ + 1];
+	char git_id_buffer[GIT_OID_HEXSZ + 1];
 
 	git_oid_tostr(git_id_buffer, sizeof(git_id_buffer), id);
-	saved_git_id = git_id_buffer;
+	free((void *)saved_git_id);
+	saved_git_id = strdup(git_id_buffer);
 }
 
 static int find_commit(git_repository *repo, const char *branch, git_commit **commit_p)
@@ -1685,11 +1743,14 @@ const char *get_sha(git_repository *repo, const char *branch)
  * If it is a git repository, we return zero for success,
  * or report an error and return 1 if the load failed.
  */
-int git_load_dives(struct git_repository *repo, const char *branch)
+int git_load_dives(struct git_repository *repo, const char *branch, struct dive_table *table, struct trip_table *trips, struct dive_site_table *sites)
 {
 	int ret;
 	struct git_parser_state state = { 0 };
 	state.repo = repo;
+	state.table = table;
+	state.trips = trips;
+	state.sites = sites;
 
 	if (repo == dummy_git_repository)
 		return report_error("Unable to open git repository at '%s'", branch);
