@@ -13,7 +13,6 @@
 #include <unistd.h>
 #include <assert.h>
 #define __USE_XOPEN
-#include <time.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/tree.h>
@@ -26,9 +25,11 @@
 #include "errorhelper.h"
 #include "subsurface-string.h"
 #include "parse.h"
+#include "subsurface-time.h"
 #include "trip.h"
 #include "device.h"
 #include "membuffer.h"
+#include "picture.h"
 #include "qthelper.h"
 #include "tag.h"
 
@@ -225,7 +226,7 @@ static void cylinder_use(char *buffer, enum cylinderuse *cyl_use, struct parser_
 		int use = cylinderuse_from_text(buffer);
 		*cyl_use = use;
 		if (use == OXYGEN)
-			state->o2pressure_sensor = state->cur_cylinder_index;
+			state->o2pressure_sensor = state->cur_dive->cylinders.nr - 1;
 	}
 }
 
@@ -343,6 +344,7 @@ static void sampletime(char *buffer, duration_t *time)
 		time->seconds = (hr * 60 + min) * 60 + sec;
 		break;
 	default:
+		time->seconds = 0;
 		printf("Strange sample time reading %s\n", buffer);
 	}
 }
@@ -408,8 +410,7 @@ static void gasmix(char *buffer, fraction_t *fraction, struct parser_state *stat
 	/* libdivecomputer does negative percentages. */
 	if (*buffer == '-')
 		return;
-	if (state->cur_cylinder_index < MAX_CYLINDERS)
-		percent(buffer, fraction);
+	percent(buffer, fraction);
 }
 
 static void gasmix_nitrogen(char *buffer, struct gasmix *gasmix)
@@ -697,29 +698,6 @@ static void try_to_match_autogroup(const char *name, char *buf)
 	nonmatch("autogroup", name, buf);
 }
 
-void add_gas_switch_event(struct dive *dive, struct divecomputer *dc, int seconds, int idx)
-{
-	/* sanity check so we don't crash */
-	if (idx < 0 || idx >= MAX_CYLINDERS)
-		return;
-	/* The gas switch event format is insane for historical reasons */
-	struct gasmix mix = dive->cylinder[idx].gasmix;
-	int o2 = get_o2(mix);
-	int he = get_he(mix);
-	struct event *ev;
-	int value;
-
-	o2 = (o2 + 5) / 10;
-	he = (he + 5) / 10;
-	value = o2 + (he << 16);
-
-	ev = add_event(dc, seconds, he ? SAMPLE_EVENT_GASCHANGE2 : SAMPLE_EVENT_GASCHANGE, 0, value, "gaschange");
-	if (ev) {
-		ev->gas.index = idx;
-		ev->gas.mix = mix;
-	}
-}
-
 static void get_cylinderindex(char *buffer, uint8_t *i, struct parser_state *state)
 {
 	*i = atoi(buffer);
@@ -1000,16 +978,35 @@ static void divinglog_place(char *place, struct dive *d, struct parser_state *st
 
 static int divinglog_dive_match(struct dive *dive, const char *name, char *buf, struct parser_state *state)
 {
+	/* For cylinder related fields, we might have to create a cylinder first. */
+	cylinder_t cyl = empty_cylinder;
+	if (MATCH("tanktype", utf8_string, &cyl.type.description)) {
+		cylinder_t *cyl0 = get_or_create_cylinder(dive, 0);
+		free((void *)cyl0->type.description);
+		cyl0->type.description = cyl.type.description;
+		return 1;
+	}
+	if (MATCH("tanksize", cylindersize, &cyl.type.size)) {
+		get_or_create_cylinder(dive, 0)->type.size = cyl.type.size;
+		return 1;
+	}
+	if (MATCH_STATE("presw", pressure, &cyl.type.workingpressure)) {
+		get_or_create_cylinder(dive, 0)->type.workingpressure = cyl.type.workingpressure;
+		return 1;
+	}
+	if (MATCH_STATE("press", pressure, &cyl.start)) {
+		get_or_create_cylinder(dive, 0)->start = cyl.start;
+		return 1;
+	}
+	if (MATCH_STATE("prese", pressure, &cyl.end)) {
+		get_or_create_cylinder(dive, 0)->end = cyl.end;
+		return 1;
+	}
 	return MATCH_STATE("divedate", divedate, &dive->when) ||
 	       MATCH_STATE("entrytime", divetime, &dive->when) ||
 	       MATCH("divetime", duration, &dive->dc.duration) ||
 	       MATCH_STATE("depth", depth, &dive->dc.maxdepth) ||
 	       MATCH_STATE("depthavg", depth, &dive->dc.meandepth) ||
-	       MATCH("tanktype", utf8_string, &dive->cylinder[0].type.description) ||
-	       MATCH("tanksize", cylindersize, &dive->cylinder[0].type.size) ||
-	       MATCH_STATE("presw", pressure, &dive->cylinder[0].type.workingpressure) ||
-	       MATCH_STATE("press", pressure, &dive->cylinder[0].start) ||
-	       MATCH_STATE("prese", pressure, &dive->cylinder[0].end) ||
 	       MATCH("comments", utf8_string, &dive->notes) ||
 	       MATCH("names.buddy", utf8_string, &dive->buddy) ||
 	       MATCH("name.country", utf8_string, &state->country) ||
@@ -1222,6 +1219,8 @@ static void gps_picture_location(char *buffer, struct picture *pic)
 static void try_to_fill_dive(struct dive *dive, const char *name, char *buf, struct parser_state *state)
 {
 	char *hash = NULL;
+	cylinder_t *cyl = dive->cylinders.nr > 0 ? get_cylinder(dive, dive->cylinders.nr - 1) : NULL;
+	pressure_t p;
 	start_match("dive", name, buf);
 
 	switch (state->import_source) {
@@ -1270,10 +1269,14 @@ static void try_to_fill_dive(struct dive *dive, const char *name, char *buf, str
 		free(hash);
 		return;
 	}
-	if (MATCH_STATE("cylinderstartpressure", pressure, &dive->cylinder[0].start))
+	if (MATCH_STATE("cylinderstartpressure", pressure, &p)) {
+		get_or_create_cylinder(dive, 0)->start = p;
 		return;
-	if (MATCH_STATE("cylinderendpressure", pressure, &dive->cylinder[0].end))
+	}
+	if (MATCH_STATE("cylinderendpressure", pressure, &p)) {
+		get_or_create_cylinder(dive, 0)->end = p;
 		return;
+	}
 	if (MATCH_STATE("gps", gps_in_dive, dive))
 		return;
 	if (MATCH_STATE("Place", gps_in_dive, dive))
@@ -1304,10 +1307,20 @@ static void try_to_fill_dive(struct dive *dive, const char *name, char *buf, str
 		return;
 	if (MATCH("buddy", utf8_string, &dive->buddy))
 		return;
+	if (MATCH("watersalinity", salinity, &dive->user_salinity))
+		return;
 	if (MATCH("rating.dive", get_rating, &dive->rating))
 		return;
 	if (MATCH("visibility.dive", get_rating, &dive->visibility))
 		return;
+	if (MATCH("wavesize.dive", get_rating, &dive->wavesize))
+		return;
+	if (MATCH("current.dive", get_rating, &dive->current))
+		return;
+	if (MATCH("surge.dive", get_rating, &dive->surge))
+		return;
+	if (MATCH("chill.dive", get_rating, &dive->chill))
+		return;
 	if (MATCH_STATE("airpressure.dive", pressure, &dive->surface_pressure))
 		return;
 	if (MATCH("description.weightsystem", utf8_string, &dive->weightsystems.weightsystems[dive->weightsystems.nr - 1].description))
@@ -1316,34 +1329,36 @@ static void try_to_fill_dive(struct dive *dive, const char *name, char *buf, str
 		return;
 	if (MATCH_STATE("weight", weight, &dive->weightsystems.weightsystems[dive->weightsystems.nr - 1].weight))
 		return;
-	if (state->cur_cylinder_index < MAX_CYLINDERS) {
-		if (MATCH("size.cylinder", cylindersize, &dive->cylinder[state->cur_cylinder_index].type.size))
+	if (cyl) {
+		if (MATCH("size.cylinder", cylindersize, &cyl->type.size))
 			return;
-		if (MATCH_STATE("workpressure.cylinder", pressure, &dive->cylinder[state->cur_cylinder_index].type.workingpressure))
+		if (MATCH_STATE("workpressure.cylinder", pressure, &cyl->type.workingpressure))
 			return;
-		if (MATCH("description.cylinder", utf8_string, &dive->cylinder[state->cur_cylinder_index].type.description))
+		if (MATCH("description.cylinder", utf8_string, &cyl->type.description))
 			return;
-		if (MATCH_STATE("start.cylinder", pressure, &dive->cylinder[state->cur_cylinder_index].start))
+		if (MATCH_STATE("start.cylinder", pressure, &cyl->start))
 			return;
-		if (MATCH_STATE("end.cylinder", pressure, &dive->cylinder[state->cur_cylinder_index].end))
+		if (MATCH_STATE("end.cylinder", pressure, &cyl->end))
 			return;
-		if (MATCH_STATE("use.cylinder", cylinder_use, &dive->cylinder[state->cur_cylinder_index].cylinder_use))
+		if (MATCH_STATE("use.cylinder", cylinder_use, &cyl->cylinder_use))
 			return;
-		if (MATCH_STATE("depth.cylinder", depth, &dive->cylinder[state->cur_cylinder_index].depth))
+		if (MATCH_STATE("depth.cylinder", depth, &cyl->depth))
 			return;
-		if (MATCH_STATE("o2", gasmix, &dive->cylinder[state->cur_cylinder_index].gasmix.o2))
+		if (MATCH_STATE("o2", gasmix, &cyl->gasmix.o2))
 			return;
-		if (MATCH_STATE("o2percent", gasmix, &dive->cylinder[state->cur_cylinder_index].gasmix.o2))
+		if (MATCH_STATE("o2percent", gasmix, &cyl->gasmix.o2))
 			return;
-		if (MATCH("n2", gasmix_nitrogen, &dive->cylinder[state->cur_cylinder_index].gasmix))
+		if (MATCH("n2", gasmix_nitrogen, &cyl->gasmix))
 			return;
-		if (MATCH_STATE("he", gasmix, &dive->cylinder[state->cur_cylinder_index].gasmix.he))
+		if (MATCH_STATE("he", gasmix, &cyl->gasmix.he))
 			return;
 	}
 	if (MATCH_STATE("air.divetemperature", temperature, &dive->airtemp))
 		return;
 	if (MATCH_STATE("water.divetemperature", temperature, &dive->watertemp))
 		return;
+	if (MATCH("invalid", get_bool, &dive->invalid))
+		return;
 
 	nonmatch("dive", name, buf);
 }
@@ -1534,13 +1549,14 @@ static void uddf_importer(struct parser_state *state)
 	state->xml_parsing_units.temperature = KELVIN;
 }
 
+typedef void (*parser_func)(struct parser_state *);
 /*
  * I'm sure this could be done as some fancy DTD rules.
  * It's just not worth the headache.
  */
 static struct nesting {
 	const char *name;
-	void (*start)(struct parser_state *), (*end)(struct parser_state *);
+	parser_func start, end;
 } nesting[] = {
 	  { "divecomputerid", dc_settings_start, dc_settings_end },
 	  { "settings", settings_start, settings_end },
@@ -1553,9 +1569,9 @@ static struct nesting {
 	  { "SAMPLE", sample_start, sample_end },
 	  { "reading", sample_start, sample_end },
 	  { "event", event_start, event_end },
-	  { "mix", cylinder_start, cylinder_end },
-	  { "gasmix", cylinder_start, cylinder_end },
-	  { "cylinder", cylinder_start, cylinder_end },
+	  { "mix", (parser_func)cylinder_start, (parser_func)cylinder_end },
+	  { "gasmix", (parser_func)cylinder_start, (parser_func)cylinder_end },
+	  { "cylinder", (parser_func)cylinder_start, (parser_func)cylinder_end },
 	  { "weightsystem", ws_start, ws_end },
 	  { "divecomputer", divecomputer_start, divecomputer_end },
 	  { "P", sample_start, sample_end },
@@ -1710,6 +1726,7 @@ int parse_dlf_buffer(unsigned char *buffer, size_t size, struct dive_table *tabl
 	struct battery_status battery_start = {0, 0, 0, 0};
 	struct battery_status battery_end = {0, 0, 0, 0};
 	uint16_t o2_sensor_calibration_values[4] = {0};
+	cylinder_t *cyl;
 	struct parser_state state;
 
 	init_parser_state(&state);
@@ -1765,8 +1782,9 @@ int parse_dlf_buffer(unsigned char *buffer, size_t size, struct dive_table *tabl
 	state.cur_dc->surface_pressure.mbar = ((ptr[25] << 8) + ptr[24]) / 10;
 
 	// Declare initial mix as first cylinder
-	state.cur_dive->cylinder[0].gasmix.o2.permille = ptr[26] * 10;
-	state.cur_dive->cylinder[0].gasmix.he.permille = ptr[27] * 10;
+	cyl = get_or_create_cylinder(state.cur_dive, 0);
+	cyl->gasmix.o2.permille = ptr[26] * 10;
+	cyl->gasmix.he.permille = ptr[27] * 10;
 
 	/* Done with parsing what we know about the dive header */
 	ptr += 32;
@@ -1874,18 +1892,19 @@ int parse_dlf_buffer(unsigned char *buffer, size_t size, struct dive_table *tabl
 				state.cur_event.value = ptr[7] << 8 ^ ptr[6];
 
 				found = false;
-				for (i = 0; i < state.cur_cylinder_index; ++i) {
-					if (state.cur_dive->cylinder[i].gasmix.o2.permille == ptr[6] * 10 && state.cur_dive->cylinder[i].gasmix.he.permille == ptr[7] * 10) {
+				for (i = 0; i < state.cur_dive->cylinders.nr; ++i) {
+					const cylinder_t *cyl = get_cylinder(state.cur_dive, i);
+					if (cyl->gasmix.o2.permille == ptr[6] * 10 && cyl->gasmix.he.permille == ptr[7] * 10) {
 						found = true;
 						break;
 					}
 				}
 				if (!found) {
-					cylinder_start(&state);
-					state.cur_dive->cylinder[state.cur_cylinder_index].gasmix.o2.permille = ptr[6] * 10;
-					state.cur_dive->cylinder[state.cur_cylinder_index].gasmix.he.permille = ptr[7] * 10;
+					cyl = cylinder_start(&state);
+					cyl->gasmix.o2.permille = ptr[6] * 10;
+					cyl->gasmix.he.permille = ptr[7] * 10;
 					cylinder_end(&state);
-					state.cur_event.gas.index = state.cur_cylinder_index;
+					state.cur_event.gas.index = state.cur_dive->cylinders.nr - 1;
 				} else {
 					state.cur_event.gas.index = i;
 				}
