@@ -4,6 +4,8 @@
 
 #define MAX_EVENT_NAME 128
 
+#include "dive.h" // for struct event!
+
 #include <sqlite3.h>
 
 typedef union {
@@ -54,7 +56,6 @@ struct parser_state {
 	bool in_settings;
 	bool in_userid;
 	struct tm cur_tm;
-	int cur_cylinder_index;
 	int lastcylinderindex, next_o2_sensor;
 	int o2pressure_sensor;
 	int sample_rate;
@@ -102,7 +103,7 @@ void trip_start(struct parser_state *state);
 void trip_end(struct parser_state *state);
 void picture_start(struct parser_state *state);
 void picture_end(struct parser_state *state);
-void cylinder_start(struct parser_state *state);
+cylinder_t *cylinder_start(struct parser_state *state);
 void cylinder_end(struct parser_state *state);
 void ws_start(struct parser_state *state);
 void ws_end(struct parser_state *state);
