@@ -9,7 +9,7 @@
 #include "core/settings/qPref.h"
 
 char *settings_suffix = NULL;
-static QTranslator *qtTranslator, *ssrfTranslator;
+static QTranslator qtTranslator, ssrfTranslator;
 
 void init_qt_late()
 {
@@ -24,7 +24,7 @@ void init_qt_late()
 
 	QCoreApplication::setOrganizationName("Subsurface");
 	QCoreApplication::setOrganizationDomain("subsurface.hohndel.org");
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)) && defined(Q_OS_LINUX)
+#if defined(Q_OS_LINUX)
 	QGuiApplication::setDesktopFileName("subsurface");
 #endif
 	// enable user specific settings (based on command line argument)
@@ -59,34 +59,44 @@ void init_qt_late()
 	QLocale loc;
 
 	// assign en_GB for use in South African locale
+	// and capture other French and Spanish speaking countries with the corresponding canonical locales
 	if (loc.country() == QLocale::SouthAfrica) {
 		loc.setDefault(QLocale("en_GB"));
 		loc = QLocale();
+	} else if (loc.language() == QLocale::French) {
+		loc.setDefault(QLocale("fr_FR"));
+		loc = QLocale();
+	} else if (loc.language() == QLocale::Spanish) {
+		loc.setDefault(QLocale("es_ES"));
+		loc = QLocale();
+	} else if (loc.language() == QLocale::German && loc.country() != QLocale::Switzerland) {
+		loc.setDefault(QLocale("de_DE"));
+		loc = QLocale();
 	}
-	QString uiLang = uiLanguage(&loc);
+	initUiLanguage();
+	QString uiLang = getUiLanguage();
+	loc = getLocale();
 	QLocale::setDefault(loc);
 
-	qtTranslator = new QTranslator;
 	QString translationLocation;
 #if defined(Q_OS_ANDROID)
-	translationLocation = QLatin1Literal("assets:/translations");
+	translationLocation = QLatin1String("assets:/translations");
 #elif defined(Q_OS_IOS)
-	translationLocation = QLatin1Literal(":/");
+	translationLocation = QLatin1String(":/");
 #else
 	translationLocation = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
 #endif
-	if (qtTranslator->load(loc, "qt", "_", translationLocation)) {
-		application->installTranslator(qtTranslator);
+	if (qtTranslator.load(loc, "qt", "_", translationLocation)) {
+		application->installTranslator(&qtTranslator);
 	} else {
 		if (verbose && uiLang != "en_US" && uiLang != "en-US")
 			qDebug() << "can't find Qt localization for locale" << uiLang << "searching in" << translationLocation;
 	}
-	ssrfTranslator = new QTranslator;
-	if (ssrfTranslator->load(loc, "subsurface", "_") ||
-	    ssrfTranslator->load(loc, "subsurface", "_", translationLocation) ||
-	    ssrfTranslator->load(loc, "subsurface", "_", getSubsurfaceDataPath("translations")) ||
-	    ssrfTranslator->load(loc, "subsurface", "_", getSubsurfaceDataPath("../translations"))) {
-		application->installTranslator(ssrfTranslator);
+	if (ssrfTranslator.load(loc, "subsurface", "_") ||
+	    ssrfTranslator.load(loc, "subsurface", "_", translationLocation) ||
+	    ssrfTranslator.load(loc, "subsurface", "_", getSubsurfaceDataPath("translations")) ||
+	    ssrfTranslator.load(loc, "subsurface", "_", getSubsurfaceDataPath("../translations"))) {
+		application->installTranslator(&ssrfTranslator);
 	} else {
 		qDebug() << "can't find Subsurface localization for locale" << uiLang;
 	}
