@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "qthelper.h"
+#include "dive.h"
 #include "core/settings/qPrefLanguage.h"
 #include "core/settings/qPrefUpdateManager.h"
 #include "subsurface-string.h"
@@ -9,16 +10,15 @@
 #include "membuffer.h"
 #include "subsurfacesysinfo.h"
 #include "version.h"
-#include "divecomputer.h"
 #include "errorhelper.h"
 #include "planner.h"
-#include "time.h"
+#include "subsurface-time.h"
 #include "gettextfromc.h"
 #include "applicationstate.h"
 #include "metadata.h"
-#include <sys/time.h>
 #include "exif.h"
 #include "file.h"
+#include "picture.h"
 #include "tag.h"
 #include "trip.h"
 #include "imagedownloader.h"
@@ -46,7 +46,7 @@ static QLocale loc;
 
 static inline QString degreeSigns()
 {
-    return QStringLiteral("dD\u00b0");
+	return QStringLiteral("dD\u00b0");
 }
 
 QString weight_string(int weight_in_grams)
@@ -120,9 +120,9 @@ extern "C" char *printGPSCoordsC(const location_t *location)
 /**
 * Try to parse in a generic manner a coordinate.
 */
-static bool parseCoord(const QString& txt, int& pos, const QString& positives,
-		       const QString& negatives, const QString& others,
-		       double& value)
+static bool parseCoord(const QString &txt, int &pos, const QString &positives,
+		       const QString &negatives, const QString &others,
+		       double &value)
 {
 	bool numberDefined = false, degreesDefined = false,
 		minutesDefined = false, secondsDefined = false;
@@ -205,15 +205,14 @@ static bool parseCoord(const QString& txt, int& pos, const QString& positives,
 		}
 		++pos;
 	}
-	if (!degreesDefined && numberDefined) {
+	if (!degreesDefined && numberDefined)
 		value = number; //just a single number => degrees
-	} else if (!minutesDefined && numberDefined) {
+	else if (!minutesDefined && numberDefined)
 		value += number / 60.0;
-	} else if (!secondsDefined && numberDefined) {
+	else if (!secondsDefined && numberDefined)
 		value += number / 3600.0;
-	} else if (numberDefined) {
+	else if (numberDefined)
 		return false;
-	}
 	if (sign == -1) value *= -1.0;
 	return true;
 }
@@ -221,7 +220,7 @@ static bool parseCoord(const QString& txt, int& pos, const QString& positives,
 /**
 * Parse special coordinate formats that cannot be handled by parseCoord.
 */
-static bool parseSpecialCoords(const QString& txt, double& latitude, double& longitude) {
+static bool parseSpecialCoords(const QString &txt, double &latitude, double &longitude) {
 	QRegExp xmlFormat("(-?\\d+(?:\\.\\d+)?),?\\s+(-?\\d+(?:\\.\\d+)?)");
 	if (xmlFormat.exactMatch(txt)) {
 		latitude = xmlFormat.cap(1).toDouble();
@@ -238,10 +237,16 @@ bool parseGpsText(const QString &gps_text, double *latitude, double *longitude)
 	static const QString POS_LON = QString("+E") + gettextFromC::tr("E");
 	static const QString NEG_LON = QString("-W") + gettextFromC::tr("W");
 
-	//remove the useless spaces (but keep the ones separating numbers)
+	// Remove the useless spaces (but keep the ones separating numbers)
+	// and normalize different ways of writing separators.
 	static const QRegExp SPACE_CLEANER("\\s*([" + POS_LAT + NEG_LAT + POS_LON +
 		NEG_LON + degreeSigns() + "'\"\\s])\\s*");
-	const QString normalized = gps_text.trimmed().toUpper().replace(SPACE_CLEANER, "\\1");
+	const QString normalized = gps_text.trimmed().toUpper().
+		replace(SPACE_CLEANER, "\\1").
+		replace(QStringLiteral("′"), "'").
+		replace(QStringLiteral("’"), "'").
+		replace(QStringLiteral("''"), "\"").
+		replace(QStringLiteral("″"), "\"");
 
 	if (normalized.isEmpty()) {
 		*latitude = 0.0;
@@ -388,13 +393,14 @@ QVector<QPair<QString, int>> selectedDivesGasUsed()
 	for_each_dive (i, d) {
 		if (!d->selected)
 			continue;
-		volume_t diveGases[MAX_CYLINDERS] = {};
-		get_gas_used(d, diveGases);
-		for (j = 0; j < MAX_CYLINDERS; j++)
+		volume_t *diveGases = get_gas_used(d);
+		for (j = 0; j < d->cylinders.nr; j++) {
 			if (diveGases[j].mliter) {
-				QString gasName = gasname(d->cylinder[j].gasmix);
+				QString gasName = gasname(get_cylinder(d, j)->gasmix);
 				gasUsed[gasName] += diveGases[j].mliter;
 			}
+		}
+		free(diveGases);
 	}
 	QVector<QPair<QString, int>> gasUsedOrdered;
 	gasUsedOrdered.reserve(gasUsed.size());
@@ -420,7 +426,7 @@ QString getUserAgent()
 	userAgent.append(arch);
 	if (arch == "i386")
 		userAgent.append("/" + SubsurfaceSysInfo::currentCpuArchitecture());
-	userAgent.append(":" + uiLanguage(NULL));
+	userAgent.append(":" + getUiLanguage());
 	return userAgent;
 
 }
@@ -432,21 +438,20 @@ extern "C" const char *subsurface_user_agent()
 	return copy_qstring(uA);
 }
 
+QString getUiLanguage()
+{
+	return prefs.locale.lang_locale;
+}
+
 /* TOOD: Move this to SettingsObjectWrapper, and also fix this complexity.
  * gezus.
  */
-QString uiLanguage(QLocale *callerLoc)
+void initUiLanguage()
 {
 	QString shortDateFormat;
 	QString dateFormat;
 	QString timeFormat;
 
-	// Language settings are already loaded, see qPref::load()
-	// so no need to reload them
-
-	// remark this method used "useSystemLanguage", which is not set
-	// instead use_system_language is loaded from disk
-
 	// set loc as system language or selected language
 	if (!qPrefLanguage::use_system_language()) {
 		loc = QLocale(qPrefLanguage::lang_locale());
@@ -465,8 +470,6 @@ QString uiLanguage(QLocale *callerLoc)
 	else
 		uiLang = languages[0];
 
-	prefs.locale.lang_locale = copy_qstring(uiLang);
-
 	// there's a stupid Qt bug on MacOS where uiLanguages doesn't give us the country info
 	if (!uiLang.contains('-') && uiLang != loc.bcp47Name()) {
 		QLocale loc2(loc.bcp47Name());
@@ -479,8 +482,9 @@ QString uiLanguage(QLocale *callerLoc)
 		else if (languages.count() > 2 && languages[2].contains('-'))
 			uiLang = languages[2];
 	}
-	if (callerLoc)
-		*callerLoc = loc;
+
+	free((void*)prefs.locale.lang_locale);
+	prefs.locale.lang_locale = copy_qstring(uiLang);
 
 	if (!prefs.date_format_override || empty_string(prefs.date_format_short) || empty_string(prefs.date_format)) {
 		// derive our standard date format from what the locale gives us
@@ -509,7 +513,6 @@ QString uiLanguage(QLocale *callerLoc)
 		free((void *)prefs.time_format);
 		prefs.time_format = copy_qstring(timeFormat);
 	}
-	return uiLang;
 }
 
 QLocale getLocale()
@@ -523,16 +526,6 @@ void set_filename(const char *filename)
 	existing_filename = copy_string(filename);
 }
 
-QString get_dc_nickname(const char *model, uint32_t deviceid)
-{
-	const DiveComputerNode *existNode = dcList.getExact(model, deviceid);
-
-	if (existNode && !existNode->nickName.isEmpty())
-		return existNode->nickName;
-	else
-		return model;
-}
-
 QString get_depth_string(int mm, bool showunit, bool showdecimal)
 {
 	if (prefs.units.length == units::METERS) {
@@ -942,6 +935,23 @@ char *get_dive_date_c_string(timestamp_t when)
 	return copy_qstring(text);
 }
 
+static QString get_dive_only_date_string(timestamp_t when)
+{
+	QDateTime ts;
+	ts.setMSecsSinceEpoch(when * 1000L);
+	return loc.toString(ts.toUTC(), QString(prefs.date_format));
+}
+
+QString get_first_dive_date_string()
+{
+	return dive_table.nr > 0 ? get_dive_only_date_string(dive_table.dives[0]->when) : gettextFromC::tr("no dives");
+}
+
+QString get_last_dive_date_string()
+{
+	return dive_table.nr > 0 ? get_dive_only_date_string(dive_table.dives[dive_table.nr - 1]->when) : gettextFromC::tr("no dives");
+}
+
 extern "C" char *get_current_date()
 {
 	QDateTime ts(QDateTime::currentDateTime());;
@@ -1182,12 +1192,6 @@ QString get_gas_string(struct gasmix gas)
 	return result;
 }
 
-QString get_divepoint_gas_string(struct dive *d, const divedatapoint &p)
-{
-	int idx = p.cylinderid;
-	return get_gas_string(d->cylinder[idx].gasmix);
-}
-
 QString get_taglist_string(struct tag_entry *tag_list)
 {
 	char *buffer = taglist_get_tagstring(tag_list);
@@ -1196,6 +1200,14 @@ QString get_taglist_string(struct tag_entry *tag_list)
 	return ret;
 }
 
+QStringList stringToList(const QString &s)
+{
+	QStringList res = s.split(",", QString::SkipEmptyParts);
+	for (QString &str: res)
+		str = str.trimmed();
+	return res;
+}
+
 weight_t string_to_weight(const char *str)
 {
 	const char *end;
@@ -1613,3 +1625,56 @@ char *copy_qstring(const QString &s)
 {
 	return strdup(qPrintable(s));
 }
+
+// function to call to allow the UI to show updates for longer running activities
+void (*uiNotificationCallback)(QString msg) = nullptr;
+
+void uiNotification(const QString &msg)
+{
+	if (uiNotificationCallback != nullptr)
+		uiNotificationCallback(msg);
+}
+
+// function to call to get changes for a git commit
+QString (*changesCallback)() = nullptr;
+
+extern "C" char *get_changes_made()
+{
+	if (changesCallback != nullptr)
+		return copy_qstring(changesCallback());
+	else
+		return nullptr;
+}
+
+// Generate a cylinder-renumber map for use when the n-th cylinder
+// of a dive with count cylinders is removed. It fills an int vector
+// with 0..n, -1, n..count-1. Each entry in the vector represents
+// the new id of the cylinder, whereby <0 means that this particular
+// cylinder does not get any new id. This should probably be moved
+// to the C-core, but using std::vector is simply more convenient.
+// The function assumes that n < count!
+std::vector<int> get_cylinder_map_for_remove(int count, int n)
+{
+	// 1) Fill mapping[0]..mapping[n-1] with 0..n-1
+	// 2) Set mapping[n] to -1
+	// 3) Fill mapping[n+1]..mapping[count-1] with n..count-2
+	std::vector<int> mapping(count);
+	std::iota(mapping.begin(), mapping.begin() + n, 0);
+	mapping[n] = -1;
+	std::iota(mapping.begin() + n + 1, mapping.end(), n);
+	return mapping;
+}
+
+// Generate a cylinder-renumber map for use when a cylinder is added
+// before the n-th cylinder. It fills an int vector with
+// with 0..n-1, n+1..count. Each entry in the vector represents
+// the new id of the cylinder. This probably should be moved
+// to the C-core, but using std::vector is simply more convenient.
+// This function assumes that that n <= count!
+std::vector<int> get_cylinder_map_for_add(int count, int n)
+{
+	std::vector<int> mapping(count);
+	std::iota(mapping.begin(), mapping.begin() + n, 0);
+	std::iota(mapping.begin() + n, mapping.end(), n + 1);
+	return mapping;
+}
