@@ -2,7 +2,6 @@
 #ifndef QTHELPER_H
 #define QTHELPER_H
 
-#include <stdint.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 #include "core/pref.h"
@@ -27,8 +26,8 @@ QString distance_string(int distanceInMeters);
 bool gpsHasChanged(struct dive *dive, struct dive *master, const QString &gps_text, bool *parsed_out = 0);
 QList<int> getDivesInTrip(struct dive_trip *trip);
 QString get_gas_string(struct gasmix gas);
-QString get_divepoint_gas_string(struct dive *d, const struct divedatapoint &dp);
 QString get_taglist_string(struct tag_entry *tag_list);
+QStringList stringToList(const QString &s);
 void read_hashes();
 void write_hashes();
 QString thumbnailFileName(const QString &filename);
@@ -59,7 +58,6 @@ QString get_pressure_unit();
 QString getSubsurfaceDataPath(QString folderToFind);
 QString getPrintingTemplatePathUser();
 QString getPrintingTemplatePathBundle();
-QString get_dc_nickname(const char *model, uint32_t deviceid);
 int gettimezoneoffset(timestamp_t when = 0);
 int parseDurationToSeconds(const QString &text);
 int parseLengthToMm(const QString &text);
@@ -72,13 +70,21 @@ QString render_seconds_to_string(int seconds);
 QString get_dive_duration_string(timestamp_t when, QString hoursText, QString minutesText, QString secondsText = gettextFromC::tr("sec"), QString separator = ":", bool isFreeDive = false);
 QString get_dive_surfint_string(timestamp_t when, QString daysText, QString hoursText, QString minutesText, QString separator = " ", int maxdays = 4);
 QString get_dive_date_string(timestamp_t when);
+QString get_first_dive_date_string();
+QString get_last_dive_date_string();
 QString get_short_dive_date_string(timestamp_t when);
 QString get_trip_date_string(timestamp_t when, int nr, bool getday);
-QString uiLanguage(QLocale *callerLoc);
+QString getUiLanguage();
+void initUiLanguage();
 QLocale getLocale();
 QVector<QPair<QString, int>> selectedDivesGasUsed();
 QString getUserAgent();
 QString printGPSCoords(const location_t *loc);
+std::vector<int> get_cylinder_map_for_remove(int count, int n);
+std::vector<int> get_cylinder_map_for_add(int count, int n);
+
+extern QString (*changesCallback)();
+void uiNotification(const QString &msg);
 
 #if defined __APPLE__
 #define TITLE_OR_TEXT(_t, _m) "", _t + "\n" + _m
@@ -111,22 +117,6 @@ void moveInVector(Vector &v, int rangeBegin, int rangeEnd, int destination)
 	else if (destination < rangeBegin)
 		std::rotate(it + destination, it + rangeBegin, it + rangeEnd);
 }
-
-// Qt overloads some signals(!) which can't therefore be used in connect() calls with
-// pointers-to-member functions where the signatures of signal and slot don't match perfectly.
-// For this case, Qt 5.7 provides the qOverload<> helper.
-// Since we still support Qt 5.5 let's reimplement it here.
-#if QT_VERSION < 0x050700
-template <typename... Args>
-struct QOverload {
-	template <typename Ret, typename Class>
-	static Ret (Class::*of(Ret (Class::*fun)(Args...)))(Args...)
-	{
-		return fun;
-	}
-};
-#endif
-
 #endif
 
 // 3) Functions visible to C and C++
@@ -163,9 +153,18 @@ depth_t string_to_depth(const char *str);
 pressure_t string_to_pressure(const char *str);
 volume_t string_to_volume(const char *str, pressure_t workp);
 fraction_t string_to_fraction(const char *str);
-
+char *get_changes_made();
 #ifdef __cplusplus
 }
 #endif
 
+// 4) SSRF_INFO macro to replace fprintf calls in our code
+//    (originally based on logging bits from libdivecomputer)
+#if !defined(Q_OS_ANDROID) && !defined(__ANDROID__)
+#define SSRF_INFO(fmt, ...)	fprintf(stderr, "INFO: " fmt "\n", ##__VA_ARGS__)
+#else
+#include <android/log.h>
+#define SSRF_INFO(fmt, ...)	__android_log_print(ANDROID_LOG_INFO, "Subsurface", "INFO: " fmt "\n", ##__VA_ARGS__);
+#endif
+
 #endif // QTHELPER_H
