@@ -12,12 +12,7 @@
 #include <libdivecomputer/version.h>
 #include <libdivecomputer/context.h>
 #include <libdivecomputer/custom.h>
-
-#if defined(Q_OS_WIN)
-	#include <winsock2.h>
-	#include <windows.h>
-	#include <ws2bth.h>
-#endif
+#include <libdivecomputer/serial.h>
 
 #ifdef BLE_SUPPORT
 # include "qt-ble.h"
@@ -35,11 +30,7 @@ typedef struct qt_serial_t {
 	/*
 	 * RFCOMM socket used for Bluetooth Serial communication.
 	 */
-#if defined(Q_OS_WIN)
-	SOCKET socket;
-#else
 	QBluetoothSocket *socket;
-#endif
 	long timeout;
 } qt_serial_t;
 
@@ -54,53 +45,6 @@ static dc_status_t qt_serial_open(qt_serial_t **io, dc_context_t*, const char* d
 	// Default to blocking reads.
 	serial_port->timeout = -1;
 
-#if defined(Q_OS_WIN)
-	// Create a RFCOMM socket
-	serial_port->socket = ::socket(AF_BTH, SOCK_STREAM, BTHPROTO_RFCOMM);
-
-	if (serial_port->socket == INVALID_SOCKET) {
-		free(serial_port);
-		return DC_STATUS_IO;
-	}
-
-	SOCKADDR_BTH socketBthAddress;
-	int socketBthAddressBth = sizeof (socketBthAddress);
-	char *address = strdup(devaddr);
-
-	ZeroMemory(&socketBthAddress, socketBthAddressBth);
-	qDebug() << "Trying to connect to address " << devaddr;
-
-	if (WSAStringToAddressA(address,
-				AF_BTH,
-				NULL,
-				(LPSOCKADDR) &socketBthAddress,
-				&socketBthAddressBth
-				) != 0) {
-		qDebug() << "Failed to convert the address " << address;
-		free(address);
-
-		return DC_STATUS_IO;
-	}
-
-	free(address);
-
-	socketBthAddress.addressFamily = AF_BTH;
-	socketBthAddress.port = BT_PORT_ANY;
-	memset(&socketBthAddress.serviceClassId, 0, sizeof(socketBthAddress.serviceClassId));
-	socketBthAddress.serviceClassId = SerialPortServiceClass_UUID;
-
-	// Try to connect to the device
-	if (::connect(serial_port->socket,
-		      (struct sockaddr *) &socketBthAddress,
-		      socketBthAddressBth
-		      ) != 0) {
-		qDebug() << "Failed to connect to device";
-
-		return DC_STATUS_NODEVICE;
-	}
-
-	qDebug() << "Successfully connected to device";
-#else
 	// Create a RFCOMM socket
 	serial_port->socket = new QBluetoothSocket(QBluetoothServiceInfo::RfcommProtocol);
 
@@ -159,7 +103,6 @@ static dc_status_t qt_serial_open(qt_serial_t **io, dc_context_t*, const char* d
 			return DC_STATUS_IO;
 		}
 	}
-#endif
 
 	*io = serial_port;
 
@@ -173,11 +116,6 @@ static dc_status_t qt_serial_close(void *io)
 	if (device == NULL)
 		return DC_STATUS_SUCCESS;
 
-#if defined(Q_OS_WIN)
-	// Cleanup
-	closesocket(device->socket);
-	free(device);
-#else
 	if (device->socket == NULL) {
 		free(device);
 		return DC_STATUS_SUCCESS;
@@ -187,7 +125,6 @@ static dc_status_t qt_serial_close(void *io)
 
 	delete device->socket;
 	free(device);
-#endif
 
 	return DC_STATUS_SUCCESS;
 }
@@ -196,110 +133,94 @@ static dc_status_t qt_serial_read(void *io, void* data, size_t size, size_t *act
 {
 	qt_serial_t *device = (qt_serial_t*) io;
 
-#if defined(Q_OS_WIN)
-	if (device == NULL)
+	if (device == NULL || device->socket == NULL || !actual)
 		return DC_STATUS_INVALIDARGS;
 
-	size_t nbytes = 0;
-	int rc;
+	*actual = 0;
+	for (;;) {
+		int rc;
 
-	while (nbytes < size) {
-		rc = recv (device->socket, (char *) data + nbytes, size - nbytes, 0);
-
-		if (rc < 0) {
-			return DC_STATUS_IO; // Error during recv call.
-		} else if (rc == 0) {
-			break; // EOF reached.
-		}
-
-		nbytes += rc;
-	}
-#else
-	if (device == NULL || device->socket == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	size_t nbytes = 0;
-	int rc;
-
-	while(nbytes < size && device->socket->state() == QBluetoothSocket::ConnectedState)
-	{
-		rc = device->socket->read((char *) data + nbytes, size - nbytes);
+		if (device->socket->state() != QBluetoothSocket::ConnectedState)
+			return DC_STATUS_IO;
 
+		rc = device->socket->read((char *) data, size);
 		if (rc < 0) {
 			if (errno == EINTR || errno == EAGAIN)
-			    continue; // Retry.
-
-			return DC_STATUS_IO; // Something really bad happened :-(
-		} else if (rc == 0) {
-			// Wait until the device is available for read operations
-			QEventLoop loop;
-			QTimer timer;
-			timer.setSingleShot(true);
-			loop.connect(&timer, SIGNAL(timeout()), SLOT(quit()));
-			loop.connect(device->socket, SIGNAL(readyRead()), SLOT(quit()));
-			timer.start(device->timeout);
-			loop.exec();
-
-			if (!timer.isActive())
-				break;
+				continue;
+			return DC_STATUS_IO;
 		}
 
-		nbytes += rc;
-	}
-#endif
-	if (actual)
-		*actual = nbytes;
+		*actual = rc;
+		if (rc > 0 || !size)
+			return DC_STATUS_SUCCESS;
 
-	return DC_STATUS_SUCCESS;
+		// Timeout handling
+		QEventLoop loop;
+		QTimer timer;
+		timer.setSingleShot(true);
+		loop.connect(&timer, SIGNAL(timeout()), SLOT(quit()));
+		loop.connect(device->socket, SIGNAL(readyRead()), SLOT(quit()));
+		timer.start(device->timeout);
+		loop.exec();
+
+		if (!timer.isActive())
+			return DC_STATUS_TIMEOUT;
+	}
 }
 
 static dc_status_t qt_serial_write(void *io, const void* data, size_t size, size_t *actual)
 {
 	qt_serial_t *device = (qt_serial_t*) io;
 
-#if defined(Q_OS_WIN)
-	if (device == NULL)
+	if (device == NULL || device->socket == NULL || !actual)
 		return DC_STATUS_INVALIDARGS;
 
-	size_t nbytes = 0;
-	int rc;
+	*actual = 0;
+	for (;;) {
+		int rc;
 
-	while (nbytes < size) {
-	    rc = send(device->socket, (char *) data + nbytes, size - nbytes, 0);
+		if (device->socket->state() != QBluetoothSocket::ConnectedState)
+			return DC_STATUS_IO;
 
-	    if (rc < 0) {
-	       return DC_STATUS_IO; // Error during send call.
-	    }
-
-	    nbytes += rc;
-	}
-#else
-	if (device == NULL || device->socket == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	size_t nbytes = 0;
-	int rc;
-
-	while(nbytes < size && device->socket->state() == QBluetoothSocket::ConnectedState)
-	{
-		rc = device->socket->write((char *) data + nbytes, size - nbytes);
+		rc = device->socket->write((char *) data, size);
 
 		if (rc < 0) {
 			if (errno == EINTR || errno == EAGAIN)
-			    continue; // Retry.
-
-			return DC_STATUS_IO; // Something really bad happened :-(
-		} else if (rc == 0) {
-			break;
+				continue;
+			return DC_STATUS_IO;
 		}
 
-		nbytes += rc;
+		*actual = rc;
+		return rc ? DC_STATUS_SUCCESS : DC_STATUS_IO;
 	}
-#endif
-	if (actual)
-		*actual = nbytes;
+}
 
-	return DC_STATUS_SUCCESS;
+static dc_status_t qt_serial_poll(void *io, int timeout)
+{
+	qt_serial_t *device = (qt_serial_t*) io;
+
+	if (!device)
+		return DC_STATUS_INVALIDARGS;
+
+	if (!device->socket)
+		return DC_STATUS_INVALIDARGS;
+
+	QEventLoop loop;
+	QTimer timer;
+	timer.setSingleShot(true);
+	loop.connect(&timer, SIGNAL(timeout()), SLOT(quit()));
+	loop.connect(device->socket, SIGNAL(readyRead()), SLOT(quit()));
+	timer.start(timeout);
+	loop.exec();
+
+	if (!timer.isActive())
+		return DC_STATUS_SUCCESS;
+	return DC_STATUS_TIMEOUT;
+}
+
+static dc_status_t qt_serial_ioctl(void *io, unsigned int request, void *data, size_t size)
+{
+	return DC_STATUS_UNSUPPORTED;
 }
 
 static dc_status_t qt_serial_purge(void *io, dc_direction_t)
@@ -308,10 +229,6 @@ static dc_status_t qt_serial_purge(void *io, dc_direction_t)
 
 	if (device == NULL)
 		return DC_STATUS_INVALIDARGS;
-#if !defined(Q_OS_WIN)
-	if (device->socket == NULL)
-		return DC_STATUS_INVALIDARGS;
-#endif
 	// TODO: add implementation
 
 	return DC_STATUS_SUCCESS;
@@ -321,19 +238,10 @@ static dc_status_t qt_serial_get_available(void *io, size_t *available)
 {
 	qt_serial_t *device = (qt_serial_t*) io;
 
-#if defined(Q_OS_WIN)
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	// TODO use WSAIoctl to get the information
-
-	*available = 0;
-#else
 	if (device == NULL || device->socket == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	*available = device->socket->bytesAvailable();
-#endif
 
 	return DC_STATUS_SUCCESS;
 }
@@ -343,19 +251,10 @@ static int qt_serial_get_transmitted(qt_serial_t *device) __attribute__ ((unused
 
 static int qt_serial_get_transmitted(qt_serial_t *device)
 {
-#if defined(Q_OS_WIN)
-	if (device == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	// TODO add implementation
-
-	return 0;
-#else
 	if (device == NULL || device->socket == NULL)
 		return DC_STATUS_INVALIDARGS;
 
 	return device->socket->bytesToWrite();
-#endif
 }
 
 static dc_status_t qt_serial_set_timeout(void *io, int timeout)
@@ -385,20 +284,20 @@ ble_packet_open(dc_iostream_t **iostream, dc_context_t *context, const char* dev
 
 	static const dc_custom_cbs_t callbacks = {
 		qt_ble_set_timeout, /* set_timeout */
-		NULL, /* set_latency */
 		NULL, /* set_break */
 		NULL, /* set_dtr */
 		NULL, /* set_rts */
 		NULL, /* get_lines */
 		NULL, /* get_received */
 		NULL, /* configure */
+		qt_ble_poll, /* poll */
 		qt_ble_read, /* read */
 		qt_ble_write, /* write */
+		qt_ble_ioctl, /* ioctl */
 		NULL, /* flush */
 		NULL, /* purge */
 		qt_custom_sleep, /* sleep */
 		qt_ble_close, /* close */
-		qt_ble_get_name, /* get_name */
 	};
 
 	rc = qt_ble_open(&io, context, devaddr, (dc_user_device_t *) userdata);
@@ -419,15 +318,16 @@ rfcomm_stream_open(dc_iostream_t **iostream, dc_context_t *context, const char*
 
 	static const dc_custom_cbs_t callbacks = {
 		qt_serial_set_timeout, /* set_timeout */
-		NULL, /* set_latency */
 		NULL, /* set_break */
 		NULL, /* set_dtr */
 		NULL, /* set_rts */
 		NULL, /* get_lines */
 		qt_serial_get_available, /* get_received */
 		NULL, /* configure */
+		qt_serial_poll, /* poll */
 		qt_serial_read, /* read */
 		qt_serial_write, /* write */
+		qt_serial_ioctl, /* ioctl */
 		NULL, /* flush */
 		qt_serial_purge, /* purge */
 		qt_custom_sleep, /* sleep */
