@@ -10,13 +10,13 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <git2.h>
 
+#include "dive.h"
 #include "divesite.h"
 #include "subsurface-string.h"
 #include "trip.h"
@@ -25,9 +25,11 @@
 #include "membuffer.h"
 #include "git-access.h"
 #include "version.h"
+#include "picture.h"
 #include "qthelper.h"
 #include "gettext.h"
 #include "tag.h"
+#include "subsurface-time.h"
 
 #define VA_BUF(b, fmt) do { va_list args; va_start(args, fmt); put_vformat(b, fmt, args); va_end(args); } while (0)
 
@@ -138,7 +140,7 @@ static void save_cylinder_info(struct membuffer *b, struct dive *dive)
 
 	nr = nr_cylinders(dive);
 	for (i = 0; i < nr; i++) {
-		cylinder_t *cylinder = dive->cylinder + i;
+		cylinder_t *cylinder = get_cylinder(dive, i);
 		int volume = cylinder->type.size.mliter;
 		const char *description = cylinder->type.description;
 		int use = cylinder->cylinder_use;
@@ -433,14 +435,21 @@ static void create_dive_buffer(struct dive *dive, struct membuffer *b)
 		put_format(b, "duration %u:%02u min\n", FRACTION(dive->dc.duration.seconds, 60));
 	SAVE("rating", rating);
 	SAVE("visibility", visibility);
+	SAVE("wavesize", wavesize);
+	SAVE("current", current);
+	SAVE("surge", surge);
+	SAVE("chill", chill);
+	if (dive->user_salinity)
+		put_format(b, "watersalinity %d g/l\n", (int)(dive->user_salinity/10));
 	if (surface_pressure.mbar)
 		SAVE("airpressure", surface_pressure.mbar);
 	cond_put_format(dive->notrip, b, "notrip\n");
+	cond_put_format(dive->invalid, b, "invalid\n");
 	save_tags(b, dive->tag_list);
 	if (dive->dive_site)
 		put_format(b, "divesiteid %08x\n", dive->dive_site->uuid);
 	if (verbose && dive->dive_site)
-		fprintf(stderr, "removed reference to non-existant dive site with uuid %08x\n", dive->dive_site->uuid);
+		SSRF_INFO("removed reference to non-existant dive site with uuid %08x\n", dive->dive_site->uuid);
 	save_overview(b, dive);
 	save_cylinder_info(b, dive);
 	save_weightsystem_info(b, dive);
@@ -483,7 +492,7 @@ static int tree_insert(git_treebuilder *dir, const char *name, int mkunique, git
 	if (ret) {
 		const git_error *gerr = giterr_last();
 		if (gerr) {
-			fprintf(stderr, "tree_insert failed with return %d error %s\n", ret, gerr->message);
+			SSRF_INFO("tree_insert failed with return %d error %s\n", ret, gerr->message);
 		}
 	}
 	return ret;
@@ -1013,34 +1022,30 @@ int update_git_checkout(git_repository *repo, git_object *parent, git_tree *tree
 	return git_checkout_tree(repo, (git_object *) tree, &opts);
 }
 
-static int get_authorship(git_repository *repo, git_signature **authorp)
+int get_authorship(git_repository *repo, git_signature **authorp)
 {
-#if LIBGIT2_VER_MAJOR || LIBGIT2_VER_MINOR >= 20
 	if (git_signature_default(authorp, repo) == 0)
 		return 0;
-#endif
-	/* try to fetch the user info from the OS, otherwise use default values. */
-	struct user_info user = { .name = NULL, .email = NULL };
-	subsurface_user_info(&user);
-	if (!user.name || !*user.name)
-		user.name = strdup("Subsurface");
-	if (!user.email)
-		user.email = strdup("subsurface-app-account@subsurface-divelog.org");
 
-	/* git_signature_default() is too recent */
-	int ret = git_signature_now(authorp, user.name, user.email);
-	free((void *)user.name);
-	free((void *)user.email);
-	return ret;
+#ifdef SUBSURFACE_MOBILE
+#define APPNAME "Subsurface-mobile"
+#else
+#define APPNAME "Subsurface"
+#endif
+	return git_signature_now(authorp, APPNAME, "subsurface-app-account@subsurface-divelog.org");
+#undef APPNAME
 }
 
 static void create_commit_message(struct membuffer *msg, bool create_empty)
 {
 	int nr = dive_table.nr;
 	struct dive *dive = get_dive(nr-1);
+	char* changes_made = get_changes_made();
 
 	if (create_empty) {
 		put_string(msg, "Initial commit to create empty repo.\n\n");
+	} else if (!empty_string(changes_made)) {
+		put_format(msg, "Changes made: \n\n%s\n", changes_made);
 	} else if (dive) {
 		dive_trip_t *trip = dive->divetrip;
 		const char *location = get_dive_location(dive) ? : "no location";
@@ -1065,6 +1070,9 @@ static void create_commit_message(struct membuffer *msg, bool create_empty)
 	const char *user_agent = subsurface_user_agent();
 	put_format(msg, "Created by %s\n", user_agent);
 	free((void *)user_agent);
+	free(changes_made);
+	if (verbose)
+		SSRF_INFO("Commit message:\n\n%s\n", mb_cstring(msg));
 }
 
 static int create_new_commit(git_repository *repo, const char *remote, const char *branch, git_oid *tree_id, bool create_empty)
@@ -1093,7 +1101,7 @@ static int create_new_commit(git_repository *repo, const char *remote, const cha
 
 		if (saved_git_id) {
 			if (existing_filename && verbose)
-				fprintf(stderr, "existing filename %s\n", existing_filename);
+				SSRF_INFO("existing filename %s\n", existing_filename);
 			const git_oid *id = git_commit_id((const git_commit *) parent);
 			/* if we are saving to the same git tree we got this from, let's make
 			 * sure there is no confusion */
@@ -1189,6 +1197,11 @@ static int write_git_tree(git_repository *repo, struct dir *tree, git_oid *resul
 
 	/* .. write out the resulting treebuilder */
 	ret = git_treebuilder_write(result, tree->files);
+	if (ret && verbose) {
+		const git_error *gerr = giterr_last();
+		if (gerr)
+			SSRF_INFO("tree_insert failed with return %d error %s\n", ret, gerr->message);
+	}
 
 	/* .. and free the now useless treebuilder */
 	git_treebuilder_free(tree->files);
@@ -1203,7 +1216,7 @@ int do_git_save(git_repository *repo, const char *branch, const char *remote, bo
 	bool cached_ok;
 
 	if (verbose)
-		fprintf(stderr, "git storage: do git save\n");
+		SSRF_INFO("git storage: do git save\n");
 
 	if (!create_empty) // so we are actually saving the dives
 		git_storage_update_progress(translate("gettextFromC", "Preparing to save data"));
@@ -1226,7 +1239,7 @@ int do_git_save(git_repository *repo, const char *branch, const char *remote, bo
 			return -1;
 
 	if (verbose)
-		fprintf(stderr, "git storage, write git tree\n");
+		SSRF_INFO("git storage, write git tree\n");
 
 	if (write_git_tree(repo, &tree, &id))
 		return report_error("git tree write failed");
