@@ -5,12 +5,15 @@
 #endif
 
 #include "save-html.h"
+#include "dive.h"
 #include "qthelper.h"
 #include "gettext.h"
 #include "divesite.h"
 #include "errorhelper.h"
 #include "file.h"
+#include "picture.h"
 #include "tag.h"
+#include "subsurface-time.h"
 #include "trip.h"
 #include <stdio.h>
 
@@ -132,7 +135,7 @@ static void put_cylinder_HTML(struct membuffer *b, struct dive *dive)
 		put_string(b, separator);
 
 	for (i = 0; i < nr; i++) {
-		cylinder_t *cylinder = dive->cylinder + i;
+		cylinder_t *cylinder = get_cylinder(dive, i);
 		put_format(b, "%s{", separator);
 		separator = ", ";
 		write_attribute(b, "Type", cylinder->type.description, ", ");
@@ -344,7 +347,7 @@ void put_HTML_tags(struct membuffer *b, struct dive *dive, const char *pre, cons
 }
 
 /* if exporting list_only mode, we neglect exporting the samples, bookmarks and cylinders */
-void write_one_dive(struct membuffer *b, struct dive *dive, const char *photos_dir, int *dive_no, const bool list_only)
+void write_one_dive(struct membuffer *b, struct dive *dive, const char *photos_dir, int *dive_no, bool list_only)
 {
 	put_string(b, "{");
 	put_format(b, "\"number\":%d,", *dive_no);
@@ -355,6 +358,10 @@ void write_one_dive(struct membuffer *b, struct dive *dive, const char *photos_d
 	put_HTML_coordinates(b, dive);
 	put_format(b, "\"rating\":%d,", dive->rating);
 	put_format(b, "\"visibility\":%d,", dive->visibility);
+	put_format(b, "\"current\":%d,", dive->current);
+	put_format(b, "\"wavesize\":%d,", dive->wavesize);
+	put_format(b, "\"surge\":%d,", dive->surge);
+	put_format(b, "\"chill\":%d,", dive->chill);
 	put_format(b, "\"dive_duration\":\"%u:%02u min\",",
 		   FRACTION(dive->duration.seconds, 60));
 	put_string(b, "\"temperature\":{");
@@ -512,8 +519,13 @@ void export_translation(const char *file_name)
 	write_attribute(b, "Advanced_Search", translate("gettextFromC", "Advanced search"), ", ");
 
 	//Dive expanded view
+
 	write_attribute(b, "Rating", translate("gettextFromC", "Rating"), ", ");
+	write_attribute(b, "WaveSize", translate("gettextFromC", "WaveSize"), ", ");
 	write_attribute(b, "Visibility", translate("gettextFromC", "Visibility"), ", ");
+	write_attribute(b, "Current", translate("gettextFromC", "Current"), ", ");
+	write_attribute(b, "Surge", translate("gettextFromC", "Surge"), ", ");
+	write_attribute(b, "Chill", translate("gettextFromC", "Chill"), ", ");
 	write_attribute(b, "Duration", translate("gettextFromC", "Duration"), ", ");
 	write_attribute(b, "DiveMaster", translate("gettextFromC", "Divemaster"), ", ");
 	write_attribute(b, "Buddy", translate("gettextFromC", "Buddy"), ", ");
