@@ -9,17 +9,19 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <time.h>
 #include <unistd.h>
 #include <fcntl.h>
 
+#include "dive.h"
 #include "divesite.h"
 #include "errorhelper.h"
 #include "subsurface-string.h"
+#include "subsurface-time.h"
 #include "trip.h"
 #include "device.h"
 #include "file.h"
 #include "membuffer.h"
+#include "picture.h"
 #include "strndup.h"
 #include "git-access.h"
 #include "qthelper.h"
@@ -179,7 +181,7 @@ static void save_cylinder_info(struct membuffer *b, struct dive *dive)
 	nr = nr_cylinders(dive);
 
 	for (i = 0; i < nr; i++) {
-		cylinder_t *cylinder = dive->cylinder + i;
+		cylinder_t *cylinder = get_cylinder(dive, i);
 		int volume = cylinder->type.size.mliter;
 		const char *description = cylinder->type.description;
 		int use = cylinder->cylinder_use;
@@ -408,7 +410,7 @@ static void save_samples(struct membuffer *b, struct dive *dive, struct divecomp
 	struct sample *s;
 	struct sample dummy = { .bearing.degrees = -1, .ndl.seconds = -1 };
 
-	/* Set up default pressure sensor indexes */
+	/* Set up default pressure sensor indices */
 	o2sensor = legacy_format_o2pressures(dive, dc);
 	if (o2sensor >= 0) {
 		dummy.sensor[0] = !o2sensor;
@@ -490,10 +492,21 @@ void save_one_dive_to_mb(struct membuffer *b, struct dive *dive, bool anonymize)
 		put_format(b, " rating='%d'", dive->rating);
 	if (dive->visibility)
 		put_format(b, " visibility='%d'", dive->visibility);
+	if (dive->wavesize)
+		put_format(b, " wavesize='%d'", dive->wavesize);
+	if (dive->current)
+		put_format(b, " current='%d'", dive->current);
+	if (dive->surge)
+		put_format(b, " surge='%d'", dive->surge);
+	if (dive->chill)
+		put_format(b, " chill='%d'", dive->chill);
+	if (dive->invalid)
+		put_format(b, " invalid='1'");
 	save_tags(b, dive->tag_list);
-	if (dive->dive_site) {
+	if (dive->dive_site)
 		put_format(b, " divesiteid='%8x'", dive->dive_site->uuid);
-	}
+	if (dive->user_salinity)
+		put_salinity(b, dive->user_salinity, " watersalinity='", " g/l'");
 	show_date(b, dive->when);
 	if (surface_pressure.mbar)
 		put_pressure(b, surface_pressure, " airpressure='", " bar'");
@@ -586,7 +599,7 @@ int save_dives(const char *filename)
 	return save_dives_logic(filename, false, false);
 }
 
-void save_dives_buffer(struct membuffer *b, const bool select_only, bool anonymize)
+static void save_dives_buffer(struct membuffer *b, bool select_only, bool anonymize)
 {
 	int i;
 	struct dive *dive;
@@ -608,7 +621,7 @@ void save_dives_buffer(struct membuffer *b, const bool select_only, bool anonymi
 		if (dive_site_is_empty(ds))
 			continue;
 		/* Only write used dive sites when exporting selected dives */
-		if (select_only && !is_dive_site_used(ds, true))
+		if (select_only && !is_dive_site_selected(ds))
 			continue;
 
 		put_format(b, "<site uuid='%8x'", ds->uuid);
@@ -811,20 +824,14 @@ int export_dives_xslt(const char *filename, const bool selected, const int units
 	return res;
 }
 
-void save_dive_sites_buffer(struct membuffer *b, const bool select_only, bool anonymize)
+static void save_dive_sites_buffer(struct membuffer *b, const struct dive_site *sites[], int nr_sites, bool anonymize)
 {
 	int i;
 	put_format(b, "<divesites program='subsurface' version='%d'>\n", DATAFORMAT_VERSION);
 
 	/* save the dive sites */
-	for (i = 0; i < dive_site_table.nr; i++) {
-		struct dive_site *ds = get_dive_site(i, &dive_site_table);
-		/* Don't export empty dive sites */
-		if (dive_site_is_empty(ds))
-			continue;
-		/* Only write used dive sites when exporting selected dives */
-		if (select_only && !is_dive_site_used(ds, true))
-			continue;
+	for (i = 0; i < nr_sites; i++) {
+		const struct dive_site *ds = sites[i];
 
 		put_format(b, "<site uuid='%8x'", ds->uuid);
 		show_utf8_blanked(b, ds->name, " name='", "'", 1, anonymize);
@@ -848,13 +855,13 @@ void save_dive_sites_buffer(struct membuffer *b, const bool select_only, bool an
 	put_format(b, "</divesites>\n");
 }
 
-int save_dive_sites_logic(const char *filename, const bool select_only, bool anonymize)
+int save_dive_sites_logic(const char *filename, const struct dive_site *sites[], int nr_sites, bool anonymize)
 {
 	struct membuffer buf = { 0 };
 	FILE *f;
 	int error = 0;
 
-	save_dive_sites_buffer(&buf, select_only, anonymize);
+	save_dive_sites_buffer(&buf, sites, nr_sites, anonymize);
 
 	if (same_string(filename, "-")) {
 		f = stdout;
