@@ -13,6 +13,8 @@ extern "C" {
 #include <stdbool.h>
 #endif
 
+#define FRACTION(n, x) ((unsigned)(n) / (x)), ((unsigned)(n) % (x))
+
 #define O2_IN_AIR 209 // permille
 #define N2_IN_AIR 781
 #define O2_DENSITY 1331 // mg/Liter
@@ -78,6 +80,8 @@ typedef struct
 	int32_t seconds; // durations up to 34 yrs
 } duration_t;
 
+static const duration_t zero_duration = { 0 };
+
 typedef struct
 {
 	int32_t seconds; // offsets up to +/- 34 yrs
@@ -137,6 +141,8 @@ typedef struct pos {
 	degrees_t lat, lon;
 } location_t;
 
+static const location_t zero_location = { { 0 }, { 0 }};
+
 extern void parse_location(const char *, location_t *);
 
 static inline bool has_location(const location_t *loc)
@@ -271,6 +277,7 @@ static inline int32_t pressure_to_altitude(int32_t pressure)	// pressure in mbar
  * keeps track of those units.
  */
 /* turns out in Win32 PASCAL is defined as a calling convention */
+/* NOTE: these enums are duplicated in mobile-widgets/qmlinterface.h */
 struct units {
 	enum LENGTH {
 		METERS,
