@@ -23,9 +23,6 @@ const char current_system_divelist_default_font[] = "Segoe UI";
 const char *system_divelist_default_font = non_standard_system_divelist_default_font;
 double system_divelist_default_font_size = -1;
 
-void subsurface_user_info(struct user_info *user)
-{ /* Encourage use of at least libgit2-0.20 */ }
-
 extern bool isWin7Or8();
 
 void subsurface_OS_pref_setup(void)
