@@ -9,7 +9,8 @@
 #include "desktop-widgets/modeldelegates.h"
 #include "desktop-widgets/mainwindow.h"
 #include "desktop-widgets/divepicturewidget.h"
-#include "core/display.h"
+#include "core/selection.h"
+#include "core/divefilter.h"
 #include <unistd.h>
 #include <QSettings>
 #include <QKeyEvent>
@@ -19,24 +20,28 @@
 #include <QStandardPaths>
 #include <QMessageBox>
 #include <QHeaderView>
+#include "commands/command.h"
 #include "core/errorhelper.h"
 #include "core/qthelper.h"
 #include "core/trip.h"
-#include "desktop-widgets/command.h"
 #include "desktop-widgets/divelistview.h"
 #include "qt-models/divepicturemodel.h"
 #include "core/metrics.h"
 #include "desktop-widgets/simplewidgets.h"
 #include "desktop-widgets/mapwidget.h"
 
-DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelection(false),
-	currentLayout(DiveTripModelBase::TREE), dontEmitDiveChangedSignal(false), selectionSaved(false),
-	initialColumnWidths(DiveTripModelBase::COLUMNS, 50)	// Set up with default length 50
+DiveListView::DiveListView(QWidget *parent) : QTreeView(parent),
+	currentLayout(DiveTripModelBase::TREE),
+	initialColumnWidths(DiveTripModelBase::COLUMNS, 50),	// Set up with default length 50
+	programmaticalSelectionChange(false)
 {
 	setItemDelegate(new DiveListDelegate(this));
 	setUniformRowHeights(true);
 	setItemDelegateForColumn(DiveTripModelBase::RATING, new StarWidgetsDelegate(this));
-	setModel(MultiFilterSortModel::instance());
+	MultiFilterSortModel *m = MultiFilterSortModel::instance();
+	setModel(m);
+	connect(m, &MultiFilterSortModel::selectionChanged, this, &DiveListView::diveSelectionChanged);
+	connect(m, &MultiFilterSortModel::currentDiveChanged, this, &DiveListView::currentDiveChanged);
 
 	setSortingEnabled(true);
 	setContextMenuPolicy(Qt::DefaultContextMenu);
@@ -45,9 +50,6 @@ DiveListView::DiveListView(QWidget *parent) : QTreeView(parent), mouseClickSelec
 
 	resetModel();
 
-	// Update selection if all selected dives were hidden by filter
-	connect(MultiFilterSortModel::instance(), &MultiFilterSortModel::filterFinished, this, &DiveListView::filterFinished);
-
 	connect(&diveListNotifier, &DiveListNotifier::tripChanged, this, &DiveListView::tripChanged);
 
 	header()->setStretchLastSection(true);
@@ -82,21 +84,24 @@ DiveListView::~DiveListView()
 
 void DiveListView::resetModel()
 {
-	MultiFilterSortModel::instance()->resetModel(currentLayout);
-	// If the model was reset, we have to reconnect the signals and tell
-	// the filter model to update its source model.
-	connect(DiveTripModelBase::instance(), &DiveTripModelBase::selectionChanged, this, &DiveListView::diveSelectionChanged);
-	connect(DiveTripModelBase::instance(), &DiveTripModelBase::newCurrentDive, this, &DiveListView::currentDiveChanged);
+	MultiFilterSortModel *m = MultiFilterSortModel::instance();
+	m->resetModel(currentLayout);
 }
 
 void DiveListView::calculateInitialColumnWidth(int col)
 {
 	const QFontMetrics metrics(defaultModelFont());
+	QString header_txt = MultiFilterSortModel::instance()->headerData(col, Qt::Horizontal, Qt::DisplayRole).toString();
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 11, 0)
 	int em = metrics.width('m');
 	int zw = metrics.width('0');
-
-	QString header_txt = DiveTripModelBase::instance()->headerData(col, Qt::Horizontal, Qt::DisplayRole).toString();
 	int width = metrics.width(header_txt);
+#else // QT 5.11 or newer
+	int em = metrics.horizontalAdvance('m');
+	int zw = metrics.horizontalAdvance('0');
+	int width = metrics.horizontalAdvance(header_txt);
+#endif
 	int sw = 0;
 	switch (col) {
 	case DiveTripModelBase::NR:
@@ -134,7 +139,7 @@ void DiveListView::calculateInitialColumnWidth(int col)
 void DiveListView::setColumnWidths()
 {
 	QSettings settings;
-	backupExpandedRows();
+	std::vector<int> expandedRows = backupExpandedRows();
 	settings.beginGroup("ListWidget");
 	/* if no width are set, use the calculated width for each column;
 	 * for that to work we need to temporarily expand all rows */
@@ -149,7 +154,7 @@ void DiveListView::setColumnWidths()
 			setColumnWidth(i, initialColumnWidths[i]);
 	}
 	settings.endGroup();
-	restoreExpandedRows();
+	restoreExpandedRows(expandedRows);
 	setColumnWidth(lastVisibleColumn(), 10);
 }
 
@@ -164,18 +169,19 @@ int DiveListView::lastVisibleColumn()
 	return lastColumn;
 }
 
-void DiveListView::backupExpandedRows()
+std::vector<int> DiveListView::backupExpandedRows()
 {
-	expandedRows.clear();
+	std::vector<int> expandedRows;
 	for (int i = 0; i < model()->rowCount(); i++)
 		if (isExpanded(model()->index(i, 0)))
 			expandedRows.push_back(i);
+	return expandedRows;
 }
 
-void DiveListView::restoreExpandedRows()
+void DiveListView::restoreExpandedRows(const std::vector<int> &expandedRows)
 {
 	setAnimated(false);
-	Q_FOREACH (const int &i, expandedRows)
+	for (int i: expandedRows)
 		setExpanded(model()->index(i, 0), true);
 	setAnimated(true);
 }
@@ -194,53 +200,46 @@ void DiveListView::reset()
 }
 
 // If items were selected, inform the selection model
-void DiveListView::diveSelectionChanged(const QVector<QModelIndex> &indexes)
+void DiveListView::diveSelectionChanged(const QVector<QModelIndex> &indices)
 {
-	// Since dives are selected dive-by-dive, send only a single signal at the
-	// end, not one for every dive.
-	dontEmitDiveChangedSignal = true;
+	// This is the entry point for programmatical selection changes.
+	// Set a flag so that selection changes are not further processed,
+	// since the core structures were already set.
+	programmaticalSelectionChange = true;
 
 	clearSelection();
-	MultiFilterSortModel *m = MultiFilterSortModel::instance();
-	QItemSelectionModel *s = selectionModel();
-	for (const QModelIndex &index: indexes) {
-		// We have to transform the indices into local indices, since
-		// there might be sorting or filtering in effect.
-		QModelIndex localIndex = m->mapFromSource(index);
+	QItemSelection selection;
+	for (const QModelIndex &index: indices)
+		selection.select(index, index); // Is there a faster way to do this?
+	selectionModel()->select(selection, QItemSelectionModel::Rows | QItemSelectionModel::Select);
 
-		// It might be possible that the item is not shown (filter is
-		// in effect). Then we get an invalid index and should ignore
-		// this selection.
-		if (!localIndex.isValid())
+	// Expand all unexpanded trips
+	std::vector<int> affectedTrips;
+	for (const QModelIndex &index: indices) {
+		if (!index.parent().isValid())
 			continue;
-
-		s->select(localIndex, QItemSelectionModel::Rows | QItemSelectionModel::Select);
-
-		// If an item of a not-yet expanded trip is selected, expand the trip.
-		if (localIndex.parent().isValid() && !isExpanded(localIndex.parent())) {
-			setAnimated(false);
-			expand(localIndex.parent());
-			setAnimated(true);
-		}
+		int row = index.parent().row();
+		if (std::find(affectedTrips.begin(), affectedTrips.end(), row) == affectedTrips.end())
+			affectedTrips.push_back(row);
+	}
+	MultiFilterSortModel *m = MultiFilterSortModel::instance();
+	for (int row: affectedTrips) {
+		QModelIndex idx = m->index(row, 0);
+		expand(idx);
 	}
 
-	dontEmitDiveChangedSignal = false;
-	emit divesSelected();
+	selectionChangeDone();
+	programmaticalSelectionChange = false;
 }
 
 void DiveListView::currentDiveChanged(QModelIndex index)
 {
-	// Transform the index into a local index, since
-	// there might be sorting or filtering in effect.
-	MultiFilterSortModel *m = MultiFilterSortModel::instance();
-	QModelIndex localIndex = m->mapFromSource(index);
-
-	// Then, set the currently activated row.
+	// Set the currently activated row.
 	// Note, we have to use the QItemSelectionModel::Current mode to avoid
 	// changing our selection (in contrast to Qt's documentation, which
 	// instructs to use QItemSelectionModel::NoUpdate, which results in
 	// funny side-effects).
-	selectionModel()->setCurrentIndex(localIndex, QItemSelectionModel::Current);
+	selectionModel()->setCurrentIndex(index, QItemSelectionModel::Current);
 }
 
 // If rows are added, check which of these rows is a trip and expand the first column
@@ -259,47 +258,6 @@ void DiveListView::rowsInserted(const QModelIndex &parent, int start, int end)
 	}
 }
 
-// this only remembers dives that were selected, not trips
-void DiveListView::rememberSelection()
-{
-	selectedDives.clear();
-	QItemSelection selection = selectionModel()->selection();
-	Q_FOREACH (const QModelIndex &index, selection.indexes()) {
-		if (index.column() != 0) // We only care about the dives, so, let's stick to rows and discard columns.
-			continue;
-		struct dive *d = index.data(DiveTripModelBase::DIVE_ROLE).value<struct dive *>();
-		if (d) {
-			selectedDives.insert(d->divetrip, get_divenr(d));
-		} else {
-			struct dive_trip *t = index.data(DiveTripModelBase::TRIP_ROLE).value<dive_trip *>();
-			if (t)
-				selectedDives.insert(t, -1);
-		}
-	}
-	selectionSaved = true;
-}
-
-void DiveListView::restoreSelection()
-{
-	if (!selectionSaved)
-		return;
-
-	selectionSaved = false;
-	QList<int> divesToSelect;
-	Q_FOREACH (dive_trip_t *trip, selectedDives.keys()) {
-		QList<int> divesOnTrip = getDivesInTrip(trip);
-		QList<int> selectedDivesOnTrip = selectedDives.values(trip);
-
-		// Only select trip if all of its dives were selected
-		if(selectedDivesOnTrip.contains(-1)) {
-			selectTrip(trip);
-			selectedDivesOnTrip.removeAll(-1);
-		}
-		divesToSelect += selectedDivesOnTrip;
-	}
-	selectDives(divesToSelect);
-}
-
 // This is a bit ugly: we hook directly into the tripChanged signal to
 // select the trip if it was edited. This feels like a layering violation:
 // Shouldn't the core-layer call us?
@@ -307,14 +265,12 @@ void DiveListView::tripChanged(dive_trip *trip, TripField)
 {
 	// First check if the trip is already selected (and only
 	// this trip, as only then is it displayed). Is so, then do nothing.
-	QList<dive_trip *> selected = selectedTrips();
-	if (selected.size() == 1 && selected[0] == trip)
+	if (singleSelectedTrip() == trip)
 		return;
 
-	dontEmitDiveChangedSignal = true;
 	unselectDives();
-	dontEmitDiveChangedSignal = false;
 	selectTrip(trip);
+	selectionChangeDone();
 }
 
 void DiveListView::selectTrip(dive_trip_t *trip)
@@ -334,27 +290,6 @@ void DiveListView::selectTrip(dive_trip_t *trip)
 	expand(idx);
 }
 
-// this is an odd one - when filtering the dive list the selection status of the trips
-// is kept - but all other selections are lost. That's gets us into rather inconsistent state
-// we call this function which clears the selection state of the trips as well, but does so
-// without updating our internal "->selected" state. So once we called this function we can
-// go back and select those dives that are still visible under the filter and everything
-// works as expected
-void DiveListView::clearTripSelection()
-{
-	// This marks the selection change as being internal - ie. we don't process it further.
-	// TODO: This should probably be done differently.
-	auto marker = diveListNotifier.enterCommand();
-
-	// we want to make sure no trips are selected
-	Q_FOREACH (const QModelIndex &index, selectionModel()->selectedRows()) {
-		dive_trip_t *trip = index.data(DiveTripModelBase::TRIP_ROLE).value<dive_trip *>();
-		if (!trip)
-			continue;
-		selectionModel()->select(index, QItemSelectionModel::Deselect);
-	}
-}
-
 void DiveListView::unselectDives()
 {
 	// make sure we don't try to redraw the dives during the selection change
@@ -373,136 +308,21 @@ void DiveListView::unselectDives()
 	}
 }
 
-QList<dive_trip_t *> DiveListView::selectedTrips()
+// This function returns a trip if there is one selected trip or NULL.
+// Returning all selected trips turned out to be too slow.
+dive_trip_t *DiveListView::singleSelectedTrip()
 {
-	QList<dive_trip_t *> ret;
-	Q_FOREACH (const QModelIndex &index, selectionModel()->selectedRows()) {
-		dive_trip_t *trip = index.data(DiveTripModelBase::TRIP_ROLE).value<dive_trip *>();
-		if (!trip)
+	dive_trip_t *res = nullptr;
+	for (const QModelIndex &index: selectionModel()->selectedRows()) {
+		if (index.parent().isValid())
 			continue;
-		ret.push_back(trip);
-	}
-	return ret;
-}
-
-void DiveListView::selectDive(QModelIndex idx, bool scrollto, bool toggle)
-{
-	if (!idx.isValid())
-		return;
-	QItemSelectionModel::SelectionFlags flags = toggle ? QItemSelectionModel::Toggle : QItemSelectionModel::Select;
-	flags |= QItemSelectionModel::Rows;
-	selectionModel()->setCurrentIndex(idx, flags);
-	if (idx.parent().isValid()) {
-		setAnimated(false);
-		expand(idx.parent());
-		if (scrollto)
-			scrollTo(idx.parent());
-		setAnimated(true);
-	}
-	if (scrollto)
-		scrollTo(idx, PositionAtCenter);
-}
-
-void DiveListView::selectDive(int i, bool scrollto, bool toggle)
-{
-	if (i == -1)
-		return;
-	QAbstractItemModel *m = model();
-	QModelIndexList match = m->match(m->index(0, 0), DiveTripModelBase::DIVE_IDX, i, 2, Qt::MatchRecursive);
-	if (match.isEmpty())
-		return;
-	QModelIndex idx = match.first();
-	selectDive(idx, scrollto, toggle);
-}
-
-void DiveListView::selectDives(const QList<int> &newDiveSelection)
-{
-	int firstInList, newSelection;
-	struct dive *d;
-
-	if (!newDiveSelection.count())
-		return;
-
-	dontEmitDiveChangedSignal = true;
-
-	// First, clear the old selection
-	unselectDives();
-
-	// select the dives, highest index first - this way the oldest of the dives
-	// becomes the selected_dive that we scroll to
-	QList<int> sortedSelection = newDiveSelection;
-	std::sort(sortedSelection.begin(), sortedSelection.end());
-	newSelection = firstInList = sortedSelection.first();
-
-	while (!sortedSelection.isEmpty())
-		selectDive(sortedSelection.takeLast());
-
-	while (!current_dive) {
-		// that can happen if we restored a selection after edit
-		// and the only selected dive is no longer visible because of a filter
-		newSelection--;
-		if (newSelection < 0)
-			newSelection = dive_table.nr - 1;
-		if (newSelection == firstInList)
-			break;
-		if ((d = get_dive(newSelection)) != NULL && !d->hidden_by_filter)
-			selectDive(newSelection);
-	}
-	QAbstractItemModel *m = model();
-	QModelIndexList idxList = m->match(m->index(0, 0), DiveTripModelBase::DIVE_IDX, get_divenr(current_dive), 2, Qt::MatchRecursive);
-	if (!idxList.isEmpty()) {
-		QModelIndex idx = idxList.first();
-		if (idx.parent().isValid())
-			scrollTo(idx.parent());
-		scrollTo(idx);
-	}
-
-	// update the selected-flag for the dive sites.
-	// the actual reloading of the dive sites will be perfomed
-	// by the main-window in response to the divesSelected signal
-	// emitted below.
-	// But don't do this if we are in divesite mode, because then
-	// the dive-site selection is controlled by the filter not
-	// by the selected dives.
-	if (!MultiFilterSortModel::instance()->diveSiteMode()) {
-		QVector<dive_site *> selectedSites;
-		for (int idx: newDiveSelection) {
-			dive *d = get_dive(idx);
-			if (!d)
-				continue;
-			dive_site *ds = d->dive_site;
-			if (ds && !selectedSites.contains(ds))
-				selectedSites.append(ds);
+		if (dive_trip_t *trip = index.data(DiveTripModelBase::TRIP_ROLE).value<dive_trip *>()) {
+			if (res)
+				return nullptr; // More than one
+			res = trip;
 		}
-		MapWidget::instance()->setSelected(selectedSites);
 	}
-
-	// now that everything is up to date, update the widgets
-	emit divesSelected();
-	dontEmitDiveChangedSignal = false;
-	return;
-}
-
-// Get index of first dive. This assumes that trips without dives are never shown.
-// May return an invalid index if no dive is found.
-QModelIndex DiveListView::indexOfFirstDive()
-{
-	// Fetch the first top-level item. If this is a trip, it is supposed to have at least
-	// one child. In that case return the child. Otherwise return the top-level item, which
-	// should be a dive.
-	QAbstractItemModel *m = model();
-	QModelIndex firstDiveOrTrip = m->index(0, 0);
-	if (!firstDiveOrTrip.isValid())
-		return QModelIndex();
-	QModelIndex child = m->index(0, 0, firstDiveOrTrip);
-	return child.isValid() ? child : firstDiveOrTrip;
-}
-
-void DiveListView::selectFirstDive()
-{
-	QModelIndex first = indexOfFirstDive();
-	if (first.isValid())
-		setCurrentIndex(first);
+	return res;
 }
 
 bool DiveListView::eventFilter(QObject *, QEvent *event)
@@ -526,17 +346,15 @@ void DiveListView::sortIndicatorChanged(int i, Qt::SortOrder order)
 	if (currentLayout == newLayout) {
 		sortByColumn(i, order);
 	} else {
-		// clear the model, repopulate with new indexes.
-		rememberSelection();
-		unselectDives();
-		if (currentLayout == DiveTripModelBase::TREE)
-			backupExpandedRows();
+		// clear the model, repopulate with new indices.
+		std::vector<int> expandedRows;
+		if(currentLayout == DiveTripModelBase::TREE)
+			expandedRows = backupExpandedRows();
 		currentLayout = newLayout;
 		resetModel();
 		sortByColumn(i, order);
 		if (newLayout == DiveTripModelBase::TREE)
-			restoreExpandedRows();
-		restoreSelection();
+			restoreExpandedRows(expandedRows);
 	}
 }
 
@@ -550,10 +368,6 @@ void DiveListView::reload()
 {
 	resetModel();
 
-	if (amount_selected && current_dive != NULL)
-		selectDive(get_divenr(current_dive), true);
-	else
-		selectFirstDive();
 	if (selectedIndexes().count()) {
 		QModelIndex curr = selectedIndexes().first();
 		curr = curr.parent().isValid() ? curr.parent() : curr;
@@ -645,9 +459,93 @@ void DiveListView::currentChanged(const QModelIndex &current, const QModelIndex&
 	scrollTo(current);
 }
 
+void DiveListView::mouseReleaseEvent(QMouseEvent *event)
+{
+	// Oy vey. We hook into QTreeView's setSelection() to update the UI
+	// after selection changes. However, there is a case when this doesn't
+	// work: when narrowing the selection. If multiple dives are selected,
+	// and the user clicks on one of them, the selection is unchanged.
+	// Only on mouse-release the selection is changed, but then
+	// setSelection() is not called.
+	// Notably, this happens when the user selects a trip and the clicks
+	// on a dive in the same trip.
+	// To solve this, we hook into the mouseReleseEvent here and detect
+	// selection changes changes by comparing the selection before and after
+	// processing of the event.
+	// We really should find another way to solve this.
+	QModelIndexList selectionBefore = selectionModel()->selectedRows();
+	QTreeView::mouseReleaseEvent(event);
+	QModelIndexList selectionAfter = selectionModel()->selectedRows();
+	if (selectionBefore != selectionAfter)
+		selectionChangeDone();
+}
+
+void DiveListView::keyPressEvent(QKeyEvent *event)
+{
+	// Hook into cursor-up and cursor-down events and update selection if necessary.
+	// See comment in mouseReleaseEvent()
+	if (event->key() != Qt::Key_Down && event->key() != Qt::Key_Up)
+		return QTreeView::keyPressEvent(event);
+	QModelIndexList selectionBefore = selectionModel()->selectedRows();
+	QTreeView::keyPressEvent(event);
+	QModelIndexList selectionAfter = selectionModel()->selectedRows();
+	if (selectionBefore != selectionAfter)
+		selectionChangeDone();
+}
+
+void DiveListView::setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags flags)
+{
+	// We hook into QTreeView's setSelection() to update the UI
+	// after selection changes. However, we must be careful:
+	// when the user clicks on an already selected dive, this is called
+	// with the QItemSelectionModel::NoUpdate flags. In this case, just
+	// route through. Moreover, we get setSelection() calls on every
+	// mouseMove event. To avoid excessive reloading of the UI check
+	// if the selection changed by comparing before and after processing
+	// the selection.
+	if (flags == QItemSelectionModel::NoUpdate)
+		return QTreeView::setSelection(rect, flags);
+	QModelIndexList selectionBefore = selectionModel()->selectedRows();
+	QTreeView::setSelection(rect, flags);
+	QModelIndexList selectionAfter = selectionModel()->selectedRows();
+	if (selectionBefore != selectionAfter)
+		selectionChangeDone();
+}
+
+void DiveListView::selectAll()
+{
+	QTreeView::selectAll();
+	selectionChangeDone();
+}
+
+void DiveListView::selectionChangeDone()
+{
+	// When receiving the divesSelected signal the main window will
+	// instruct the map to update the flags. Thus, make sure that
+	// the selected maps are registered correctly.
+	// But don't do this if we are in divesite mode, because then
+	// the dive-site selection is controlled by the filter not
+	// by the selected dives.
+	if (!DiveFilter::instance()->diveSiteMode()) {
+		// This is truly sad, but taking the list of selected indices and turning them
+		// into dive sites turned out to be unreasonably slow. Therefore, let's access
+		// the core list directly. In my tests, this went down from 700 to 0 ms!
+		QVector<dive_site *> selectedSites;
+		selectedSites.reserve(amount_selected);
+		int i;
+		dive *d;
+		for_each_dive(i, d) {
+			if (d->selected && !d->hidden_by_filter && d->dive_site && !selectedSites.contains(d->dive_site))
+				selectedSites.push_back(d->dive_site);
+		}
+		MapWidget::instance()->setSelected(selectedSites);
+	}
+	emit divesSelected();
+}
+
 void DiveListView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
 {
-	if (diveListNotifier.inCommand()) {
+	if (programmaticalSelectionChange) {
 		// This is a programmatical change of the selection.
 		// Call the QTreeView base function to reflect the selection in the display,
 		// but don't process it any further.
@@ -692,25 +590,6 @@ void DiveListView::selectionChanged(const QItemSelection &selected, const QItemS
 			select_dive(dive);
 		}
 	}
-	if (!dontEmitDiveChangedSignal) {
-		// When receiving the divesSelected signal the main window will
-		// instruct the map to update the flags. Thus, make sure that
-		// the selected maps are registered correctly.
-		// But don't do this if we are in divesite mode, because then
-		// the dive-site selection is controlled by the filter not
-		// by the selected dives.
-		if (!MultiFilterSortModel::instance()->diveSiteMode()) {
-			QVector<dive_site *> selectedSites;
-			for (QModelIndex index: selectionModel()->selection().indexes()) {
-				const QAbstractItemModel *model = index.model();
-				struct dive *dive = model->data(index, DiveTripModelBase::DIVE_ROLE).value<struct dive *>();
-				if (dive && dive->dive_site)
-					selectedSites.push_back(dive->dive_site);
-			}
-			MapWidget::instance()->setSelected(selectedSites);
-		}
-		emit divesSelected();
-	}
 
 	// Display the new, processed, selection
 	QTreeView::selectionChanged(selectionModel()->selection(), newDeselected);
@@ -889,22 +768,12 @@ void DiveListView::addToTrip(int delta)
 
 void DiveListView::markDiveInvalid()
 {
-	int i;
-	struct dive *d = contextMenuIndex.data(DiveTripModelBase::DIVE_ROLE).value<struct dive *>();
-	if (!d)
-		return;
-	for_each_dive (i, d) {
-		if (!d->selected)
-			continue;
-		//TODO: this should be done in the future
-		// now mark the dive invalid... how do we do THAT?
-		// d->invalid = true;
-	}
-	mark_divelist_changed(true);
-	MainWindow::instance()->refreshDisplay();
-	if (prefs.display_invalid_dives == false) {
-		clearSelection();
-	}
+	Command::editInvalid(true, false);
+}
+
+void DiveListView::markDiveValid()
+{
+	Command::editInvalid(false, false);
 }
 
 void DiveListView::deleteDive()
@@ -948,18 +817,18 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 			}
 		}
 		if (needs_expand)
-			popup.addAction(tr("Expand all"), this, SLOT(expandAll()));
+			popup.addAction(tr("Expand all"), this, &QTreeView::expandAll);
 		if (needs_collapse)
-			popup.addAction(tr("Collapse all"), this, SLOT(collapseAll()));
+			popup.addAction(tr("Collapse all"), this, &QTreeView::collapseAll);
 
 		// verify if there`s a need for collapse others
 		if (expanded_nodes > 1)
-			collapseAction = popup.addAction(tr("Collapse others"), this, SLOT(collapseAll()));
+			collapseAction = popup.addAction(tr("Collapse others"), this, &QTreeView::collapseAll);
 
 
 		if (d) {
-			popup.addAction(tr("Remove dive(s) from trip"), this, SLOT(removeFromTrip()));
-			popup.addAction(tr("Create new trip above"), this, SLOT(newTripAbove()));
+			popup.addAction(tr("Remove dive(s) from trip"), this, &DiveListView::removeFromTrip);
+			popup.addAction(tr("Create new trip above"), this, &DiveListView::newTripAbove);
 			if (!d->divetrip) {
 				struct dive *top = d;
 				struct dive *bottom = d;
@@ -974,30 +843,31 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 					}
 				}
 				if (is_trip_before_after(top, (currentOrder == Qt::AscendingOrder)))
-					popup.addAction(tr("Add dive(s) to trip immediately above"), this, SLOT(addToTripAbove()));
+					popup.addAction(tr("Add dive(s) to trip immediately above"), this, &DiveListView::addToTripAbove);
 				if (is_trip_before_after(bottom, (currentOrder == Qt::DescendingOrder)))
-					popup.addAction(tr("Add dive(s) to trip immediately below"), this, SLOT(addToTripBelow()));
+					popup.addAction(tr("Add dive(s) to trip immediately below"), this, &DiveListView::addToTripBelow);
 			}
 		}
 		if (trip) {
-			popup.addAction(tr("Merge trip with trip above"), this, SLOT(mergeTripAbove()));
-			popup.addAction(tr("Merge trip with trip below"), this, SLOT(mergeTripBelow()));
+			popup.addAction(tr("Merge trip with trip above"), this, &DiveListView::mergeTripAbove);
+			popup.addAction(tr("Merge trip with trip below"), this, &DiveListView::mergeTripBelow);
 		}
 	}
 	if (d) {
-		popup.addAction(tr("Delete dive(s)"), this, SLOT(deleteDive()));
-#if 0
-		popup.addAction(tr("Mark dive(s) invalid", this, SLOT(markDiveInvalid())));
-#endif
+		popup.addAction(tr("Delete dive(s)"), this, &DiveListView::deleteDive);
+		if (d->invalid)
+			popup.addAction(tr("Mark dive(s) valid"), this, &DiveListView::markDiveValid);
+		else
+			popup.addAction(tr("Mark dive(s) invalid"), this, &DiveListView::markDiveInvalid);
 	}
 	if (amount_selected > 1 && consecutive_selected())
-		popup.addAction(tr("Merge selected dives"), this, SLOT(mergeDives()));
+		popup.addAction(tr("Merge selected dives"), this, &DiveListView::mergeDives);
 	if (amount_selected >= 1) {
-		popup.addAction(tr("Renumber dive(s)"), this, SLOT(renumberDives()));
-		popup.addAction(tr("Shift dive times"), this, SLOT(shiftTimes()));
-		popup.addAction(tr("Split selected dives"), this, SLOT(splitDives()));
-		popup.addAction(tr("Load media from file(s)"), this, SLOT(loadImages()));
-		popup.addAction(tr("Load media from web"), this, SLOT(loadWebImages()));
+		popup.addAction(tr("Renumber dive(s)"), this, &DiveListView::renumberDives);
+		popup.addAction(tr("Shift dive times"), this, &DiveListView::shiftTimes);
+		popup.addAction(tr("Split selected dives"), this, &DiveListView::splitDives);
+		popup.addAction(tr("Load media from file(s)"), this, &DiveListView::loadImages);
+		popup.addAction(tr("Load media from web"), this, &DiveListView::loadWebImages);
 	}
 
 	// "collapse all" really closes all trips,
@@ -1005,7 +875,6 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 	QAction *actionTaken = popup.exec(event->globalPos());
 	if (actionTaken == collapseAction && collapseAction) {
 		this->setAnimated(false);
-		selectDive(get_divenr(current_dive), true);
 		scrollTo(selectedIndexes().first());
 		this->setAnimated(true);
 	}
@@ -1100,19 +969,6 @@ void DiveListView::loadImageFromURL(QUrl url)
 	}
 }
 
-void DiveListView::filterFinished()
-{
-	// first make sure the trips are no longer shown as selected
-	// (but without updating the selection state of the dives... this just cleans
-	// up an oddity in the filter handling)
-	clearTripSelection();
-
-	// If there are no more selected dives, select the first visible dive
-	if (!selectionModel()->hasSelection())
-		selectFirstDive();
-	emit divesSelected();
-}
-
 QString DiveListView::lastUsedImageDir()
 {
 	QSettings settings;
