@@ -16,44 +16,33 @@
 #include <QLineEdit>
 #include <QNetworkAccessManager>
 #include "qt-models/divetripmodel.h"
-#include "core/subsurface-qt/DiveListNotifier.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 
 class DiveListView : public QTreeView {
 	Q_OBJECT
 public:
 	DiveListView(QWidget *parent = 0);
 	~DiveListView();
-	void mouseDoubleClickEvent(QMouseEvent * event);
-	void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
-	void currentChanged(const QModelIndex &current, const QModelIndex &previous);
 	void setSortOrder(int i, Qt::SortOrder order); // Call to set sort order
 	void reload(); // Call to reload model data
-	bool eventFilter(QObject *, QEvent *);
-	void unselectDives();
-	void clearTripSelection();
-	void selectDive(QModelIndex index, bool scrollto = false, bool toggle = false);
-	void selectDive(int dive_table_idx, bool scrollto = false, bool toggle = false);
-	void selectDives(const QList<int> &newDiveSelection);
-	void selectFirstDive();
-	QModelIndex indexOfFirstDive();
-	void rememberSelection();
-	void restoreSelection();
-	void contextMenuEvent(QContextMenuEvent *event);
-	QList<dive_trip *> selectedTrips();
+	dive_trip *singleSelectedTrip();
 	static QString lastUsedImageDir();
 	static void updateLastUsedImageDir(const QString &s);
+	void loadImages();
+	void loadWebImages();
 signals:
 	void divesSelected();
 public
 slots:
-	void toggleColumnVisibilityByIndex();
 	void reloadHeaderActions();
+private
+slots:
+	void toggleColumnVisibilityByIndex();
 	void sortIndicatorChanged(int index, Qt::SortOrder order);
 	void removeFromTrip();
 	void deleteDive();
 	void markDiveInvalid();
-	void rowsInserted(const QModelIndex &parent, int start, int end) override;
-	void reset() override;
+	void markDiveValid();
 	void mergeTripAbove();
 	void mergeTripBelow();
 	void newTripAbove();
@@ -63,30 +52,30 @@ slots:
 	void splitDives();
 	void renumberDives();
 	void shiftTimes();
-	void loadImages();
-	void loadWebImages();
-	void diveSelectionChanged(const QVector<QModelIndex> &indexes);
+	void diveSelectionChanged(const QVector<QModelIndex> &indices);
 	void currentDiveChanged(QModelIndex index);
-	void filterFinished();
 	void tripChanged(dive_trip *trip, TripField);
 private:
-	bool mouseClickSelection;
-	QList<int> expandedRows;
+	void rowsInserted(const QModelIndex &parent, int start, int end) override;
+	void reset() override;
+	void setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags flags) override;
+	void unselectDives();
+	void mouseReleaseEvent(QMouseEvent *event) override;
+	void keyPressEvent(QKeyEvent *event) override;
+	void selectAll() override;
+	void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
+	void selectionChangeDone();
 	DiveTripModelBase::Layout currentLayout;
 	QModelIndex contextMenuIndex;
-	bool dontEmitDiveChangedSignal;
-	bool selectionSaved;
 	// Remember the initial column widths, to avoid writing unchanged widths to the settings
 	QVector<int> initialColumnWidths;
 
-	/* if dive_trip_t is null, there's no problem. */
-	QMultiHash<dive_trip *, int> selectedDives;
 	void resetModel();	// Call after model changed
 	void merge_trip(const QModelIndex &a, const int offset);
 	void setColumnWidths();
 	void calculateInitialColumnWidth(int col);
-	void backupExpandedRows();
-	void restoreExpandedRows();
+	std::vector<int> backupExpandedRows();
+	void restoreExpandedRows(const std::vector<int> &);
 	int lastVisibleColumn();
 	void selectTrip(dive_trip *trip);
 	void updateLastImageTimeOffset(int offset);
@@ -94,7 +83,12 @@ private:
 	void addToTrip(int delta);
 	void matchImagesToDives(QStringList fileNames);
 	void loadImageFromURL(QUrl url);
+	bool eventFilter(QObject *, QEvent *) override;
+	void mouseDoubleClickEvent(QMouseEvent *event) override;
+	void contextMenuEvent(QContextMenuEvent *event) override;
+	void currentChanged(const QModelIndex &current, const QModelIndex &previous) override;
 	QNetworkAccessManager manager;
+	bool programmaticalSelectionChange;
 };
 
 #endif // DIVELISTVIEW_H
