@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "desktop-widgets/divelogimportdialog.h"
 #include "desktop-widgets/mainwindow.h"
-#include "desktop-widgets/command.h"
+#include "commands/command.h"
 #include "core/color.h"
 #include "ui_divelogimportdialog.h"
 #include <QShortcut>
@@ -362,7 +362,7 @@ DiveLogImportDialog::DiveLogImportDialog(QStringList fn, QWidget *parent) : QDia
 	hw = "";
 	txtLog = false;
 
-	/* Add indexes of XSLTs requiring special handling to the list */
+	/* Add indices of XSLTs requiring special handling to the list */
 	specialCSV << SENSUS;
 	specialCSV << SUBSURFACE;
 	specialCSV << DL7;
@@ -904,9 +904,9 @@ int DiveLogImportDialog::parseTxtHeader(QString fileName, char **params, int pnr
 
 void DiveLogImportDialog::on_buttonBox_accepted()
 {
-	struct dive_table table = { 0 };
-	struct trip_table trips = { 0 };
-	struct dive_site_table sites = { 0 };
+	struct dive_table table = empty_dive_table;
+	struct trip_table trips = empty_trip_table;
+	struct dive_site_table sites = empty_dive_site_table;
 	QStringList r = resultModel->result();
 	if (ui->knownImports->currentText() != "Manual import") {
 		for (int i = 0; i < fileNames.size(); ++i) {
@@ -916,7 +916,7 @@ void DiveLogImportDialog::on_buttonBox_accepted()
 				QPair<QString, QString> pair = poseidonFileNames(fileNames[i]);
 				parse_txt_file(qPrintable(pair.second), qPrintable(pair.first), &table, &trips, &sites);
 			} else {
-				char *params[49];
+				char *params[50];
 				int pnr = 0;
 
 				QRegExp apdRe("^.*[/\\][0-9a-zA-Z]*_([0-9]{6})_([0-9]{6})\\.apd");
