@@ -7,6 +7,7 @@
 #include "core/units.h"
 #include "core/settings/qPrefDivePlanner.h"
 #include "core/gettextfromc.h"
+#include "backend-shared/plannershared.h"
 
 #include "qt-models/cylindermodel.h"
 #include "qt-models/models.h"
@@ -23,10 +24,6 @@
 #define MAX_DEPTH M_OR_FT(150, 450)
 #define MIN_DEPTH M_OR_FT(20, 60)
 
-#define UNIT_FACTOR ((prefs.units.length == units::METERS) ? 1000.0 / 60.0 : feet_to_mm(1.0) / 60.0)
-
-static DivePlannerPointsModel* plannerModel = DivePlannerPointsModel::instance();
-
 DiveHandler::DiveHandler() : QGraphicsEllipseItem()
 {
 	setRect(-5, -5, 10, 10);
@@ -47,6 +44,7 @@ void DiveHandler::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 	QMenu m;
 	// Don't have a gas selection for the last point
 	emit released();
+	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 	QModelIndex index = plannerModel->index(parentIndex(), DivePlannerPointsModel::GAS);
 	if (index.sibling(index.row() + 1, index.column()).isValid()) {
 		GasSelectionModel *model = GasSelectionModel::instance();
@@ -56,14 +54,14 @@ void DiveHandler::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 			QAction *action = new QAction(&m);
 			action->setText(model->data(model->index(i, 0), Qt::DisplayRole).toString());
 			action->setData(i);
-			connect(action, SIGNAL(triggered(bool)), this, SLOT(changeGas()));
+			connect(action, &QAction::triggered, this, &DiveHandler::changeGas);
 			m.addAction(action);
 		}
 	}
 	// don't allow removing the last point
 	if (plannerModel->rowCount() > 1) {
 		m.addSeparator();
-		m.addAction(gettextFromC::tr("Remove this point"), this, SLOT(selfRemove()));
+		m.addAction(gettextFromC::tr("Remove this point"), this, &DiveHandler::selfRemove);
 		m.exec(event->screenPos());
 	}
 }
@@ -78,6 +76,7 @@ void DiveHandler::selfRemove()
 void DiveHandler::changeGas()
 {
 	QAction *action = qobject_cast<QAction *>(sender());
+	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 	QModelIndex index = plannerModel->index(parentIndex(), DivePlannerPointsModel::GAS);
 	plannerModel->gasChange(index.sibling(index.row() + 1, index.column()), action->data().toInt());
 }
@@ -110,16 +109,21 @@ void DiveHandler::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 
 DivePlannerWidget::DivePlannerWidget(QWidget *parent, Qt::WindowFlags f) : QWidget(parent, f)
 {
+	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
+	CylindersModel *cylinders = DivePlannerPointsModel::instance()->cylindersModel();
+
 	ui.setupUi(this);
 	ui.dateEdit->setDisplayFormat(prefs.date_format);
 	ui.tableWidget->setTitle(tr("Dive planner points"));
 	ui.tableWidget->setModel(plannerModel);
+	connect(ui.tableWidget, &TableView::itemClicked, plannerModel, &DivePlannerPointsModel::remove);
 	plannerModel->setRecalc(true);
 	ui.tableWidget->view()->setItemDelegateForColumn(DivePlannerPointsModel::GAS, new AirTypesDelegate(this));
 	ui.tableWidget->view()->setItemDelegateForColumn(DivePlannerPointsModel::DIVEMODE, new DiveTypesDelegate(this));
 	ui.cylinderTableWidget->setTitle(tr("Available gases"));
 	ui.cylinderTableWidget->setBtnToolTip(tr("Add cylinder"));
-	ui.cylinderTableWidget->setModel(CylindersModel::instance());
+	ui.cylinderTableWidget->setModel(cylinders);
+	connect(ui.cylinderTableWidget, &TableView::itemClicked, cylinders, &CylindersModel::remove);
 	ui.waterType->setItemData(0, FRESHWATER_SALINITY);
 	ui.waterType->setItemData(1, SEAWATER_SALINITY);
 	ui.waterType->setItemData(2, EN13319_SALINITY);
@@ -130,42 +134,39 @@ DivePlannerWidget::DivePlannerWidget(QWidget *parent, Qt::WindowFlags f) : QWidg
 	view->setColumnHidden(CylindersModel::END, true);
 	view->setColumnHidden(CylindersModel::DEPTH, false);
 	view->setItemDelegateForColumn(CylindersModel::TYPE, new TankInfoDelegate(this));
-	connect(ui.cylinderTableWidget, SIGNAL(addButtonClicked()), plannerModel, SLOT(addCylinder_clicked()));
+	connect(ui.cylinderTableWidget, &TableView::addButtonClicked, plannerModel, &DivePlannerPointsModel::addCylinder_clicked);
+
+	// addStop actually accept a call with no parameters, due to default parameters, but the connect() syntax without SIGNAL/SLOT
+	// does not understand default parameters and causes errors to be thrown.
+	// Continue to use old syntax, to avoid problems.
 	connect(ui.tableWidget, SIGNAL(addButtonClicked()), plannerModel, SLOT(addStop()));
 
-	connect(CylindersModel::instance(), SIGNAL(dataChanged(QModelIndex, QModelIndex)),
-		GasSelectionModel::instance(), SLOT(repopulate()));
-	connect(CylindersModel::instance(), SIGNAL(rowsInserted(QModelIndex, int, int)),
-		GasSelectionModel::instance(), SLOT(repopulate()));
-	connect(CylindersModel::instance(), SIGNAL(rowsRemoved(QModelIndex, int, int)),
-		GasSelectionModel::instance(), SLOT(repopulate()));
-	connect(CylindersModel::instance(), SIGNAL(dataChanged(QModelIndex, QModelIndex)),
-		plannerModel, SLOT(emitDataChanged()));
-	connect(CylindersModel::instance(), SIGNAL(dataChanged(QModelIndex, QModelIndex)),
-		plannerModel, SIGNAL(cylinderModelEdited()));
-	connect(CylindersModel::instance(), SIGNAL(rowsInserted(QModelIndex, int, int)),
-		plannerModel, SIGNAL(cylinderModelEdited()));
-	connect(CylindersModel::instance(), SIGNAL(rowsRemoved(QModelIndex, int, int)),
-		plannerModel, SIGNAL(cylinderModelEdited()));
-	connect(plannerModel, SIGNAL(calculatedPlanNotes()), MainWindow::instance(), SLOT(setPlanNotes()));
+	connect(cylinders, &CylindersModel::dataChanged, GasSelectionModel::instance(), &GasSelectionModel::repopulate);
+	connect(cylinders, &CylindersModel::rowsInserted, GasSelectionModel::instance(), &GasSelectionModel::repopulate);
+	connect(cylinders, &CylindersModel::rowsRemoved, GasSelectionModel::instance(), &GasSelectionModel::repopulate);
+	connect(cylinders, &CylindersModel::dataChanged, plannerModel, &DivePlannerPointsModel::emitDataChanged);
+	connect(cylinders, &CylindersModel::dataChanged, plannerModel, &DivePlannerPointsModel::cylinderModelEdited);
+	connect(cylinders, &CylindersModel::rowsInserted, plannerModel, &DivePlannerPointsModel::cylinderModelEdited);
+	connect(cylinders, &CylindersModel::rowsRemoved, plannerModel, &DivePlannerPointsModel::cylinderModelEdited);
+	connect(plannerModel, &DivePlannerPointsModel::calculatedPlanNotes, MainWindow::instance(), &MainWindow::setPlanNotes);
 
 
 	ui.tableWidget->setBtnToolTip(tr("Add dive data point"));
-	connect(ui.startTime, SIGNAL(timeChanged(QTime)), plannerModel, SLOT(setStartTime(QTime)));
-	connect(ui.dateEdit, SIGNAL(dateChanged(QDate)), plannerModel, SLOT(setStartDate(QDate)));
-	connect(ui.ATMPressure, SIGNAL(valueChanged(int)), this, SLOT(atmPressureChanged(int)));
-	connect(ui.atmHeight, SIGNAL(valueChanged(int)), this, SLOT(heightChanged(int)));
-	connect(ui.waterType, SIGNAL(currentIndexChanged(int)), this, SLOT(waterTypeChanged(int)));
-	connect(ui.customSalinity, SIGNAL(valueChanged(double)), this, SLOT(customSalinityChanged(double)));
-	connect(plannerModel, SIGNAL(startTimeChanged(QDateTime)), this, SLOT(setupStartTime(QDateTime)));
+	connect(ui.startTime, &QDateEdit::timeChanged, plannerModel, &DivePlannerPointsModel::setStartTime);
+	connect(ui.dateEdit, &QDateEdit::dateChanged, plannerModel, &DivePlannerPointsModel::setStartDate);
+	connect(ui.ATMPressure, QOverload<int>::of(&QSpinBox::valueChanged), this, &DivePlannerWidget::atmPressureChanged);
+	connect(ui.atmHeight, QOverload<int>::of(&QSpinBox::valueChanged), this, &DivePlannerWidget::heightChanged);
+	connect(ui.waterType, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &DivePlannerWidget::waterTypeChanged);
+	connect(ui.customSalinity, QOverload<double>::of(&QDoubleSpinBox::valueChanged), this, &DivePlannerWidget::customSalinityChanged);
+	connect(plannerModel, &DivePlannerPointsModel::startTimeChanged, this, &DivePlannerWidget::setupStartTime);
 
 	// Creating (and canceling) the plan
 	replanButton = ui.buttonBox->addButton(tr("Save new"), QDialogButtonBox::ActionRole);
-	connect(replanButton, SIGNAL(clicked()), plannerModel, SLOT(saveDuplicatePlan()));
-	connect(ui.buttonBox, SIGNAL(accepted()), plannerModel, SLOT(savePlan()));
-	connect(ui.buttonBox, SIGNAL(rejected()), plannerModel, SLOT(cancelPlan()));
+	connect(replanButton, &QAbstractButton::clicked, plannerModel, &DivePlannerPointsModel::saveDuplicatePlan);
+	connect(ui.buttonBox, &QDialogButtonBox::accepted, plannerModel, &DivePlannerPointsModel::savePlan);
+	connect(ui.buttonBox, &QDialogButtonBox::rejected, plannerModel, &DivePlannerPointsModel::cancelPlan);
 	QShortcut *closeKey = new QShortcut(QKeySequence(Qt::Key_Escape), this);
-	connect(closeKey, SIGNAL(activated()), plannerModel, SLOT(cancelPlan()));
+	connect(closeKey, &QShortcut::activated, plannerModel, &DivePlannerPointsModel::cancelPlan);
 
 	// This makes shure the spinbox gets a setMinimum(0) on it so we can't have negative time or depth.
 	ui.tableWidget->view()->setItemDelegateForColumn(DivePlannerPointsModel::DEPTH, new SpinBoxDelegate(0, INT_MAX, 1, this));
@@ -220,7 +221,7 @@ void DivePlannerWidget::setSalinity(int salinity)
 		ui.customSalinity->setEnabled(true);
 		ui.customSalinity->setValue(salinity / 10000.0);
 	}
-	plannerModel->setSalinity(salinity);
+	DivePlannerPointsModel::instance()->setSalinity(salinity);
 }
 
 void DivePlannerWidget::settingsChanged()
@@ -236,13 +237,13 @@ void DivePlannerWidget::settingsChanged()
 		ui.atmHeight->setMaximum(3000);
 	}
 	ui.atmHeight->blockSignals(true);
-	ui.atmHeight->setValue((int) get_depth_units((int) pressure_to_altitude(plannerModel->getSurfacePressure()), NULL,NULL));
+	ui.atmHeight->setValue((int) get_depth_units((int) pressure_to_altitude(DivePlannerPointsModel::instance()->getSurfacePressure()), NULL,NULL));
 	ui.atmHeight->blockSignals(false);
 }
 
 void DivePlannerWidget::atmPressureChanged(const int pressure)
 {
-	plannerModel->setSurfacePressure(pressure);
+	DivePlannerPointsModel::instance()->setSurfacePressure(pressure);
 	ui.atmHeight->blockSignals(true);
 	ui.atmHeight->setValue((int) get_depth_units((int) pressure_to_altitude(pressure), NULL,NULL));
 	ui.atmHeight->blockSignals(false);
@@ -254,7 +255,7 @@ void DivePlannerWidget::heightChanged(const int height)
 	ui.ATMPressure->blockSignals(true);
 	ui.ATMPressure->setValue(pressure);
 	ui.ATMPressure->blockSignals(false);
-	plannerModel->setSurfacePressure(pressure);
+	DivePlannerPointsModel::instance()->setSurfacePressure(pressure);
 }
 
 void DivePlannerWidget::waterTypeUpdateTexts()
@@ -275,7 +276,7 @@ void DivePlannerWidget::waterTypeChanged(const int index)
 {
 	ui.customSalinity->setEnabled(index == ui.waterType->count() - 1);
 	ui.customSalinity->setValue(ui.waterType->itemData(index).toInt() / 10000.0);
-	plannerModel->setSalinity(ui.waterType->itemData(index).toInt());
+	DivePlannerPointsModel::instance()->setSalinity(ui.waterType->itemData(index).toInt());
 }
 
 void DivePlannerWidget::customSalinityChanged(double density)
@@ -283,20 +284,10 @@ void DivePlannerWidget::customSalinityChanged(double density)
 	if (ui.customSalinity->isEnabled()) {
 		int newSalinity = (int)(density * 10000.0);
 		ui.waterType->setItemData(ui.waterType->count() - 1, newSalinity);
-		plannerModel->setSalinity(newSalinity);
+		DivePlannerPointsModel::instance()->setSalinity(newSalinity);
 	}
 }
 
-void PlannerSettingsWidget::bottomSacChanged(const double bottomSac)
-{
-	plannerModel->setBottomSac(bottomSac);
-}
-
-void PlannerSettingsWidget::decoSacChanged(const double decosac)
-{
-	plannerModel->setDecoSac(decosac);
-}
-
 void PlannerSettingsWidget::disableDecoElements(int mode)
 {
 	if (mode == RECREATIONAL) {
@@ -365,7 +356,7 @@ void PlannerSettingsWidget::disableDecoElements(int mode)
 		ui.label_min_switch_duration->setDisabled(false);
 		ui.sacfactor->setDisabled(false);
 		ui.problemsolvingtime->setDisabled(false);
-		ui.sacfactor->setValue(prefs.sacfactor / 100.0);
+		ui.sacfactor->setValue(PlannerShared::sacfactor());
 		ui.problemsolvingtime->setValue(prefs.problemsolvingtime);
 		ui.display_variations->setDisabled(false);
 	}
@@ -400,7 +391,7 @@ void PlannerSettingsWidget::disableDecoElements(int mode)
 		ui.label_min_switch_duration->setDisabled(false);
 		ui.sacfactor->setDisabled(false);
 		ui.problemsolvingtime->setDisabled(false);
-		ui.sacfactor->setValue(prefs.sacfactor / 100.0);
+		ui.sacfactor->setValue(PlannerShared::sacfactor());
 		ui.problemsolvingtime->setValue(prefs.problemsolvingtime);
 		ui.display_variations->setDisabled(false);
 	}
@@ -433,6 +424,7 @@ PlannerSettingsWidget::PlannerSettingsWidget(QWidget *parent, Qt::WindowFlags f)
 {
 	ui.setupUi(this);
 
+	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 	plannerModel->getDiveplan().bottomsac = prefs.bottomsac;
 	plannerModel->getDiveplan().decosac = prefs.decosac;
 
@@ -444,17 +436,18 @@ PlannerSettingsWidget::PlannerSettingsWidget(QWidget *parent, Qt::WindowFlags f)
 	ui.display_transitions->setChecked(prefs.display_transitions);
 	ui.display_variations->setChecked(prefs.display_variations);
 	ui.safetystop->setChecked(prefs.safetystop);
-	ui.sacfactor->setValue(prefs.sacfactor / 100.0);
+	ui.sacfactor->setValue(PlannerShared::sacfactor());
 	ui.problemsolvingtime->setValue(prefs.problemsolvingtime);
-	ui.bottompo2->setValue(prefs.bottompo2 / 1000.0);
-	ui.decopo2->setValue(prefs.decopo2 / 1000.0);
+	ui.bottompo2->setValue(PlannerShared::bottompo2());
+	ui.decopo2->setValue(PlannerShared::decopo2());
 	ui.backgasBreaks->setChecked(prefs.doo2breaks);
-	setBailout(false);
+	PlannerShared::set_dobailout(false);
 	setBailoutVisibility(false);
+	ui.o2narcotic->setChecked(prefs.o2narcotic);
 	ui.drop_stone_mode->setChecked(prefs.drop_stone_mode);
 	ui.switch_at_req_stop->setChecked(prefs.switch_at_req_stop);
-	ui.min_switch_duration->setValue(prefs.min_switch_duration / 60);
-	ui.surface_segment->setValue(prefs.surface_segment / 60);
+	ui.min_switch_duration->setValue(PlannerShared::min_switch_duration());
+	ui.surface_segment->setValue(PlannerShared::surface_segment());
 	ui.recreational_deco->setChecked(prefs.planner_deco_mode == RECREATIONAL);
 	ui.buehlmann_deco->setChecked(prefs.planner_deco_mode == BUEHLMANN);
 	ui.vpmb_deco->setChecked(prefs.planner_deco_mode == VPMB);
@@ -466,58 +459,47 @@ PlannerSettingsWidget::PlannerSettingsWidget(QWidget *parent, Qt::WindowFlags f)
 		rebreather_modes.append(gettextFromC::tr(divemode_text_ui[i]));
 	ui.rebreathermode->insertItems(0, rebreather_modes);
 
-	modeMapper = new QSignalMapper(this);
-	connect(modeMapper, SIGNAL(mapped(int)) , plannerModel, SLOT(setDecoMode(int)));
-	modeMapper->setMapping(ui.recreational_deco, int(RECREATIONAL));
-	modeMapper->setMapping(ui.buehlmann_deco, int(BUEHLMANN));
-	modeMapper->setMapping(ui.vpmb_deco, int(VPMB));
+	connect(ui.recreational_deco, &QAbstractButton::clicked, [] { PlannerShared::set_planner_deco_mode(RECREATIONAL); });
+	connect(ui.buehlmann_deco, &QAbstractButton::clicked, [] { PlannerShared::set_planner_deco_mode(BUEHLMANN); });
+	connect(ui.vpmb_deco, &QAbstractButton::clicked, [] { PlannerShared::set_planner_deco_mode(VPMB); });
 
-	connect(ui.recreational_deco, SIGNAL(clicked()), modeMapper, SLOT(map()));
-	connect(ui.buehlmann_deco, SIGNAL(clicked()), modeMapper, SLOT(map()));
-	connect(ui.vpmb_deco, SIGNAL(clicked()), modeMapper, SLOT(map()));
+	connect(ui.lastStop, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setLastStop6m);
+	connect(ui.lastStop, &QAbstractButton::toggled, this, &PlannerSettingsWidget::disableBackgasBreaks);
+	connect(ui.verbatim_plan, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setVerbatim);
+	connect(ui.display_duration, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setDisplayDuration);
+	connect(ui.display_runtime, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setDisplayRuntime);
+	connect(ui.display_transitions, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setDisplayTransitions);
+	connect(ui.safetystop, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setSafetyStop);
+	connect(ui.reserve_gas, QOverload<int>::of(&QSpinBox::valueChanged), &PlannerShared::set_reserve_gas);
+	connect(ui.ascRate75, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setAscrate75Display);
+	connect(ui.ascRate50, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setAscrate50Display);
+	connect(ui.ascRateStops, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setAscratestopsDisplay);
+	connect(ui.ascRateLast6m, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setAscratelast6mDisplay);
+	connect(ui.descRate, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setDescrateDisplay);
+	connect(ui.drop_stone_mode, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setDropStoneMode);
+	connect(ui.gfhigh, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setGFHigh);
+	connect(ui.gflow, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setGFLow);
+	connect(ui.vpmb_conservatism, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setVpmbConservatism);
+	connect(ui.backgasBreaks, &QAbstractButton::toggled, this, &PlannerSettingsWidget::setBackgasBreaks);
+	connect(ui.bailout, &QAbstractButton::toggled, &PlannerShared::set_dobailout);
+	connect(ui.o2narcotic, &QAbstractButton::toggled, &PlannerShared::set_o2narcotic);
+	connect(ui.switch_at_req_stop, &QAbstractButton::toggled, plannerModel, &DivePlannerPointsModel::setSwitchAtReqStop);
+	connect(ui.min_switch_duration, QOverload<int>::of(&QSpinBox::valueChanged), &PlannerShared::set_min_switch_duration);
+	connect(ui.surface_segment, QOverload<int>::of(&QSpinBox::valueChanged), &PlannerShared::set_surface_segment);
+	connect(ui.rebreathermode, QOverload<int>::of(&QComboBox::currentIndexChanged), plannerModel, &DivePlannerPointsModel::setRebreatherMode);
+	connect(ui.rebreathermode, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &PlannerSettingsWidget::setBailoutVisibility);
 
-	connect(ui.lastStop, SIGNAL(toggled(bool)), plannerModel, SLOT(setLastStop6m(bool)));
-	connect(ui.lastStop, SIGNAL(toggled(bool)), this, SLOT(disableBackgasBreaks(bool)));
-	connect(ui.verbatim_plan, SIGNAL(toggled(bool)), plannerModel, SLOT(setVerbatim(bool)));
-	connect(ui.display_duration, SIGNAL(toggled(bool)), plannerModel, SLOT(setDisplayDuration(bool)));
-	connect(ui.display_runtime, SIGNAL(toggled(bool)), plannerModel, SLOT(setDisplayRuntime(bool)));
-	connect(ui.display_transitions, SIGNAL(toggled(bool)), plannerModel, SLOT(setDisplayTransitions(bool)));
-	connect(ui.display_variations, SIGNAL(toggled(bool)), plannerModel, SLOT(setDisplayVariations(bool)));
-	connect(ui.safetystop, SIGNAL(toggled(bool)), plannerModel, SLOT(setSafetyStop(bool)));
-	connect(ui.reserve_gas, SIGNAL(valueChanged(int)), plannerModel, SLOT(setReserveGas(int)));
-	connect(ui.ascRate75, SIGNAL(valueChanged(int)), plannerModel, SLOT(setAscrate75(int)));
-	connect(ui.ascRate50, SIGNAL(valueChanged(int)), plannerModel, SLOT(setAscrate50(int)));
-	connect(ui.ascRateStops, SIGNAL(valueChanged(int)), plannerModel, SLOT(setAscratestops(int)));
-	connect(ui.ascRateLast6m, SIGNAL(valueChanged(int)), plannerModel, SLOT(setAscratelast6m(int)));
-	connect(ui.descRate, SIGNAL(valueChanged(int)), plannerModel, SLOT(setDescrate(int)));
-	connect(ui.drop_stone_mode, SIGNAL(toggled(bool)), plannerModel, SLOT(setDropStoneMode(bool)));
-	connect(ui.gfhigh, SIGNAL(valueChanged(int)), plannerModel, SLOT(setGFHigh(int)));
-	connect(ui.gflow, SIGNAL(valueChanged(int)), plannerModel, SLOT(setGFLow(int)));
-	connect(ui.vpmb_conservatism, SIGNAL(valueChanged(int)), plannerModel, SLOT(setVpmbConservatism(int)));
-	connect(ui.backgasBreaks, SIGNAL(toggled(bool)), this, SLOT(setBackgasBreaks(bool)));
-	connect(ui.bailout, SIGNAL(toggled(bool)), this, SLOT(setBailout(bool)));
-	connect(ui.switch_at_req_stop, SIGNAL(toggled(bool)), plannerModel, SLOT(setSwitchAtReqStop(bool)));
-	connect(ui.min_switch_duration, SIGNAL(valueChanged(int)), plannerModel, SLOT(setMinSwitchDuration(int)));
-	connect(ui.surface_segment, SIGNAL(valueChanged(int)), plannerModel, SLOT(setSurfaceSegment(int)));
-	connect(ui.rebreathermode, SIGNAL(currentIndexChanged(int)), plannerModel, SLOT(setRebreatherMode(int)));
-	connect(ui.rebreathermode, SIGNAL(currentIndexChanged(int)), this, SLOT(setBailoutVisibility(int)));
+	connect(ui.recreational_deco, &QAbstractButton::clicked, [this] { disableDecoElements(RECREATIONAL); });
+	connect(ui.buehlmann_deco, &QAbstractButton::clicked, [this] { disableDecoElements(BUEHLMANN); });
+	connect(ui.vpmb_deco, &QAbstractButton::clicked, [this] { disableDecoElements(VPMB); });
 
-	connect(ui.bottompo2, SIGNAL(valueChanged(double)), CylindersModel::instance(), SLOT(updateBestMixes()));
-	connect(ui.bestmixEND, SIGNAL(valueChanged(int)), CylindersModel::instance(), SLOT(updateBestMixes()));
-
-	connect(modeMapper, SIGNAL(mapped(int)), this, SLOT(disableDecoElements(int)));
-	connect(ui.ascRate75, SIGNAL(valueChanged(int)), this, SLOT(setAscrate75(int)));
-	connect(ui.ascRate50, SIGNAL(valueChanged(int)), this, SLOT(setAscrate50(int)));
-	connect(ui.descRate, SIGNAL(valueChanged(int)), this, SLOT(setDescrate(int)));
-	connect(ui.ascRateStops, SIGNAL(valueChanged(int)), this, SLOT(setAscratestops(int)));
-	connect(ui.ascRateLast6m, SIGNAL(valueChanged(int)), this, SLOT(setAscratelast6m(int)));
-	connect(ui.sacfactor, SIGNAL(valueChanged(double)), this, SLOT(sacFactorChanged(double)));
-	connect(ui.problemsolvingtime, SIGNAL(valueChanged(int)), this, SLOT(problemSolvingTimeChanged(int)));
-	connect(ui.bottompo2, SIGNAL(valueChanged(double)), this, SLOT(setBottomPo2(double)));
-	connect(ui.decopo2, SIGNAL(valueChanged(double)), this, SLOT(setDecoPo2(double)));
-	connect(ui.bestmixEND, SIGNAL(valueChanged(int)), this, SLOT(setBestmixEND(int)));
-	connect(ui.bottomSAC, SIGNAL(valueChanged(double)), this, SLOT(bottomSacChanged(double)));
-	connect(ui.decoStopSAC, SIGNAL(valueChanged(double)), this, SLOT(decoSacChanged(double)));
+	connect(ui.sacfactor, QOverload<double>::of(&QDoubleSpinBox::valueChanged), &PlannerShared::set_sacfactor);
+	connect(ui.problemsolvingtime, QOverload<int>::of(&QSpinBox::valueChanged), plannerModel, &DivePlannerPointsModel::setProblemSolvingTime);
+	connect(ui.bottompo2, QOverload<double>::of(&QDoubleSpinBox::valueChanged), &PlannerShared::set_bottompo2);
+	connect(ui.decopo2, QOverload<double>::of(&QDoubleSpinBox::valueChanged), &PlannerShared::set_decopo2);
+	connect(ui.bestmixEND, QOverload<int>::of(&QSpinBox::valueChanged), &PlannerShared::set_bestmixend);
+	connect(ui.bottomSAC, QOverload<double>::of(&QDoubleSpinBox::valueChanged), &PlannerShared::set_bottomsac);
+	connect(ui.decoStopSAC, QOverload<double>::of(&QDoubleSpinBox::valueChanged), &PlannerShared::set_decosac);
 
 	settingsChanged();
 	ui.gflow->setValue(prefs.gflow);
@@ -538,11 +520,12 @@ PlannerSettingsWidget::PlannerSettingsWidget(QWidget *parent, Qt::WindowFlags f)
 
 void PlannerSettingsWidget::updateUnitsUI()
 {
-	ui.ascRate75->setValue(lrint(prefs.ascrate75 / UNIT_FACTOR));
-	ui.ascRate50->setValue(lrint(prefs.ascrate50 / UNIT_FACTOR));
-	ui.ascRateStops->setValue(lrint(prefs.ascratestops / UNIT_FACTOR));
-	ui.ascRateLast6m->setValue(lrint(prefs.ascratelast6m / UNIT_FACTOR));
-	ui.descRate->setValue(lrint(prefs.descrate / UNIT_FACTOR));
+	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
+	ui.ascRate75->setValue(plannerModel->ascrate75Display());
+	ui.ascRate50->setValue(plannerModel->ascrate50Display());
+	ui.ascRateStops->setValue(plannerModel->ascratestopsDisplay());
+	ui.ascRateLast6m->setValue(plannerModel->ascratelast6mDisplay());
+	ui.descRate->setValue(lrint(plannerModel->descrateDisplay()));
 	ui.bestmixEND->setValue(lrint(get_depth_units(prefs.bestmixend.mm, NULL, NULL)));
 }
 
@@ -576,8 +559,8 @@ void PlannerSettingsWidget::settingsChanged()
 		ui.bottomSAC->setSingleStep(0.1);
 		ui.decoStopSAC->setDecimals(2);
 		ui.decoStopSAC->setSingleStep(0.1);
-		ui.bottomSAC->setValue(ml_to_cuft(prefs.bottomsac));
-		ui.decoStopSAC->setValue(ml_to_cuft(prefs.decosac));
+		ui.bottomSAC->setValue(PlannerShared::bottomsac());
+		ui.decoStopSAC->setValue(PlannerShared::decosac());
 	} else {
 		ui.bottomSAC->setSuffix(tr("ℓ/min"));
 		ui.decoStopSAC->setSuffix(tr("ℓ/min"));
@@ -585,8 +568,8 @@ void PlannerSettingsWidget::settingsChanged()
 		ui.bottomSAC->setSingleStep(1);
 		ui.decoStopSAC->setDecimals(0);
 		ui.decoStopSAC->setSingleStep(1);
-		ui.bottomSAC->setValue((double) prefs.bottomsac / 1000.0);
-		ui.decoStopSAC->setValue((double) prefs.decosac / 1000.0);
+		ui.bottomSAC->setValue(PlannerShared::bottomsac());
+		ui.decoStopSAC->setValue(PlannerShared::decosac());
 	}
 	if(get_units()->pressure == units::BAR) {
 		ui.reserve_gas->setSuffix(tr("bar"));
@@ -614,69 +597,9 @@ void PlannerSettingsWidget::printDecoPlan()
 {
 }
 
-void PlannerSettingsWidget::setAscrate75(int rate)
-{
-	qPrefDivePlanner::instance()->set_ascrate75(lrint(rate * UNIT_FACTOR));
-}
-
-void PlannerSettingsWidget::setAscrate50(int rate)
-{
-	qPrefDivePlanner::instance()->set_ascrate50(lrint(rate * UNIT_FACTOR));
-}
-
-void PlannerSettingsWidget::setAscratestops(int rate)
-{
-	qPrefDivePlanner::instance()->set_ascratestops(lrint(rate * UNIT_FACTOR));
-}
-
-void PlannerSettingsWidget::setAscratelast6m(int rate)
-{
-	qPrefDivePlanner::instance()->set_ascratelast6m(lrint(rate * UNIT_FACTOR));
-}
-
-void PlannerSettingsWidget::setDescrate(int rate)
-{
-	qPrefDivePlanner::instance()->set_descrate(lrint(rate * UNIT_FACTOR));
-}
-
-void PlannerSettingsWidget::sacFactorChanged(const double factor)
-{
-	plannerModel->setSacFactor(factor);
-}
-
-void PlannerSettingsWidget::problemSolvingTimeChanged(const int minutes)
-{
-	plannerModel->setProblemSolvingTime(minutes);
-}
-
-void PlannerSettingsWidget::setBottomPo2(double po2)
-{
-	qPrefDivePlanner::instance()->set_bottompo2((int) (po2 * 1000.0));
-}
-
-void PlannerSettingsWidget::setDecoPo2(double po2)
-{
-	pressure_t olddecopo2;
-	olddecopo2.mbar = prefs.decopo2;
-	qPrefDivePlanner::instance()->set_decopo2((int) (po2 * 1000.0));
-	CylindersModel::instance()->updateDecoDepths(olddecopo2);
-}
-
-void PlannerSettingsWidget::setBestmixEND(int depth)
-{
-	qPrefDivePlanner::instance()->set_bestmixend(units_to_depth(depth).mm);
-}
-
 void PlannerSettingsWidget::setBackgasBreaks(bool dobreaks)
 {
-	qPrefDivePlanner::instance()->set_doo2breaks(dobreaks);
-	plannerModel->emitDataChanged();
-}
-
-void PlannerSettingsWidget::setBailout(bool dobailout)
-{
-	qPrefDivePlanner::instance()->set_dobailout(dobailout);
-	plannerModel->emitDataChanged();
+	PlannerShared::set_doo2breaks(dobreaks);
 }
 
 void PlannerSettingsWidget::setBailoutVisibility(int mode)
