@@ -15,11 +15,9 @@
 #include <QStatusBar>
 #include <QNetworkProxy>
 #include <QUndoStack>
-#include <QtConcurrentRun>
 
 #include "core/color.h"
 #include "core/device.h"
-#include "core/divecomputer.h"
 #include "core/divesitehelpers.h"
 #include "core/errorhelper.h"
 #include "core/file.h"
@@ -38,10 +36,7 @@
 #include "core/settings/qPrefPartialPressureGas.h"
 #include "core/settings/qPrefTechnicalDetails.h"
 
-#include "core/subsurface-qt/DiveListNotifier.h"
-
 #include "desktop-widgets/about.h"
-#include "desktop-widgets/command.h"
 #include "desktop-widgets/divecomputermanagementdialog.h"
 #include "desktop-widgets/divelistview.h"
 #include "desktop-widgets/divelogexportdialog.h"
@@ -55,8 +50,8 @@
 #include "desktop-widgets/subsurfacewebservices.h"
 #include "desktop-widgets/tab-widgets/maintab.h"
 #include "desktop-widgets/updatemanager.h"
-#include "desktop-widgets/usersurvey.h"
 #include "desktop-widgets/simplewidgets.h"
+#include "commands/command.h"
 
 #include "profile-widget/profilewidget2.h"
 
@@ -89,7 +84,6 @@ namespace {
 	};
 }
 
-
 extern "C" int updateProgress(const char *text)
 {
 	if (verbose)
@@ -97,7 +91,11 @@ extern "C" int updateProgress(const char *text)
 	if (progressDialog) {
 		// apparently we don't always get enough space to show the full label
 		// so let's manually make enough space (but don't shrink the existing size)
+#if QT_VERSION < QT_VERSION_CHECK(5, 11, 0)
 		int width = QFontMetrics(qApp->font()).width(text) + 100;
+#else // QT 5.11 or newer
+		int width = QFontMetrics(qApp->font()).horizontalAdvance(text) + 100;
+#endif
 		if (width > progressDialog->width())
 			progressDialog->resize(width + 20, progressDialog->height());
 		progressDialog->setLabelText(text);
@@ -125,7 +123,6 @@ MainWindow::MainWindow() : QMainWindow(),
 	helpView(0),
 #endif
 	state(VIEWALL),
-	survey(nullptr),
 	findMovedImagesDialog(nullptr)
 {
 	Q_ASSERT_X(m_Instance == NULL, "MainWindow", "MainWindow recreated!");
@@ -214,6 +211,12 @@ MainWindow::MainWindow() : QMainWindow(),
 	connect(this, &MainWindow::showError, ui.mainErrorMessage, &NotificationWidget::showError, Qt::AutoConnection);
 
 	connect(&windowTitleUpdate, &WindowTitleUpdate::updateTitle, this, &MainWindow::setAutomaticTitle);
+	connect(&diveListNotifier, &DiveListNotifier::numShownChanged, this, &MainWindow::setAutomaticTitle);
+	// monitor when dives changed - but only in verbose mode
+	// careful - changing verbose at runtime isn't enough (of course that could be added if we want it)
+	if (verbose)
+		connect(&diveListNotifier, &DiveListNotifier::divesChanged, this, &MainWindow::divesChanged);
+
 #ifdef NO_PRINTING
 	plannerDetails->printPlan()->hide();
 	ui.menuFile->removeAction(ui.actionPrint);
@@ -320,7 +323,6 @@ MainWindow::MainWindow() : QMainWindow(),
 	connect(graphics, &ProfileWidget2::enableToolbar ,this, &MainWindow::setEnabledToolbar);
 	connect(graphics, &ProfileWidget2::disableShortcuts, this, &MainWindow::disableShortcuts);
 	connect(graphics, &ProfileWidget2::enableShortcuts, this, &MainWindow::enableShortcuts);
-	connect(graphics, &ProfileWidget2::refreshDisplay, this, &MainWindow::refreshDisplay);
 	connect(graphics, &ProfileWidget2::editCurrentDive, this, &MainWindow::editCurrentDive);
 	connect(graphics, &ProfileWidget2::updateDiveInfo, mainTab.get(), &MainTab::updateDiveInfo);
 
@@ -409,7 +411,7 @@ void MainWindow::refreshDisplay(bool doRecreateDiveList)
 	mainTab->reload();
 	TankInfoModel::instance()->update();
 	if (doRecreateDiveList)
-		recreateDiveList();
+		diveList->reload();
 
 	MapWidget::instance()->reload();
 	setApplicationState(ApplicationState::Default);
@@ -419,12 +421,6 @@ void MainWindow::refreshDisplay(bool doRecreateDiveList)
 	ui.actionAutoGroup->setChecked(autogroup);
 }
 
-void MainWindow::recreateDiveList()
-{
-	diveList->reload();
-	MultiFilterSortModel::instance()->myInvalidate();
-}
-
 void MainWindow::configureToolbar()
 {
 	if (current_dive) {
@@ -547,7 +543,6 @@ void MainWindow::on_actionCloudstorageopen_triggered()
 	if (!parse_file(fileNamePtr.data(), &dive_table, &trip_table, &dive_site_table))
 		setCurrentFile(fileNamePtr.data());
 	process_loaded_dives();
-	Command::clear();
 	hideProgressBar();
 	refreshDisplay();
 }
@@ -634,21 +629,6 @@ void MainWindow::on_actionCloudOnline_triggered()
 	updateCloudOnlineStatus();
 }
 
-void MainWindow::cleanUpEmpty()
-{
-	current_dive = nullptr;
-	mainTab->clearTabs();
-	mainTab->updateDiveInfo();
-	graphics->setEmptyState();
-	diveList->reload();
-	diveList->setSortOrder(DiveTripModelBase::NR, Qt::DescendingOrder);
-	MapWidget::instance()->reload();
-	LocationInformationModel::instance()->update();
-	if (!existing_filename)
-		setTitle();
-	disableShortcuts();
-}
-
 bool MainWindow::okToClose(QString message)
 {
 	if (DivePlannerPointsModel::instance()->currentMode() != DivePlannerPointsModel::NOTHING ||
@@ -670,17 +650,16 @@ void MainWindow::setFileClean()
 
 void MainWindow::closeCurrentFile()
 {
-	graphics->setEmptyState();
 	/* free the dives and trips */
 	clear_git_id();
-	clear_dive_file_data();
+	MultiFilterSortModel::instance()->clear(); // this clears all the core data structures
 	setCurrentFile(nullptr);
-	cleanUpEmpty();
+	diveList->setSortOrder(DiveTripModelBase::NR, Qt::DescendingOrder);
+	MapWidget::instance()->reload();
+	if (!existing_filename)
+		setTitle();
+	disableShortcuts();
 	setFileClean();
-
-	clear_events();
-
-	dcList.dcs.clear();
 }
 
 void MainWindow::updateCloudOnlineStatus()
@@ -704,7 +683,6 @@ void MainWindow::on_actionClose_triggered()
 		closeCurrentFile();
 		DivePictureModel::instance()->updateDivePictures();
 		setApplicationState(ApplicationState::Default);
-		recreateDiveList();
 	}
 }
 
@@ -724,6 +702,8 @@ void MainWindow::on_actionPrint_triggered()
 
 void MainWindow::disableShortcuts(bool disablePaste)
 {
+	undoAction->setEnabled(false);
+	redoAction->setEnabled(false);
 	ui.actionPreviousDC->setShortcut(QKeySequence());
 	ui.actionNextDC->setShortcut(QKeySequence());
 	ui.copy->setShortcut(QKeySequence());
@@ -733,6 +713,8 @@ void MainWindow::disableShortcuts(bool disablePaste)
 
 void MainWindow::enableShortcuts()
 {
+	undoAction->setEnabled(true);
+	redoAction->setEnabled(true);
 	ui.actionPreviousDC->setShortcut(Qt::Key_Left);
 	ui.actionNextDC->setShortcut(Qt::Key_Right);
 	ui.copy->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_C));
@@ -850,8 +832,10 @@ void MainWindow::updateVariations(QString variations)
 void MainWindow::printPlan()
 {
 #ifndef NO_PRINTING
-	QString diveplan = plannerDetails->divePlanOutput()->toHtml();
-	QString withDisclaimer = QString("<img height=50 src=\":subsurface-icon\"> ") + diveplan + QString(disclaimer);
+	char *disclaimer = get_planner_disclaimer_formatted();
+	QString diveplan = QStringLiteral("<img height=50 src=\":subsurface-icon\"> ") +
+			   QString(disclaimer) + plannerDetails->divePlanOutput()->toHtml();
+	free(disclaimer);
 
 	QPrinter printer;
 	QPrintDialog *dialog = new QPrintDialog(&printer, this);
@@ -885,9 +869,9 @@ void MainWindow::printPlan()
 	QBuffer buffer(&byteArray);
 	pixmap.save(&buffer, "PNG");
 	QString profileImage = QString("<img src=\"data:image/png;base64,") + byteArray.toBase64() + "\"/><br><br>";
-	withDisclaimer = profileImage + withDisclaimer;
+	diveplan = profileImage + diveplan;
 
-	plannerDetails->divePlanOutput()->setHtml(withDisclaimer);
+	plannerDetails->divePlanOutput()->setHtml(diveplan);
 	plannerDetails->divePlanOutput()->print(&printer);
 	plannerDetails->divePlanOutput()->setHtml(displayed_dive.notes);
 #endif
@@ -930,7 +914,7 @@ void MainWindow::on_actionReplanDive_triggered()
 		divePlannerWidget->setSalinity(current_dive->salinity);
 	DivePlannerPointsModel::instance()->loadFromDive(current_dive);
 	reset_cylinders(&displayed_dive, true);
-	CylindersModel::instance()->updateDive();
+	DivePlannerPointsModel::instance()->cylindersModel()->updateDive(&displayed_dive);
 }
 
 void MainWindow::on_actionDivePlanner_triggered()
@@ -977,10 +961,6 @@ void MainWindow::on_actionAddDive_triggered()
 	fixup_dive(&d);
 
 	Command::addDive(&d, autogroup, true);
-
-	// Plot dive actually copies current_dive to displayed_dive and therefore ensures that the
-	// correct data are displayed!
-	graphics->plotDive(nullptr, false, true);
 }
 
 void MainWindow::on_actionRenumber_triggered()
@@ -1013,9 +993,10 @@ void MainWindow::on_actionYearlyStatistics_triggered()
 	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), &d);
 	connect(quit, SIGNAL(activated()), this, SLOT(close()));
 	d.setWindowFlags(Qt::Window | Qt::CustomizeWindowHint
-		| Qt::WindowCloseButtonHint | Qt::WindowTitleHint);
+		| Qt::WindowCloseButtonHint | Qt::WindowTitleHint | Qt::WindowMaximizeButtonHint);
 	d.setWindowTitle(tr("Yearly statistics"));
 	d.setWindowIcon(QIcon(":subsurface-icon"));
+	d.setSizeGripEnabled(true);
 	d.exec();
 }
 
@@ -1099,40 +1080,6 @@ void MainWindow::on_actionViewAll_triggered()
 	ui.bottomSplitter->setCollapsible(1,false);
 }
 
-void MainWindow::enterEditState()
-{
-	undoAction->setEnabled(false);
-	redoAction->setEnabled(false);
-	stateBeforeEdit = state;
-	if (state == VIEWALL || state == INFO_MAXIMIZED)
-		return;
-	toggleCollapsible(true);
-	beginChangeState(EDIT);
-	ui.topSplitter->setSizes({ EXPANDED, EXPANDED });
-	ui.mainSplitter->setSizes({ EXPANDED, COLLAPSED });
-	int appW = qApp->desktop()->size().width();
-	QList<int> infoProfileSizes { round_int(appW * 0.3), round_int(appW * 0.7) };
-
-	QSettings settings;
-	settings.beginGroup("MainWindow");
-	if (settings.value("mainSplitter").isValid()) {
-		ui.topSplitter->restoreState(settings.value("topSplitter").toByteArray());
-		if (ui.topSplitter->sizes().first() == 0 || ui.topSplitter->sizes().last() == 0)
-			ui.topSplitter->setSizes(infoProfileSizes);
-	} else {
-		ui.topSplitter->setSizes(infoProfileSizes);
-	}
-}
-
-void MainWindow::exitEditState()
-{
-	undoAction->setEnabled(true);
-	redoAction->setEnabled(true);
-	if (stateBeforeEdit == state)
-		return;
-	enterState(stateBeforeEdit);
-}
-
 void MainWindow::enterState(CurrentState newState)
 {
 	state = newState;
@@ -1152,8 +1099,6 @@ void MainWindow::enterState(CurrentState newState)
 	case PROFILE_MAXIMIZED:
 		on_actionViewProfile_triggered();
 		break;
-	case EDIT:
-		break;
 	}
 }
 
@@ -1225,13 +1170,6 @@ void MainWindow::on_actionUserManual_triggered()
 #endif
 }
 
-void MainWindow::on_actionUserSurvey_triggered()
-{
-	if(!survey)
-		survey = new UserSurvey(this);
-	survey->show();
-}
-
 void MainWindow::on_actionHash_images_triggered()
 {
 	if(!findMovedImagesDialog)
@@ -1381,40 +1319,16 @@ void MainWindow::initialUiSetup()
 
 void MainWindow::readSettings()
 {
-	static bool firstRun = true;
 	init_proxy();
 
 	// now make sure that the cloud menu items are enabled IFF cloud account is verified
 	enableDisableCloudActions();
 
 	loadRecentFiles();
-	if (firstRun) {
-		checkSurvey();
-		firstRun = false;
-	}
 }
 
 #undef TOOLBOX_PREF_BUTTON
 
-void MainWindow::checkSurvey()
-{
-	QSettings s;
-	s.beginGroup("UserSurvey");
-	if (!s.contains("FirstUse42")) {
-		QVariant value = QDate().currentDate();
-		s.setValue("FirstUse42", value);
-	}
-	// wait a week for production versions, but not at all for non-tagged builds
-	int waitTime = 7;
-	QDate firstUse42 = s.value("FirstUse42").toDate();
-	if (run_survey || (firstUse42.daysTo(QDate().currentDate()) > waitTime && !s.contains("SurveyDone"))) {
-		if (!survey)
-			survey = new UserSurvey(this);
-		survey->show();
-	}
-	s.endGroup();
-}
-
 void MainWindow::writeSettings()
 {
 	QSettings settings;
@@ -1674,9 +1588,9 @@ void MainWindow::importFiles(const QStringList fileNames)
 		return;
 
 	QByteArray fileNamePtr;
-	struct dive_table table = { 0 };
-	struct trip_table trips = { 0 };
-	struct dive_site_table sites = { 0 };
+	struct dive_table table = empty_dive_table;
+	struct trip_table trips = empty_trip_table;
+	struct dive_site_table sites = empty_dive_site_table;
 
 	for (int i = 0; i < fileNames.size(); ++i) {
 		fileNamePtr = QFile::encodeName(fileNames.at(i));
@@ -1705,7 +1619,6 @@ void MainWindow::loadFiles(const QStringList fileNames)
 	hideProgressBar();
 	updateRecentFiles();
 	process_loaded_dives();
-	Command::clear();
 
 	refreshDisplay();
 
@@ -1769,9 +1682,9 @@ void MainWindow::on_actionImportDiveSites_triggered()
 		return;
 	updateLastUsedDir(QFileInfo(fileNames[0]).dir().path());
 
-	struct dive_table table = { 0 };
-	struct trip_table trips = { 0 };
-	struct dive_site_table sites = { 0 };
+	struct dive_table table = empty_dive_table;
+	struct trip_table trips = empty_trip_table;
+	struct dive_site_table sites = empty_dive_site_table;
 
 	for (const QString &s: fileNames) {
 		QByteArray fileNamePtr = QFile::encodeName(s);
@@ -1805,20 +1718,18 @@ void MainWindow::editCurrentDive()
 	struct dive *d = current_dive;
 	QString defaultDC(d->dc.model);
 	DivePlannerPointsModel::instance()->clear();
-	disableShortcuts();
 	if (defaultDC == "manually added dive") {
+		disableShortcuts();
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::ADD);
 		graphics->setAddState();
 		setApplicationState(ApplicationState::EditDive);
 		DivePlannerPointsModel::instance()->loadFromDive(d);
-		mainTab->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
+		mainTab->enableEdition();
 	} else if (defaultDC == "planned dive") {
+		disableShortcuts();
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
 		setApplicationState(ApplicationState::EditPlannedDive);
 		DivePlannerPointsModel::instance()->loadFromDive(d);
-		mainTab->enableEdition(MainTab::MANUALLY_ADDED_DIVE);
-	} else {
-		setApplicationState(ApplicationState::EditDive);
 		mainTab->enableEdition();
 	}
 }
@@ -1957,3 +1868,13 @@ void MainWindow::unsetProfTissues()
 	ui.profTissues->setChecked(false);
 	qPrefTechnicalDetails::set_percentagegraph(false);
 }
+
+void MainWindow::divesChanged(const QVector<dive *> &dives, DiveField field)
+{
+	Q_UNUSED(field)
+	for (struct dive *d: dives) {
+		qDebug() << "dive #" << d->number << "changed, cache is" << (dive_cache_is_valid(d) ? "valid" : "invalidated");
+		// a brute force way to deal with that would of course be to call
+		// invalidate_dive_cache(d);
+	}
+}
