@@ -21,6 +21,8 @@
 #include "desktop-widgets/filterwidget2.h"
 #include "core/applicationstate.h"
 #include "core/gpslocation.h"
+#include "core/dive.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 
 #define NUM_RECENT_FILES 4
 
@@ -56,7 +58,6 @@ public:
 		INFO_MAXIMIZED,
 		PROFILE_MAXIMIZED,
 		LIST_MAXIMIZED,
-		EDIT,
 	};
 
 	MainWindow();
@@ -71,17 +72,13 @@ public:
 
 	void loadFiles(const QStringList files);
 	void importFiles(const QStringList importFiles);
-	void cleanUpEmpty();
 	void setToolButtonsEnabled(bool enabled);
 	void printPlan();
-	void checkSurvey();
 	void setApplicationState(ApplicationState state);
 	bool inPlanner();
 	NotificationWidget *getNotificationWidget();
 	void enableDisableCloudActions();
 	void enableDisableOtherDCsActions();
-	void enterEditState();
-	void exitEditState();
 	void editDiveSite(dive_site *ds);
 
 	std::unique_ptr<MainTab> mainTab;
@@ -129,7 +126,6 @@ slots:
 	/* other menu actions */
 	void on_actionAboutSubsurface_triggered();
 	void on_actionUserManual_triggered();
-	void on_actionUserSurvey_triggered();
 	void on_actionDivePlanner_triggered();
 	void on_actionReplanDive_triggered();
 	void on_action_Check_for_Updates_triggered();
@@ -163,7 +159,6 @@ slots:
 	void turnOffNdlTts();
 	void readSettings();
 	void refreshDisplay(bool doRecreateDiveList = true);
-	void recreateDiveList();
 	void showProfile();
 	void refreshProfile();
 	void editCurrentDive();
@@ -220,7 +215,6 @@ private:
 	void setupForAddAndPlan(const char *model);
 	void configureToolbar();
 	void setupSocialNetworkMenu();
-	QDialog *survey;
 	QDialog *findMovedImagesDialog;
 	struct dive copyPasteDive;
 	struct dive_components what;
@@ -253,6 +247,7 @@ private:
 	GpsLocation *locationProvider;
 	QMenu *connections;
 	QAction *share_on_fb;
+	void divesChanged(const QVector<dive *> &dives, DiveField field);
 };
 
 #endif // MAINWINDOW_H
