@@ -38,7 +38,6 @@ QSize DiveListDelegate::sizeHint(const QStyleOptionViewItem&, const QModelIndex&
 // Gets the index of the model in the currentRow and column.
 // currCombo is defined below.
 #define IDX(_XX) mymodel->index(currCombo.currRow, (_XX))
-static bool keyboardFinished = false;
 
 StarWidgetsDelegate::StarWidgetsDelegate(QWidget *parent) : QStyledItemDelegate(parent),
 	parentWidget(parent)
@@ -85,10 +84,7 @@ const QSize& StarWidgetsDelegate::starSize() const
 ComboBoxDelegate::ComboBoxDelegate(QAbstractItemModel *model, QObject *parent, bool allowEdit) : QStyledItemDelegate(parent), model(model)
 {
 	editable = allowEdit;
-	connect(this, SIGNAL(closeEditor(QWidget *, QAbstractItemDelegate::EndEditHint)),
-		this, SLOT(revertModelData(QWidget *, QAbstractItemDelegate::EndEditHint)));
-	connect(this, SIGNAL(closeEditor(QWidget *, QAbstractItemDelegate::EndEditHint)),
-		this, SLOT(fixTabBehavior()));
+	connect(this, &ComboBoxDelegate::closeEditor, this, &ComboBoxDelegate::editorClosed);
 }
 
 void ComboBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
@@ -103,21 +99,12 @@ void ComboBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index)
 	c->lineEdit()->setSelection(0, c->lineEdit()->text().length());
 }
 
-static struct CurrSelected {
-	QComboBox *comboEditor;
-	int currRow;
-	QString activeText;
-	QAbstractItemModel *model;
-	bool ignoreSelection;
-} currCombo;
-
 QWidget *ComboBoxDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem&, const QModelIndex &index) const
 {
 	QComboBox *comboDelegate = new QComboBox(parent);
 	comboDelegate->setModel(model);
 	comboDelegate->setEditable(true);
-	comboDelegate->setAutoCompletion(true);
-	comboDelegate->setAutoCompletionCaseSensitivity(Qt::CaseInsensitive);
+	comboDelegate->completer()->setCaseSensitivity(Qt::CaseInsensitive);
 	comboDelegate->completer()->setCompletionMode(QCompleter::PopupCompletion);
 	comboDelegate->view()->setEditTriggers(QAbstractItemView::AllEditTriggers);
 	comboDelegate->lineEdit()->installEventFilter(const_cast<QObject *>(qobject_cast<const QObject *>(this)));
@@ -132,7 +119,7 @@ QWidget *ComboBoxDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
 	currCombo.comboEditor = comboDelegate;
 	currCombo.currRow = index.row();
 	currCombo.model = const_cast<QAbstractItemModel *>(index.model());
-	keyboardFinished = false;
+	currCombo.activeText = currCombo.model->data(index).toString();
 
 	// Current display of things on Gnome3 looks like shit, so
 	// let`s fix that.
@@ -180,16 +167,6 @@ void ComboBoxDelegate::fakeActivation()
 	QStyledItemDelegate::eventFilter(currCombo.comboEditor, &ev);
 }
 
-// This 'reverts' the model data to what we actually choosed,
-// becaus e a TAB is being understood by Qt as 'cancel' while
-// we are on a QComboBox ( but not on a QLineEdit.
-void ComboBoxDelegate::fixTabBehavior()
-{
-	if (keyboardFinished) {
-		setModelData(0, 0, QModelIndex());
-	}
-}
-
 bool ComboBoxDelegate::eventFilter(QObject *object, QEvent *event)
 {
 	// Reacts on Key_UP and Key_DOWN to show the QComboBox - list of choices.
@@ -203,10 +180,8 @@ bool ComboBoxDelegate::eventFilter(QObject *object, QEvent *event)
 					return true;
 				}
 			}
-			if (ev->key() == Qt::Key_Tab || ev->key() == Qt::Key_Enter || ev->key() == Qt::Key_Return) {
+			if (ev->key() == Qt::Key_Tab || ev->key() == Qt::Key_Enter || ev->key() == Qt::Key_Return)
 				currCombo.activeText = currCombo.comboEditor->currentText();
-				keyboardFinished = true;
-			}
 		} else { // the 'Drop Down Menu' part.
 			QKeyEvent *ev = static_cast<QKeyEvent *>(event);
 			if (ev->key() == Qt::Key_Enter || ev->key() == Qt::Key_Return ||
@@ -233,17 +208,11 @@ void ComboBoxDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionV
 	editor->setGeometry(defaultRect);
 }
 
-static struct RevertCylinderData {
-	QString type;
-	int pressure;
-	int size;
-} currCylinderData;
-
 void TankInfoDelegate::setModelData(QWidget*, QAbstractItemModel*, const QModelIndex&) const
 {
-	CylindersModel *mymodel = qobject_cast<CylindersModel *>(currCombo.model);
+	QAbstractItemModel *mymodel = currCombo.model;
 	TankInfoModel *tanks = TankInfoModel::instance();
-	QModelIndexList matches = tanks->match(tanks->index(0, 0), Qt::DisplayRole, currCombo.activeText);
+	QModelIndexList matches = tanks->match(tanks->index(0, 0), Qt::DisplayRole, currCombo.activeText, 1, Qt::MatchFixedString | Qt::MatchWrap);
 	int row;
 	QString cylinderName = currCombo.activeText;
 	if (matches.isEmpty()) {
@@ -257,48 +226,25 @@ void TankInfoDelegate::setModelData(QWidget*, QAbstractItemModel*, const QModelI
 	int tankSize = tanks->data(tanks->index(row, TankInfoModel::ML)).toInt();
 	int tankPressure = tanks->data(tanks->index(row, TankInfoModel::BAR)).toInt();
 
-	mymodel->setData(IDX(CylindersModel::TYPE), cylinderName, Qt::EditRole);
-	mymodel->passInData(IDX(CylindersModel::WORKINGPRESS), tankPressure);
-	mymodel->passInData(IDX(CylindersModel::SIZE), tankSize);
+	mymodel->setData(IDX(CylindersModel::TYPE), cylinderName, CylindersModel::TEMP_ROLE);
+	mymodel->setData(IDX(CylindersModel::WORKINGPRESS), tankPressure, CylindersModel::TEMP_ROLE);
+	mymodel->setData(IDX(CylindersModel::SIZE), tankSize, CylindersModel::TEMP_ROLE);
 }
 
 TankInfoDelegate::TankInfoDelegate(QObject *parent) : ComboBoxDelegate(TankInfoModel::instance(), parent, true)
 {
-	connect(this, SIGNAL(closeEditor(QWidget *, QAbstractItemDelegate::EndEditHint)),
-		this, SLOT(reenableReplot(QWidget *, QAbstractItemDelegate::EndEditHint)));
 }
 
-void TankInfoDelegate::reenableReplot(QWidget*, QAbstractItemDelegate::EndEditHint)
+void TankInfoDelegate::editorClosed(QWidget*, QAbstractItemDelegate::EndEditHint hint)
 {
-	MainWindow::instance()->graphics->setReplot(true);
-	// FIXME: We need to replot after a cylinder is selected but the replot below overwrites
-	//        the newly selected cylinder.
-	//	MainWindow::instance()->graphics->replot();
-}
-
-void TankInfoDelegate::revertModelData(QWidget*, QAbstractItemDelegate::EndEditHint hint)
-{
-	if (hint == QAbstractItemDelegate::NoHint ||
-	    hint == QAbstractItemDelegate::RevertModelCache) {
-		CylindersModel *mymodel = qobject_cast<CylindersModel *>(currCombo.model);
-		mymodel->setData(IDX(CylindersModel::TYPE), currCylinderData.type, Qt::EditRole);
-		mymodel->passInData(IDX(CylindersModel::WORKINGPRESS), currCylinderData.pressure);
-		mymodel->passInData(IDX(CylindersModel::SIZE), currCylinderData.size);
-	}
-}
-
-QWidget *TankInfoDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
-{
-	// ncreate editor needs to be called before because it will populate a few
-	// things in the currCombo global var.
-	QWidget *delegate = ComboBoxDelegate::createEditor(parent, option, index);
-	CylindersModel *mymodel = qobject_cast<CylindersModel *>(currCombo.model);
-	cylinder_t *cyl = mymodel->cylinderAt(index);
-	currCylinderData.type = cyl->type.description;
-	currCylinderData.pressure = cyl->type.workingpressure.mbar;
-	currCylinderData.size = cyl->type.size.mliter;
-	MainWindow::instance()->graphics->setReplot(false);
-	return delegate;
+	QAbstractItemModel *mymodel = currCombo.model;
+	// Ugly hack: We misuse setData() with COMMIT_ROLE or REVERT_ROLE to commit or
+	// revert the current row. We send in the type, because we may get multiple
+	// end events and thus can prevent multiple commits.
+	if (hint == QAbstractItemDelegate::RevertModelCache)
+		mymodel->setData(IDX(CylindersModel::TYPE), currCombo.activeText, CylindersModel::REVERT_ROLE);
+	else
+		mymodel->setData(IDX(CylindersModel::TYPE), currCombo.activeText, CylindersModel::COMMIT_ROLE);
 }
 
 TankUseDelegate::TankUseDelegate(QObject *parent) : QStyledItemDelegate(parent)
@@ -326,58 +272,36 @@ void TankUseDelegate::setModelData(QWidget * editor, QAbstractItemModel * model,
 	model->setData(index, comboBox->currentIndex());
 }
 
-static struct RevertWeightData {
-	QString type;
-	int weight;
-} currWeight;
-
-void WSInfoDelegate::revertModelData(QWidget*, QAbstractItemDelegate::EndEditHint hint)
+void WSInfoDelegate::editorClosed(QWidget*, QAbstractItemDelegate::EndEditHint hint)
 {
-	if (hint == QAbstractItemDelegate::NoHint ||
-	    hint == QAbstractItemDelegate::RevertModelCache) {
-		WeightModel *mymodel = qobject_cast<WeightModel *>(currCombo.model);
-		mymodel->setData(IDX(WeightModel::TYPE), currWeight.type, Qt::EditRole);
-		mymodel->passInData(IDX(WeightModel::WEIGHT), currWeight.weight);
-	}
+	WeightModel *mymodel = qobject_cast<WeightModel *>(currCombo.model);
+	if (hint == QAbstractItemDelegate::RevertModelCache)
+		mymodel->clearTempWS();
+	else
+		mymodel->commitTempWS();
 }
 
 void WSInfoDelegate::setModelData(QWidget*, QAbstractItemModel*, const QModelIndex&) const
 {
 	WeightModel *mymodel = qobject_cast<WeightModel *>(currCombo.model);
 	WSInfoModel *wsim = WSInfoModel::instance();
-	QModelIndexList matches = wsim->match(wsim->index(0, 0), Qt::DisplayRole, currCombo.activeText);
-	int row;
-	if (matches.isEmpty()) {
-		// we need to add this puppy
-		wsim->insertRows(wsim->rowCount(), 1);
-		wsim->setData(wsim->index(wsim->rowCount() - 1, 0), currCombo.activeText);
-		row = wsim->rowCount() - 1;
-	} else {
-		row = matches.first().row();
+	QString weightName = currCombo.activeText;
+	QModelIndexList matches = wsim->match(wsim->index(0, 0), Qt::DisplayRole, weightName, 1, Qt::MatchFixedString | Qt::MatchWrap);
+	int grams = 0;
+	if (!matches.isEmpty()) {
+		int row = matches.first().row();
+		weightName = matches.first().data().toString();
+		grams = wsim->data(wsim->index(row, WSInfoModel::GR)).toInt();
 	}
-	int grams = wsim->data(wsim->index(row, WSInfoModel::GR)).toInt();
-	QVariant v = QString(currCombo.activeText);
 
-	mymodel->setData(IDX(WeightModel::TYPE), v, Qt::EditRole);
-	mymodel->passInData(IDX(WeightModel::WEIGHT), grams);
+	mymodel->setTempWS(currCombo.currRow, weightsystem_t{ { grams }, copy_qstring(weightName) });
 }
 
 WSInfoDelegate::WSInfoDelegate(QObject *parent) : ComboBoxDelegate(WSInfoModel::instance(), parent, true)
 {
 }
 
-QWidget *WSInfoDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
-{
-	/* First, call the combobox-create editor, it will setup our globals. */
-	QWidget *editor = ComboBoxDelegate::createEditor(parent, option, index);
-	WeightModel *mymodel = qobject_cast<WeightModel *>(currCombo.model);
-	weightsystem_t *ws = mymodel->weightSystemAt(index);
-	currWeight.type = ws->description;
-	currWeight.weight = ws->weight.grams;
-	return editor;
-}
-
-void AirTypesDelegate::revertModelData(QWidget*, QAbstractItemDelegate::EndEditHint)
+void AirTypesDelegate::editorClosed(QWidget*, QAbstractItemDelegate::EndEditHint)
 {
 }
 
@@ -393,7 +317,7 @@ AirTypesDelegate::AirTypesDelegate(QObject *parent) : ComboBoxDelegate(GasSelect
 {
 }
 
-void DiveTypesDelegate::revertModelData(QWidget*, QAbstractItemDelegate::EndEditHint)
+void DiveTypesDelegate::editorClosed(QWidget*, QAbstractItemDelegate::EndEditHint)
 {
 }
 
@@ -441,7 +365,7 @@ QWidget *DoubleSpinBoxDelegate::createEditor(QWidget *parent, const QStyleOption
 	return w;
 }
 
-LocationFilterDelegate::LocationFilterDelegate(QObject *) : currentLocation({0, 0})
+LocationFilterDelegate::LocationFilterDelegate(QObject *) : currentLocation(zero_location)
 {
 }
 
