@@ -18,12 +18,11 @@
 #include "core/qthelper.h"
 #include "libdivecomputer/parser.h"
 #include "desktop-widgets/divelistview.h"
-#include "core/display.h"
+#include "core/selection.h"
 #include "profile-widget/profilewidget2.h"
-#include "desktop-widgets/command.h"
+#include "commands/command.h"
 #include "core/metadata.h"
 #include "core/tag.h"
-#include "core/divelist.h" // for mark_divelist_changed
 
 double MinMaxAvgWidget::average() const
 {
@@ -149,7 +148,6 @@ void RenumberDialog::renumberOnlySelected(bool selected)
 void RenumberDialog::buttonClicked(QAbstractButton *button)
 {
 	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole) {
-		MainWindow::instance()->diveList->rememberSelection();
 		// we remember a list from dive uuid to a new number
 		QVector<QPair<dive *, int>> renumberedDives;
 		int i;
@@ -175,38 +173,21 @@ RenumberDialog::RenumberDialog(QWidget *parent) : QDialog(parent), selectedOnly(
 	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
 }
 
-SetpointDialog *SetpointDialog::instance()
-{
-	static SetpointDialog *self = new SetpointDialog(MainWindow::instance());
-	return self;
-}
-
-void SetpointDialog::setpointData(struct divecomputer *divecomputer, int second)
-{
-	dc = divecomputer;
-	time = second < 0 ? 0 : second;
-}
-
 void SetpointDialog::buttonClicked(QAbstractButton *button)
 {
-	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole && dc) {
-		add_event(dc, time, SAMPLE_EVENT_PO2, 0, (int)(1000.0 * ui.spinbox->value()),
-			QT_TRANSLATE_NOOP("gettextFromC", "SP change"));
-		invalidate_dive_cache(current_dive);
-	}
-	mark_divelist_changed(true);
-	MainWindow::instance()->graphics->replot();
+	if (ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole)
+		Command::addEventSetpointChange(d, dcNr, time, pressure_t { (int)(1000.0 * ui.spinbox->value()) });
 }
 
-SetpointDialog::SetpointDialog(QWidget *parent) : QDialog(parent),
-	dc(0), time(0)
+SetpointDialog::SetpointDialog(struct dive *dIn, int dcNrIn, int seconds) : QDialog(MainWindow::instance()),
+	d(dIn), dcNr(dcNrIn), time(seconds < 0 ? 0 : seconds)
 {
 	ui.setupUi(this);
-	connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(buttonClicked(QAbstractButton *)));
+	connect(ui.buttonBox, &QDialogButtonBox::clicked, this, &SetpointDialog::buttonClicked);
 	QShortcut *close = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_W), this);
-	connect(close, SIGNAL(activated()), this, SLOT(close()));
+	connect(close, &QShortcut::activated, this, &QDialog::close);
 	QShortcut *quit = new QShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q), this);
-	connect(quit, SIGNAL(activated()), parent, SLOT(close()));
+	connect(quit, &QShortcut::activated, MainWindow::instance(), &QWidget::close);
 }
 
 ShiftTimesDialog *ShiftTimesDialog::instance()
@@ -223,17 +204,8 @@ void ShiftTimesDialog::buttonClicked(QAbstractButton *button)
 		amount = ui.timeEdit->time().hour() * 3600 + ui.timeEdit->time().minute() * 60;
 		if (ui.backwards->isChecked())
 			amount *= -1;
-		if (amount != 0) {
-			// DANGER, DANGER - this could get our dive_table unsorted...
-			int i;
-			struct dive *d;
-			QVector<dive *> affectedDives;
-			for_each_dive (i, d) {
-				if (d->selected)
-					affectedDives.append(d);
-			}
-			Command::shiftTime(affectedDives, amount);
-		}
+		if (amount != 0)
+			Command::shiftTime(getDiveSelection(), amount);
 	}
 }
 
@@ -510,9 +482,9 @@ void DiveComponentSelection::buttonClicked(QAbstractButton *button)
 		if (what->cylinders) {
 			int cyl;
 			text << tr("Cylinders:\n");
-			for (cyl = 0; cyl < MAX_CYLINDERS; cyl++) {
+			for (cyl = 0; cyl < displayed_dive.cylinders.nr; cyl++) {
 				if (is_cylinder_used(&displayed_dive, cyl))
-					text << displayed_dive.cylinder[cyl].type.description << " " << gasname(displayed_dive.cylinder[cyl].gasmix) << "\n";
+					text << get_cylinder(&displayed_dive, cyl)->type.description << " " << gasname(get_cylinder(&displayed_dive, cyl)->gasmix) << "\n";
 			}
 		}
 		if (what->weights) {
@@ -635,10 +607,10 @@ QString TextHyperlinkEventFilter::tryToFormulateUrl(QTextCursor *cursor)
 		maybeUrlStr = left + right;
 	}
 
-	return stringMeetsOurUrlRequirements(maybeUrlStr) ? maybeUrlStr : QString::null;
+	return stringMeetsOurUrlRequirements(maybeUrlStr) ? maybeUrlStr : QString();
 }
 
-QString TextHyperlinkEventFilter::fromCursorTilWhitespace(QTextCursor *cursor, const bool searchBackwards)
+QString TextHyperlinkEventFilter::fromCursorTilWhitespace(QTextCursor *cursor, bool searchBackwards)
 {
 	// fromCursorTilWhitespace calls cursor->movePosition repeatedly, while
 	// preserving the original 'anchor' (qt terminology) of the cursor.
