@@ -52,14 +52,14 @@ void TabDivePhotos::clear()
 void TabDivePhotos::contextMenuEvent(QContextMenuEvent *event)
 {
 	QMenu popup(this);
-	popup.addAction(tr("Load media from file(s)"), this, SLOT(addPhotosFromFile()));
-	popup.addAction(tr("Load media file(s) from web"), this, SLOT(addPhotosFromURL()));
+	popup.addAction(tr("Load media from file(s)"), this, &TabDivePhotos::addPhotosFromFile);
+	popup.addAction(tr("Load media file(s) from web"), this, &TabDivePhotos::addPhotosFromURL);
 	popup.addSeparator();
-	popup.addAction(tr("Delete selected media files"), this, SLOT(removeSelectedPhotos()));
-	popup.addAction(tr("Delete all media files"), this, SLOT(removeAllPhotos()));
-	popup.addAction(tr("Open folder of selected media files"), this, SLOT(openFolderOfSelectedFiles()));
-	popup.addAction(tr("Recalculate selected thumbnails"), this, SLOT(recalculateSelectedThumbnails()));
-	popup.addAction(tr("Save dive data as subtitles"), this, SLOT(saveSubtitles()));
+	popup.addAction(tr("Delete selected media files"), this, &TabDivePhotos::removeSelectedPhotos);
+	popup.addAction(tr("Delete all media files"), this, &TabDivePhotos::removeAllPhotos);
+	popup.addAction(tr("Open folder of selected media files"), this, &TabDivePhotos::openFolderOfSelectedFiles);
+	popup.addAction(tr("Recalculate selected thumbnails"), this, &TabDivePhotos::recalculateSelectedThumbnails);
+	popup.addAction(tr("Save dive data as subtitles"), this, &TabDivePhotos::saveSubtitles);
 	popup.exec(event->globalPos());
 	event->accept();
 }
@@ -69,11 +69,11 @@ QVector<QString> TabDivePhotos::getSelectedFilenames() const
 	QVector<QString> selectedPhotos;
 	if (!ui->photosView->selectionModel()->hasSelection())
 		return selectedPhotos;
-	QModelIndexList indexes = ui->photosView->selectionModel()->selectedRows();
-	if (indexes.count() == 0)
-		indexes = ui->photosView->selectionModel()->selectedIndexes();
-	selectedPhotos.reserve(indexes.count());
-	for (const auto &photo: indexes) {
+	QModelIndexList indices = ui->photosView->selectionModel()->selectedRows();
+	if (indices.count() == 0)
+		indices = ui->photosView->selectionModel()->selectedIndexes();
+	selectedPhotos.reserve(indices.count());
+	for (const auto &photo: indices) {
 		if (photo.isValid()) {
 			QString fileUrl = photo.data(Qt::DisplayPropertyRole).toString();
 			if (!fileUrl.isEmpty())
@@ -114,11 +114,11 @@ void TabDivePhotos::saveSubtitles()
 	QVector<QString> selectedPhotos;
 	if (!ui->photosView->selectionModel()->hasSelection())
 		return;
-	QModelIndexList indexes = ui->photosView->selectionModel()->selectedRows();
-	if (indexes.count() == 0)
-		indexes = ui->photosView->selectionModel()->selectedIndexes();
-	selectedPhotos.reserve(indexes.count());
-	for (const auto &photo: indexes) {
+	QModelIndexList indices = ui->photosView->selectionModel()->selectedRows();
+	if (indices.count() == 0)
+		indices = ui->photosView->selectionModel()->selectedIndexes();
+	selectedPhotos.reserve(indices.count());
+	for (const auto &photo: indices) {
 		if (photo.isValid()) {
 			QString fileUrl = photo.data(Qt::DisplayPropertyRole).toString();
 			if (!fileUrl.isEmpty()) {
@@ -132,7 +132,7 @@ void TabDivePhotos::saveSubtitles()
 					continue;
 				struct membuffer b = { 0 };
 				save_subtitles_buffer(&b, &displayed_dive, offset, duration);
-				char *data = detach_buffer(&b);
+				char *data = detach_cstring(&b);
 				subtitlefile.open(QIODevice::WriteOnly);
 				subtitlefile.write(data, strlen(data));
 				subtitlefile.close();
