@@ -2,9 +2,9 @@
 #include "TabDiveStatistics.h"
 #include "ui_TabDiveStatistics.h"
 
-#include <core/qthelper.h>
-#include <core/display.h>
-#include <core/statistics.h>
+#include "core/qthelper.h"
+#include "core/selection.h"
+#include "core/statistics.h"
 
 TabDiveStatistics::TabDiveStatistics(QWidget *parent) : TabBase(parent), ui(new Ui::TabDiveStatistics())
 {
@@ -21,6 +21,8 @@ TabDiveStatistics::TabDiveStatistics(QWidget *parent) : TabBase(parent), ui(new
 	ui->timeLimits->overrideMinToolTipText(tr("Shortest dive"));
 	ui->timeLimits->overrideAvgToolTipText(tr("Average length of all selected dives"));
 
+	connect(&diveListNotifier, &DiveListNotifier::divesChanged, this, &TabDiveStatistics::divesChanged);
+
 	const auto l = findChildren<QLabel *>(QString(), Qt::FindDirectChildrenOnly);
 	for (QLabel *label: l) {
 		label->setAlignment(Qt::AlignHCenter);
@@ -42,18 +44,32 @@ void TabDiveStatistics::clear()
 	ui->timeLimits->clear();
 }
 
+// This function gets called if a field gets updated by an undo command.
+// Refresh the corresponding UI field.
+void TabDiveStatistics::divesChanged(const QVector<dive *> &dives, DiveField field)
+{
+	// If none of the changed dives is selected, do nothing
+	if (std::none_of(dives.begin(), dives.end(), [] (const dive *d) { return d->selected; }))
+		return;
+
+	// TODO: make this more fine grained. Currently, the core can only calculate *all* statistics.
+	if (field.duration || field.depth || field.mode || field.air_temp || field.water_temp)
+		updateData();
+}
+
 void TabDiveStatistics::updateData()
 {
 	stats_t stats_selection;
 	calculate_stats_selected(&stats_selection);
 	clear();
-	ui->depthLimits->setMaximum(get_depth_string(stats_selection.max_depth, true));
 	if (amount_selected > 1) {
+		ui->depthLimits->setMaximum(get_depth_string(stats_selection.max_depth, true));
 		ui->depthLimits->setMinimum(get_depth_string(stats_selection.min_depth, true));
 		ui->depthLimits->setAverage(get_depth_string(stats_selection.combined_max_depth.mm / stats_selection.selection_size, true));
 	} else {
+		ui->depthLimits->setMaximum("");
 		ui->depthLimits->setMinimum("");
-		ui->depthLimits->setAverage("");
+		ui->depthLimits->setAverage(get_depth_string(stats_selection.max_depth, true));
 	}
 
 	if (stats_selection.max_sac.mliter && (stats_selection.max_sac.mliter != stats_selection.avg_sac.mliter))
@@ -69,8 +85,10 @@ void TabDiveStatistics::updateData()
 	else
 		ui->sacLimits->setAverage("");
 
-	ui->tempLimits->setMaximum(get_temperature_string(stats_selection.max_temp, true));
-	ui->tempLimits->setMinimum(get_temperature_string(stats_selection.min_temp, true));
+	if (stats_selection.combined_count > 1) {
+		ui->tempLimits->setMaximum(get_temperature_string(stats_selection.max_temp, true));
+		ui->tempLimits->setMinimum(get_temperature_string(stats_selection.min_temp, true));
+	}
 	if (stats_selection.combined_temp.mkelvin && stats_selection.combined_count) {
 		temperature_t avg_temp;
 		avg_temp.mkelvin = stats_selection.combined_temp.mkelvin / stats_selection.combined_count;
@@ -100,16 +118,12 @@ void TabDiveStatistics::updateData()
 	QVector<QPair<QString, int> > gasUsed = selectedDivesGasUsed();
 	QString gasUsedString;
 	volume_t vol;
-	for (int j = 0; j < MAX_CYLINDERS; j++) {
-		if (gasUsed.isEmpty())
-			break;
+	while (!gasUsed.isEmpty()) {
 		QPair<QString, int> gasPair = gasUsed.last();
 		gasUsed.pop_back();
 		vol.mliter = gasPair.second;
 		gasUsedString.append(gasPair.first).append(": ").append(get_volume_string(vol, true)).append("\n");
 	}
-	if (!gasUsed.isEmpty())
-		gasUsedString.append("...");
 	volume_t o2_tot = {}, he_tot = {};
 	selected_dives_gas_parts(&o2_tot, &he_tot);
 
