@@ -3,6 +3,7 @@
 #define TAB_DIVE_STATISTICS_H
 
 #include "TabBase.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 
 namespace Ui {
 	class TabDiveStatistics;
@@ -13,8 +14,11 @@ class TabDiveStatistics : public TabBase {
 public:
 	TabDiveStatistics(QWidget *parent = 0);
 	~TabDiveStatistics();
-	void updateData();
-	void clear();
+	void updateData() override;
+	void clear() override;
+
+private slots:
+	void divesChanged(const QVector<dive *> &dives, DiveField field);
 
 private:
 	Ui::TabDiveStatistics *ui;
