@@ -12,7 +12,7 @@
 #include "core/trip.h"
 #include "qt-models/diveplannermodel.h"
 #include "desktop-widgets/divelistview.h"
-#include "core/display.h"
+#include "core/selection.h"
 #include "profile-widget/profilewidget2.h"
 #include "desktop-widgets/diveplanner.h"
 #include "core/divesitehelpers.h"
@@ -24,8 +24,8 @@
 #include "core/subsurface-string.h"
 #include "core/gettextfromc.h"
 #include "desktop-widgets/locationinformation.h"
-#include "desktop-widgets/command.h"
 #include "desktop-widgets/simplewidgets.h"
+#include "commands/command.h"
 
 #include "TabDiveEquipment.h"
 #include "TabDiveExtraInfo.h"
@@ -49,7 +49,8 @@ struct Completers {
 };
 
 MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
-	editMode(NONE),
+	editMode(false),
+	ignoreInput(false),
 	lastSelectedDive(true),
 	lastTabSelectedDive(0),
 	lastTabSelectedDiveTrip(0),
@@ -116,13 +117,6 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	// filled from a dive, they are made writeable
 	setEnabled(false);
 
-	// This needs to be the same order as enum dive_comp_type in dive.h!
-	QStringList types = QStringList();
-	for (int i = 0; i < NUM_DIVEMODE; i++)
-		types.append(gettextFromC::tr(divemode_text_ui[i]));
-	ui.DiveType->insertItems(0, types);
-	connect(ui.DiveType, SIGNAL(currentIndexChanged(int)), this, SLOT(divetype_Changed(int)));
-
 	Completers completers;
 	completers.buddy = new QCompleter(&buddyModel, ui.buddy);
 	completers.divemaster = new QCompleter(&diveMasterModel, ui.divemaster);
@@ -193,7 +187,6 @@ MainTab::~MainTab()
 void MainTab::hideMessage()
 {
 	ui.diveNotesMessage->animatedHide();
-	updateTextLabels(false);
 }
 
 void MainTab::closeMessage()
@@ -212,68 +205,21 @@ void MainTab::displayMessage(QString str)
 	ui.diveNotesMessage->setCloseButtonVisible(false);
 	ui.diveNotesMessage->setText(str);
 	ui.diveNotesMessage->animatedShow();
-	updateTextLabels();
 }
 
-void MainTab::updateTextLabels(bool showUnits)
+void MainTab::enableEdition()
 {
-	if (showUnits) {
-		ui.airTempLabel->setText(tr("Air temp. [%1]").arg(get_temp_unit()));
-		ui.waterTempLabel->setText(tr("Water temp. [%1]").arg(get_temp_unit()));
-	} else {
-		ui.airTempLabel->setText(tr("Air temp."));
-		ui.waterTempLabel->setText(tr("Water temp."));
-	}
-}
-
-void MainTab::enableEdition(EditMode newEditMode)
-{
-	if (((newEditMode == DIVE || newEditMode == NONE) && current_dive == NULL) || editMode != NONE)
+	if (current_dive == NULL || editMode)
 		return;
-	modified = false;
-	if ((newEditMode == DIVE || newEditMode == NONE) &&
-	    current_dive->dc.model &&
-	    strcmp(current_dive->dc.model, "manually added dive") == 0) {
-		// editCurrentDive will call enableEdition with newEditMode == MANUALLY_ADDED_DIVE
-		// so exit this function here after editCurrentDive() returns
-
-
-
-		// FIXME : can we get rid of this recursive crap?
-
-
-
-		MainWindow::instance()->editCurrentDive();
-		return;
-	}
 
 	ui.editDiveSiteButton->setEnabled(false);
 	MainWindow::instance()->diveList->setEnabled(false);
 	MainWindow::instance()->setEnabledToolbar(false);
-	MainWindow::instance()->enterEditState();
-	ui.tabWidget->setTabEnabled(2, false);
-	ui.tabWidget->setTabEnabled(3, false);
-	ui.tabWidget->setTabEnabled(5, false);
 
 	ui.dateEdit->setEnabled(true);
-	if (amount_selected > 1) {
-		displayMessage(tr("Multiple dives are being edited."));
-	} else {
-		displayMessage(tr("This dive is being edited."));
-	}
-	editMode = newEditMode != NONE ? newEditMode : DIVE;
-}
+	displayMessage(tr("This dive is being edited."));
 
-static void profileFromDive(struct dive *d)
-{
-	// TODO: We have to put these manipulations into a setPlanState()/setProfileState() pair,
-	// because otherwise the DivePlannerPointsModel and the profile get out of sync.
-	// This can lead to crashes. Let's try to detangle these subtleties.
-	MainWindow::instance()->graphics->setPlanState();
-	DivePlannerPointsModel::instance()->loadFromDive(d);
-	MainWindow::instance()->graphics->setReplot(true);
-	MainWindow::instance()->graphics->plotDive(current_dive, true);
-	MainWindow::instance()->graphics->setProfileState();
+	editMode = true;
 }
 
 // This function gets called if a field gets updated by an undo command.
@@ -284,53 +230,31 @@ void MainTab::divesChanged(const QVector<dive *> &dives, DiveField field)
 	if (!current_dive || !dives.contains(current_dive))
 		return;
 
-	switch(field) {
-	case DiveField::DURATION:
+	if (field.duration)
 		ui.duration->setText(render_seconds_to_string(current_dive->duration.seconds));
-		profileFromDive(current_dive);
-		break;
-	case DiveField::DEPTH:
+	if (field.depth)
 		ui.depth->setText(get_depth_string(current_dive->maxdepth, true));
-		profileFromDive(current_dive);
-		break;
-	case DiveField::AIR_TEMP:
-		ui.airtemp->setText(get_temperature_string(current_dive->airtemp, true));
-		break;
-	case DiveField::WATER_TEMP:
-		ui.watertemp->setText(get_temperature_string(current_dive->watertemp, true));
-		break;
-	case DiveField::RATING:
+	if (field.rating)
 		ui.rating->setCurrentStars(current_dive->rating);
-		break;
-	case DiveField::VISIBILITY:
-		ui.visibility->setCurrentStars(current_dive->visibility);
-		break;
-	case DiveField::NOTES:
+	if (field.notes)
 		updateNotes(current_dive);
-		break;
-	case DiveField::MODE:
-		updateMode(current_dive);
-		break;
-	case DiveField::DATETIME:
+	if (field.datetime) {
 		updateDateTime(current_dive);
 		MainWindow::instance()->graphics->dateTimeChanged();
 		DivePlannerPointsModel::instance()->getDiveplan().when = current_dive->when;
-		break;
-	case DiveField::DIVESITE:
-		updateDiveSite(current_dive);
-		break;
-	case DiveField::TAGS:
-		ui.tagWidget->setText(get_taglist_string(current_dive->tag_list));
-		break;
-	case DiveField::BUDDY:
-		ui.buddy->setText(current_dive->buddy);
-		break;
-	case DiveField::DIVEMASTER:
-		ui.divemaster->setText(current_dive->divemaster);
-		break;
-	default:
-		break;
 	}
+	if (field.divesite)
+		updateDiveSite(current_dive);
+	if (field.tags)
+		ui.tagWidget->setText(get_taglist_string(current_dive->tag_list));
+	if (field.buddy)
+		ui.buddy->setText(current_dive->buddy);
+	if (field.divemaster)
+		ui.divemaster->setText(current_dive->divemaster);
+
+	// If duration or depth changed, the profile needs to be replotted
+	if (field.duration || field.depth)
+		MainWindow::instance()->graphics->plotDive(current_dive, true);
 }
 
 void MainTab::diveSiteEdited(dive_site *ds, int)
@@ -347,16 +271,10 @@ void MainTab::tripChanged(dive_trip *trip, TripField field)
 	if (currentTrip != trip)
 		return;
 
-	switch(field) {
-	case TripField::NOTES:
+	if (field.notes)
 		ui.notes->setText(currentTrip->notes);
-		break;
-	case TripField::LOCATION:
+	if (field.location)
 		ui.diveTripLocation->setText(currentTrip->location);
-		break;
-	default:
-		break;
-	}
 }
 
 void MainTab::nextInputField(QKeyEvent *event)
@@ -366,7 +284,7 @@ void MainTab::nextInputField(QKeyEvent *event)
 
 bool MainTab::isEditing()
 {
-	return editMode != NONE;
+	return editMode;
 }
 
 static bool isHtml(const QString &s)
@@ -384,12 +302,6 @@ void MainTab::updateNotes(const struct dive *d)
 	}
 }
 
-void MainTab::updateMode(struct dive *d)
-{
-	ui.DiveType->setCurrentIndex(get_dive_dc(d, dc_number)->divemode);
-	MainWindow::instance()->graphics->recalcCeiling();
-}
-
 static QDateTime timestampToDateTime(timestamp_t when)
 {
 	// Subsurface always uses "local time" as in "whatever was the local time at the location"
@@ -430,9 +342,8 @@ void MainTab::updateDiveSite(struct dive *d)
 void MainTab::updateDiveInfo()
 {
 	ui.location->refreshDiveSiteCache();
-	EditMode rememberEM = editMode;
 	// don't execute this while adding / planning a dive
-	if (editMode == MANUALLY_ADDED_DIVE || MainWindow::instance()->graphics->isPlanner())
+	if (editMode || MainWindow::instance()->graphics->isPlanner())
 		return;
 
 	// If there is no current dive, disable all widgets except the last, which is the dive site tab.
@@ -442,7 +353,7 @@ void MainTab::updateDiveInfo()
 	for (int i = 0; i < extraWidgets.size() - 1; ++i)
 		extraWidgets[i]->setEnabled(enabled);
 
-	editMode = IGNORE_MODE; // don't trigger on changes to the widgets
+	ignoreInput = true; // don't trigger on changes to the widgets
 
 	for (TabBase *widget: extraWidgets)
 		widget->updateData();
@@ -454,15 +365,11 @@ void MainTab::updateDiveInfo()
 		// 2) the filter is reset, potentially erasing the current trip under our feet.
 		// TODO: Don't hard code tab location!
 		bool onDiveSiteTab = ui.tabWidget->currentIndex() == 6;
-		if (MainWindow::instance() && MainWindow::instance()->diveList->selectedTrips().count() == 1) {
-			currentTrip = MainWindow::instance()->diveList->selectedTrips().front();
+		if ((currentTrip = MainWindow::instance()->diveList->singleSelectedTrip()) != nullptr) {
 			// Remember the tab selected for last dive but only if we're not on the dive site tab
 			if (lastSelectedDive && !onDiveSiteTab)
 				lastTabSelectedDive = ui.tabWidget->currentIndex();
 			ui.tabWidget->setTabText(0, tr("Trip notes"));
-			ui.tabWidget->setTabEnabled(1, false);
-			ui.tabWidget->setTabEnabled(2, false);
-			ui.tabWidget->setTabEnabled(5, false);
 			// Recover the tab selected for last dive trip but only if we're not on the dive site tab
 			if (lastSelectedDive && !onDiveSiteTab)
 				ui.tabWidget->setCurrentIndex(lastTabSelectedDiveTrip);
@@ -474,16 +381,8 @@ void MainTab::updateDiveInfo()
 			ui.BuddyLabel->setVisible(false);
 			ui.rating->setVisible(false);
 			ui.RatingLabel->setVisible(false);
-			ui.visibility->setVisible(false);
-			ui.visibilityLabel->setVisible(false);
 			ui.tagWidget->setVisible(false);
 			ui.TagLabel->setVisible(false);
-			ui.airTempLabel->setVisible(false);
-			ui.airtemp->setVisible(false);
-			ui.DiveType->setVisible(false);
-			ui.TypeLabel->setVisible(false);
-			ui.waterTempLabel->setVisible(false);
-			ui.watertemp->setVisible(false);
 			ui.dateEdit->setReadOnly(true);
 			ui.timeLabel->setVisible(false);
 			ui.timeEdit->setVisible(false);
@@ -509,11 +408,6 @@ void MainTab::updateDiveInfo()
 			if (!lastSelectedDive && !onDiveSiteTab)
 				lastTabSelectedDiveTrip = ui.tabWidget->currentIndex();
 			ui.tabWidget->setTabText(0, tr("Notes"));
-			ui.tabWidget->setTabEnabled(1, true);
-			ui.tabWidget->setTabEnabled(2, true);
-			ui.tabWidget->setTabEnabled(3, true);
-			ui.tabWidget->setTabEnabled(4, true);
-			ui.tabWidget->setTabEnabled(5, true);
 			// Recover the tab selected for last dive but only if we're not on the dive site tab
 			if (!lastSelectedDive && !onDiveSiteTab)
 				ui.tabWidget->setCurrentIndex(lastTabSelectedDive);
@@ -528,24 +422,15 @@ void MainTab::updateDiveInfo()
 			ui.buddy->setVisible(true);
 			ui.rating->setVisible(true);
 			ui.RatingLabel->setVisible(true);
-			ui.visibility->setVisible(true);
-			ui.visibilityLabel->setVisible(true);
 			ui.BuddyLabel->setVisible(true);
 			ui.DivemasterLabel->setVisible(true);
 			ui.TagLabel->setVisible(true);
 			ui.tagWidget->setVisible(true);
-			ui.airTempLabel->setVisible(true);
-			ui.airtemp->setVisible(true);
-			ui.TypeLabel->setVisible(true);
-			ui.DiveType->setVisible(true);
-			ui.waterTempLabel->setVisible(true);
-			ui.watertemp->setVisible(true);
 			ui.dateEdit->setReadOnly(false);
 			ui.timeLabel->setVisible(true);
 			ui.timeEdit->setVisible(true);
 			/* and fill them from the dive */
 			ui.rating->setCurrentStars(current_dive->rating);
-			ui.visibility->setCurrentStars(current_dive->visibility);
 			// reset labels in case we last displayed trip notes
 			ui.LocationLabel->setText(tr("Location"));
 			ui.NotesLabel->setText(tr("Notes"));
@@ -557,13 +442,10 @@ void MainTab::updateDiveInfo()
 			ui.durationLabel->setVisible(isManual);
 
 			updateNotes(current_dive);
-			updateMode(current_dive);
 			updateDiveSite(current_dive);
 			updateDateTime(current_dive);
 			ui.divemaster->setText(current_dive->divemaster);
 			ui.buddy->setText(current_dive->buddy);
-			ui.airtemp->setText(get_temperature_string(current_dive->airtemp, true));
-			ui.watertemp->setText(get_temperature_string(current_dive->watertemp, true));
 		}
 		ui.duration->setText(render_seconds_to_string(current_dive->duration.seconds));
 		ui.depth->setText(get_depth_string(current_dive->maxdepth, true));
@@ -580,12 +462,9 @@ void MainTab::updateDiveInfo()
 		/* clear the fields */
 		clearTabs();
 		ui.rating->setCurrentStars(0);
-		ui.visibility->setCurrentStars(0);
 		ui.location->clear();
 		ui.divemaster->clear();
 		ui.buddy->clear();
-		ui.airtemp->clear();
-		ui.watertemp->clear();
 		ui.notes->clear();
 		/* set date and time to minimums which triggers showing the special value text */
 		ui.dateEdit->setSpecialValueText(QString("-"));
@@ -596,7 +475,7 @@ void MainTab::updateDiveInfo()
 		ui.timeEdit->setTime(QTime(0, 0, 0, 0));
 		ui.tagWidget->clear();
 	}
-	editMode = rememberEM;
+	ignoreInput = false;
 
 	if (verbose && current_dive && current_dive->dive_site)
 		qDebug() << "Set the current dive site:" << current_dive->dive_site->uuid;
@@ -607,7 +486,6 @@ void MainTab::reload()
 	buddyModel.updateModel();
 	diveMasterModel.updateModel();
 	tagModel.updateModel();
-	LocationInformationModel::instance()->update();
 }
 
 void MainTab::refreshDisplayedDiveSite()
@@ -617,80 +495,38 @@ void MainTab::refreshDisplayedDiveSite()
 
 void MainTab::acceptChanges()
 {
-	int addedId = -1;
-
 	if (ui.location->hasFocus())
 		stealFocus();
 
-	EditMode lastMode = editMode;
-	editMode = IGNORE_MODE;
-	tabBar()->setTabIcon(0, QIcon()); // Notes
-	tabBar()->setTabIcon(1, QIcon()); // Equipment
+	ignoreInput = true;
 	ui.dateEdit->setEnabled(true);
 	hideMessage();
 
-	if (lastMode == MANUALLY_ADDED_DIVE) {
-		// preserve any changes to the profile
-		free(current_dive->dc.sample);
-		copy_samples(&displayed_dive.dc, &current_dive->dc);
-		addedId = displayed_dive.id;
-	}
-
-	// TODO: This is a temporary hack until the equipment tab is included in the undo system:
-	// The equipment tab is hardcoded at the first place of the "extra widgets".
-	((TabDiveEquipment *)extraWidgets[0])->acceptChanges();
-
-	if (lastMode == MANUALLY_ADDED_DIVE) {
-		// we just added or edited the dive, let fixup_dive() make
-		// sure we get the max. depth right
-		current_dive->maxdepth.mm = current_dc->maxdepth.mm = 0;
-		fixup_dive(current_dive);
-		set_dive_nr_for_current_dive();
+	if (editMode) {
 		MainWindow::instance()->showProfile();
-		mark_divelist_changed(true);
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
+		Command::editProfile(&displayed_dive);
 	}
 	int scrolledBy = MainWindow::instance()->diveList->verticalScrollBar()->sliderPosition();
-	if (lastMode == MANUALLY_ADDED_DIVE) {
+	if (editMode) {
 		MainWindow::instance()->diveList->reload();
-		int newDiveNr = get_divenr(get_dive_by_uniq_id(addedId));
-		MainWindow::instance()->diveList->unselectDives();
-		MainWindow::instance()->diveList->selectDive(newDiveNr, true);
 		MainWindow::instance()->refreshDisplay();
 		MainWindow::instance()->graphics->replot();
 	} else {
-		MainWindow::instance()->diveList->rememberSelection();
 		MainWindow::instance()->refreshDisplay();
-		MainWindow::instance()->diveList->restoreSelection();
 	}
 	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
 	MainWindow::instance()->diveList->verticalScrollBar()->setSliderPosition(scrolledBy);
 	MainWindow::instance()->diveList->setFocus();
-	MainWindow::instance()->exitEditState();
 	MainWindow::instance()->setEnabledToolbar(true);
 	ui.editDiveSiteButton->setEnabled(!ui.location->text().isEmpty());
-	editMode = NONE;
-}
-
-bool weightsystems_equal(const dive *d1, const dive *d2)
-{
-	if (d1->weightsystems.nr != d2->weightsystems.nr)
-		return false;
-	for (int i = 0; i < d1->weightsystems.nr; ++i) {
-		if (!same_weightsystem(d1->weightsystems.weightsystems[i], d2->weightsystems.weightsystems[i]))
-			return false;
-	}
-	return true;
+	ignoreInput = false;
+	editMode = false;
 }
 
 void MainTab::rejectChanges()
 {
-	EditMode lastMode = editMode;
-
-	if (lastMode != NONE && current_dive &&
-	    (modified ||
-	     memcmp(&current_dive->cylinder[0], &displayed_dive.cylinder[0], sizeof(cylinder_t) * MAX_CYLINDERS) ||
-	     !weightsystems_equal(current_dive, &displayed_dive))) {
+	if (editMode && current_dive) {
 		if (QMessageBox::warning(MainWindow::instance(), TITLE_OR_TEXT(tr("Discard the changes?"),
 									       tr("You are about to discard your changes.")),
 					 QMessageBox::Discard | QMessageBox::Cancel, QMessageBox::Discard) != QMessageBox::Discard) {
@@ -698,9 +534,7 @@ void MainTab::rejectChanges()
 		}
 	}
 	ui.dateEdit->setEnabled(true);
-	editMode = NONE;
-	tabBar()->setTabIcon(0, QIcon()); // Notes
-	tabBar()->setTabIcon(1, QIcon()); // Equipment
+	editMode = false;
 	hideMessage();
 	// no harm done to call cancelPlan even if we were not PLAN mode...
 	DivePlannerPointsModel::instance()->cancelPlan();
@@ -712,21 +546,9 @@ void MainTab::rejectChanges()
 		clear_dive(&displayed_dive);
 	updateDiveInfo();
 
-	for (auto widget: extraWidgets) {
-		widget->updateData();
-	}
-
-	// TODO: This is a temporary hack until the equipment tab is included in the undo system:
-	// The equipment tab is hardcoded at the first place of the "extra widgets".
-	((TabDiveEquipment *)extraWidgets[0])->rejectChanges();
-
-	// the user could have edited the location and then canceled the edit
-	// let's get the correct location back in view
-	MapWidget::instance()->centerOnDiveSite(current_dive ? current_dive->dive_site : nullptr);
 	// show the profile and dive info
 	MainWindow::instance()->graphics->replot();
 	MainWindow::instance()->setEnabledToolbar(true);
-	MainWindow::instance()->exitEditState();
 	ui.editDiveSiteButton->setEnabled(!ui.location->text().isEmpty());
 }
 
@@ -740,17 +562,9 @@ void MainTab::divesEdited(int i)
 	ui.multiDiveWarningMessage->show();
 }
 
-static QStringList stringToList(const QString &s)
-{
-	QStringList res = s.split(",", QString::SkipEmptyParts);
-	for (QString &str: res)
-		str = str.trimmed();
-	return res;
-}
-
 void MainTab::on_buddy_editingFinished()
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 
 	divesEdited(Command::editBuddies(stringToList(ui.buddy->toPlainText()), false));
@@ -758,7 +572,7 @@ void MainTab::on_buddy_editingFinished()
 
 void MainTab::on_divemaster_editingFinished()
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 
 	divesEdited(Command::editDiveMaster(stringToList(ui.divemaster->toPlainText()), false));
@@ -766,7 +580,7 @@ void MainTab::on_divemaster_editingFinished()
 
 void MainTab::on_duration_editingFinished()
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 
 	// Duration editing is special: we only edit the current dive.
@@ -775,62 +589,27 @@ void MainTab::on_duration_editingFinished()
 
 void MainTab::on_depth_editingFinished()
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 
 	// Depth editing is special: we only edit the current dive.
 	divesEdited(Command::editDepth(parseLengthToMm(ui.depth->text()), true));
 }
 
-void MainTab::on_airtemp_editingFinished()
-{
-	// If the field wasn't modified by the user, don't post a new undo command.
-	// Owing to rounding errors, this might lead to undo commands that have
-	// no user visible effects. These can be very confusing.
-	if (editMode == IGNORE_MODE || !ui.airtemp->isModified() || !current_dive)
-		return;
-	divesEdited(Command::editAirTemp(parseTemperatureToMkelvin(ui.airtemp->text()), false));
-}
-
-void MainTab::divetype_Changed(int index)
-{
-	if (editMode == IGNORE_MODE || !current_dive)
-		return;
-	divesEdited(Command::editMode(dc_number, (enum divemode_t)index, false));
-}
-
-void MainTab::on_watertemp_editingFinished()
-{
-	// If the field wasn't modified by the user, don't post a new undo command.
-	// Owing to rounding errors, this might lead to undo commands that have
-	// no user visible effects. These can be very confusing.
-	if (editMode == IGNORE_MODE || !ui.watertemp->isModified() || !current_dive)
-		return;
-	divesEdited(Command::editWaterTemp(parseTemperatureToMkelvin(ui.watertemp->text()), false));
-}
-
 // Editing of the dive time is different. If multiple dives are edited,
 // all dives are shifted by an offset.
 static void shiftTime(QDateTime &dateTime)
 {
-	QVector<dive *> dives;
-	struct dive *d;
-	int i;
-	for_each_dive (i, d) {
-		if (d->selected)
-			dives.append(d);
-	}
-
 	timestamp_t when = dateTime.toTime_t();
 	if (current_dive && current_dive->when != when) {
 		timestamp_t offset = when - current_dive->when;
-		Command::shiftTime(dives, (int)offset);
+		Command::shiftTime(getDiveSelection(), (int)offset);
 	}
 }
 
 void MainTab::on_dateEdit_dateChanged(const QDate &date)
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 	QDateTime dateTime = QDateTime::fromMSecsSinceEpoch(1000*current_dive->when, Qt::UTC);
 	dateTime.setTimeSpec(Qt::UTC);
@@ -840,7 +619,7 @@ void MainTab::on_dateEdit_dateChanged(const QDate &date)
 
 void MainTab::on_timeEdit_timeChanged(const QTime &time)
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 	QDateTime dateTime = QDateTime::fromMSecsSinceEpoch(1000*current_dive->when, Qt::UTC);
 	dateTime.setTimeSpec(Qt::UTC);
@@ -850,7 +629,7 @@ void MainTab::on_timeEdit_timeChanged(const QTime &time)
 
 void MainTab::on_tagWidget_editingFinished()
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 
 	divesEdited(Command::editTags(ui.tagWidget->getBlockStringList(), false));
@@ -858,7 +637,7 @@ void MainTab::on_tagWidget_editingFinished()
 
 void MainTab::on_location_diveSiteSelected()
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 
 	struct dive_site *newDs = ui.location->currDiveSite();
@@ -896,20 +675,12 @@ void MainTab::on_notes_editingFinished()
 
 void MainTab::on_rating_valueChanged(int value)
 {
-	if (editMode == IGNORE_MODE || !current_dive)
+	if (ignoreInput || !current_dive)
 		return;
 
 	divesEdited(Command::editRating(value, false));
 }
 
-void MainTab::on_visibility_valueChanged(int value)
-{
-	if (editMode == IGNORE_MODE || !current_dive)
-		return;
-
-	divesEdited(Command::editVisibility(value, false));
-}
-
 // Remove focus from any active field to update the corresponding value in the dive.
 // Do this by setting the focus to ourself
 void MainTab::stealFocus()
@@ -921,7 +692,7 @@ void MainTab::escDetected()
 {
 	// In edit mode, pressing escape cancels the current changes.
 	// In standard mode, remove focus of any active widget to
-	if (editMode != NONE)
+	if (editMode)
 		rejectChanges();
 	else
 		stealFocus();
