@@ -17,7 +17,7 @@
 #include "qt-models/completionmodels.h"
 #include "qt-models/divelocationmodel.h"
 #include "core/dive.h"
-#include "core/subsurface-qt/DiveListNotifier.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 
 class ExtraDataModel;
 class DivePictureModel;
@@ -26,13 +26,6 @@ class TabBase;
 class MainTab : public QTabWidget {
 	Q_OBJECT
 public:
-	enum EditMode {
-		NONE,
-		DIVE,
-		MANUALLY_ADDED_DIVE,
-		IGNORE_MODE
-	};
-
 	MainTab(QWidget *parent = 0);
 	~MainTab();
 	void clearTabs();
@@ -51,7 +44,6 @@ slots:
 	void tripChanged(dive_trip *trip, TripField field);
 	void updateDiveInfo();
 	void updateNotes(const struct dive *d);
-	void updateMode(struct dive *d);
 	void updateDateTime(const struct dive *d);
 	void updateTripDate(const struct dive_trip *t);
 	void updateDiveSite(struct dive *d);
@@ -63,30 +55,25 @@ slots:
 	void on_buddy_editingFinished();
 	void on_diveTripLocation_editingFinished();
 	void on_notes_editingFinished();
-	void on_airtemp_editingFinished();
 	void on_duration_editingFinished();
 	void on_depth_editingFinished();
-	void divetype_Changed(int);
-	void on_watertemp_editingFinished();
 	void on_dateEdit_dateChanged(const QDate &date);
 	void on_timeEdit_timeChanged(const QTime & time);
 	void on_rating_valueChanged(int value);
-	void on_visibility_valueChanged(int value);
 	void on_tagWidget_editingFinished();
 	void hideMessage();
 	void closeMessage();
 	void closeWarning();
 	void displayMessage(QString str);
-	void enableEdition(EditMode newEditMode = NONE);
-	void updateTextLabels(bool showUnits = true);
+	void enableEdition();
 	void escDetected(void);
 private:
 	Ui::MainTab ui;
-	EditMode editMode;
+	bool editMode;
+	bool ignoreInput; // When computionally editing fields, we have to ignore changed-signals
 	BuddyCompletionModel buddyModel;
 	DiveMasterCompletionModel diveMasterModel;
 	TagCompletionModel tagModel;
-	bool modified;
 	bool lastSelectedDive;
 	int lastTabSelectedDive;
 	int lastTabSelectedDiveTrip;
