@@ -4,7 +4,8 @@
 #include <list>
 
 #include "templatelayout.h"
-#include "core/display.h"
+#include "core/divelist.h"
+#include "core/selection.h"
 
 QList<QString> grantlee_templates, grantlee_statistics_templates;
 
@@ -100,7 +101,7 @@ TemplateLayout::TemplateLayout(print_options *printOptions, template_options *te
 }
 
 /* a HTML pre-processor stage. acts like a compatibility layer
- * between some Grantlee variables and DiveObjectHelper Q_PROPERTIES:
+ * between some Grantlee variables and DiveObjectHelperGrantlee Q_PROPERTIES:
  *	dive.weights -> dive.weightList
  * 	dive.weight# -> dive.weights.#
  *	dive.cylinders -> dive.cylinderList
@@ -134,19 +135,13 @@ QString TemplateLayout::generate()
 	Grantlee::registerMetaType<template_options>();
 	Grantlee::registerMetaType<print_options>();
 	Grantlee::registerMetaType<CylinderObjectHelper>(); // TODO: Remove when grantlee supports Q_GADGET
+	Grantlee::registerMetaType<DiveObjectHelperGrantlee>(); // TODO: Remove when grantlee supports Q_GADGET
 
-	// Note: Currently, this should not be transformed into a QVector<> or std::vector<>,
-	// as diveList contains pointers to elements in this list. But vectors might relocate
-	// and thus invalidate the pointers! std::list<> is used here, because the new elements
-	// can be directly constructed in the list with the emplace_back() call.
-	// Ultimately, the memory management should be fixed.
-	std::list<DiveObjectHelper> diveObjectList;
 	QVariantList diveList;
 
 	struct dive *dive;
 	if (in_planner()) {
-		diveObjectList.emplace_back(&displayed_dive);
-		diveList.append(QVariant::fromValue(&diveObjectList.back()));
+		diveList.append(QVariant::fromValue(DiveObjectHelperGrantlee(&displayed_dive)));
 		emit progressUpdated(100.0);
 	} else {
 		int i;
@@ -154,8 +149,7 @@ QString TemplateLayout::generate()
 			//TODO check for exporting selected dives only
 			if (!dive->selected && printOptions->print_selected)
 				continue;
-			diveObjectList.emplace_back(dive);
-			diveList.append(QVariant::fromValue(&diveObjectList.back()));
+			diveList.append(QVariant::fromValue(DiveObjectHelperGrantlee(dive)));
 			progress++;
 			emit progressUpdated(lrint(progress * 100.0 / totalWork));
 		}
@@ -198,6 +192,7 @@ QString TemplateLayout::generateStatistics()
 	Grantlee::registerMetaType<template_options>();
 	Grantlee::registerMetaType<print_options>();
 	Grantlee::registerMetaType<CylinderObjectHelper>(); // TODO: Remove when grantlee supports Q_GADGET
+	Grantlee::registerMetaType<DiveObjectHelperGrantlee>(); // TODO: Remove when grantlee supports Q_GADGET
 
 	QVariantList years;
 
