@@ -39,7 +39,7 @@ SUBSURFACE_DESKTOP=OFF
 # Which arch should we build for?
 ARCH=arm
 # Read build variables
-source subsurface/packaging/android/variables.sh 
+source $SUBSURFACE_SOURCE/packaging/android/variables.sh
 
 QUICK=""
 
@@ -63,7 +63,7 @@ while [ "$#" -gt 0 ] ; do
 			SUBSURFACE_DESKTOP=ON
 			shift
 			;;
-		arm|arm64|x86)
+		arm|arm64|x86|x86_64)
 			ARCH=$1
 			shift
 			;;
@@ -93,16 +93,8 @@ if [ -n "${QT5_ANDROID+X}" ] ; then
 	echo "Using Qt5 in $QT5_ANDROID"
 elif [ -d "$SUBSURFACE_SOURCE/../Qt/${LATEST_QT}" ] ; then
 	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/${LATEST_QT}
-elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.9.3" ] ; then
-	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.9.3
-elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.9.1" ] ; then
-	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.9.1
-elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.9" ] ; then
-	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.9
-elif [ -d "$SUBSURFACE_SOURCE/../Qt/5.8" ] ; then
-	export QT5_ANDROID=$SUBSURFACE_SOURCE/../Qt/5.8
 else
-	echo "Cannot find Qt 5.8 or newer under $SUBSURFACE_SOURCE/../Qt"
+	echo "Cannot find Qt 5.12 or newer under $SUBSURFACE_SOURCE/../Qt"
 	exit 1
 fi
 
@@ -129,6 +121,11 @@ elif [ "$ARCH" = "x86" ] ; then
 	BUILDCHAIN=i686-linux-android
 	OPENSSL_MACHINE=i686
 	ANDROID_ABI=x86
+elif [ "$ARCH" = "x86_64" ] ; then
+	QT_ARCH=$ARCH
+	BUILDCHAIN=x86_64-linux-android
+	OPENSSL_MACHINE=$ARCH
+	ANDROID_ABI=$ARCH
 fi
 
 # Verify Qt install and adjust for single-arch Qt install layout
@@ -146,9 +143,11 @@ fi
 if [ ! -e ndk-"$ARCH" ] ; then
 	"$ANDROID_NDK_ROOT/build/tools/make_standalone_toolchain.py" --arch="$ARCH" --install-dir=ndk-"$ARCH" --api=$ANDROID_PLATFORM_LEVEL
 fi
+
 export BUILDROOT=$PWD
+mkdir -p "${BUILDROOT}"/install-root-"${ARCH}"
+export PREFIX="${BUILDROOT}"/install-root-"${ARCH}"
 export PATH=${BUILDROOT}/ndk-$ARCH/bin:$PATH
-export PREFIX=${BUILDROOT}/ndk-$ARCH/sysroot/usr
 export PKG_CONFIG_LIBDIR=$PREFIX/lib/pkgconfig
 export CC=${BUILDROOT}/ndk-$ARCH/bin/clang
 export CXX=${BUILDROOT}/ndk-$ARCH/bin/clang++
@@ -247,28 +246,24 @@ if [ "$QUICK" = "" ] ; then
 		# Use env to make all these temporary, so they don't pollute later builds.
 		env	PATH=$ANDROID_NDK_HOME/toolchains/llvm/prebuilt/linux-x86_64/bin:$PATH \
 			CC=clang \
-			./Configure android-"$ARCH" no-ssl2 no-ssl3 no-comp no-hw no-engine no-asm --prefix="$PREFIX" -DOPENSSL_NO_UI_CONSOLE -DOPENSSL_NO_STDIO
+			./Configure shared android-"$ARCH" no-ssl2 no-ssl3 no-comp no-hw no-engine no-asm \
+				--prefix="$PREFIX" -DOPENSSL_NO_UI_CONSOLE -DOPENSSL_NO_STDIO \
+				-D__ANDROID_API__=$ANDROID_PLATFORM_LEVEL
 		make depend
-		make build_libs
-
-		# now fix the reference to libcrypto.so.1.1 and libssl.so.1.1 to be just unversioned
-		# as androiddeployqt and Android itself does not like versioned shared objects
-		perl -pi -e 's/libcrypto.so.1.1/libcrypto.so\x00\x00\x00\x00/' libcrypto.so.1.1
-		perl -pi -e 's/libcrypto.so.1.1/libcrypto.so\x00\x00\x00\x00/' libssl.so.1.1
-		perl -pi -e 's/libssl.so.1.1/libssl.so\x00\x00\x00\x00/' libcrypto.so.1.1
-		perl -pi -e 's/libssl.so.1.1/libssl.so\x00\x00\x00\x00/' libssl.so.1.1
+		# follow the suggestions here: https://doc.qt.io/qt-5/android-openssl-support.html
+		make SHLIB_VERSION_NUMBER= SHLIB_EXT=_1_1.so build_libs
 
 		cp -RL include/openssl $PREFIX/include/openssl
 		cp libcrypto.a $PREFIX/lib
-		cp libcrypto.so* $PREFIX/lib
+		cp libcrypto_1_1.so* $PREFIX/lib
 		cp libssl.a $PREFIX/lib
-		cp libssl.so* $PREFIX/lib
+		cp libssl_1_1.so* $PREFIX/lib
 		cp *.pc $PKG_CONFIG_LIBDIR
 
 		popd
 	fi
 
-"${SUBSURFACE_SOURCE}"/scripts/get-dep-lib.sh singleAndroid . libzip
+	"${SUBSURFACE_SOURCE}"/scripts/get-dep-lib.sh singleAndroid . libzip
 	if [ ! -e "$PKG_CONFIG_LIBDIR/libzip.pc" ] ; then
 		# libzip expects a predefined macro that isn't there for our compiler
 		pushd libzip
@@ -276,6 +271,7 @@ if [ "$QUICK" = "" ] ; then
 		sed -i 's/SIZEOF_SIZE_T/__SIZEOF_SIZE_T__/g' lib/compat.h
 		# also, don't deal with manuals and bzip2
 		sed -i 's/ADD_SUBDIRECTORY(man)//' CMakeLists.txt
+		sed -i 's/^FIND_PACKAGE(ZLIB/#&/' CMakeLists.txt
 		popd
 		mkdir -p libzip-build-"$ARCH"
 		pushd libzip-build-"$ARCH"
@@ -286,6 +282,8 @@ if [ "$QUICK" = "" ] ; then
 			-DCMAKE_INSTALL_LIBDIR="lib" \
 			-DBUILD_SHARED_LIBS=OFF \
 			-DCMAKE_DISABLE_FIND_PACKAGE_BZip2=TRUE \
+			-DZLIB_VERSION_STRING=1.2.7 \
+			-DZLIB_LIBRARY=z \
 			../libzip/
 		make
 		make install
@@ -304,9 +302,9 @@ if [ "$QUICK" = "" ] ; then
 			-DCMAKE_INSTALL_PREFIX="$PREFIX" \
 			-DCURL=OFF \
 			-DUSE_SSH=OFF \
-			-DOPENSSL_SSL_LIBRARY="$PREFIX"/lib/libssl.so \
-			-DOPENSSL_CRYPTO_LIBRARY="$PREFIX"/lib/libcrypto.so \
-			-DOPENSSL_INCLUDE_DIR="$PREFIX"/include/openssl \
+			-DOPENSSL_SSL_LIBRARY="$PREFIX"/lib/libssl_1_1.so \
+			-DOPENSSL_CRYPTO_LIBRARY="$PREFIX"/lib/libcrypto_1_1.so \
+			-DOPENSSL_INCLUDE_DIR="$PREFIX"/include \
 			-D_OPENSSL_VERSION="${OPENSSL_VERSION}" \
 			-DCMAKE_DISABLE_FIND_PACKAGE_HTTP_Parser=TRUE \
 			../libgit2/
@@ -317,43 +315,6 @@ if [ "$QUICK" = "" ] ; then
 		popd
 	fi
 
-	"${SUBSURFACE_SOURCE}"/scripts/get-dep-lib.sh singleAndroid . libusb
-	if ! grep -q libusb_set_android_open_callback libusb/libusb/libusb.h ; then
-		# Patch in our libusb callback
-		pushd libusb
-		patch -p1 < "$SUBSURFACE_SOURCE"/packaging/android/patches/libusb-android.patch
-		popd
-	fi
-	if [ ! -e libusb/configure ] ; then
-		pushd libusb
-		mkdir m4
-		autoreconf -i
-		popd
-	fi
-	if [ ! -e "$PKG_CONFIG_LIBDIR/libusb-1.0.pc" ] ; then
-		mkdir -p libusb-build-"$ARCH"
-		pushd libusb-build-"$ARCH"
-		../libusb/configure --host=${BUILDCHAIN} --prefix="$PREFIX" --enable-static --disable-shared --disable-udev --enable-system-log
-		# --enable-debug-log
-		make
-		make install
-		popd
-	fi
-
-	"${SUBSURFACE_SOURCE}"/scripts/get-dep-lib.sh singleAndroid . libftdi1
-	if [ ! -e "$PKG_CONFIG_LIBDIR/libftdi1.pc" ] && [ "$PLATFORM" != "Darwin" ] ; then
-		mkdir -p libftdi1-build-"$ARCH"
-		pushd libftdi1-build-"$ARCH"
-		cmake ../libftdi1 -DCMAKE_C_COMPILER="$CC" -DCMAKE_INSTALL_PREFIX="$PREFIX" -DCMAKE_PREFIX_PATH="$PREFIX" -DSTATICLIBS=ON -DPYTHON_BINDINGS=OFF -DDOCUMENTATION=OFF -DFTDIPP=OFF -DBUILD_TESTS=OFF -DEXAMPLES=OFF -DFTDI_EEPROM=OFF
-		make
-		make install
-		popd
-	fi
-	# Blast away the shared version to force static linking
-	if [ -e "$PREFIX/lib/libftdi1.so" ] ; then
-		rm "$PREFIX"/lib/libftdi1.so*
-	fi
-
 fi # QUICK
 
 pushd "$SUBSURFACE_SOURCE"
@@ -404,12 +365,6 @@ else
 	BUILD_NAME=Subsurface
 fi
 
-if [ "$PLATFORM" = "Darwin" ] ; then
-	FTDI=OFF
-else
-	FTDI=ON
-fi
-
 PKGCONF=$(which pkg-config)
 cmake $MOBILE_CMAKE \
 	-DCMAKE_SYSTEM_NAME="Android" \
@@ -430,9 +385,9 @@ cmake $MOBILE_CMAKE \
 	-DCMAKE_PREFIX_PATH:UNINITIALIZED="$QT5_ANDROID_CMAKE" \
 	-DCMAKE_BUILD_TYPE="$BUILD_TYPE" \
 	-DMAKE_TESTS=OFF \
-	-DFTDISUPPORT=${FTDI} \
-	-DANDROID_NATIVE_LIBSSL="$BUILDROOT/ndk-$ARCH/sysroot/usr/lib/libssl.so" \
-	-DANDROID_NATIVE_LIBCRYPT="$BUILDROOT/ndk-$ARCH/sysroot/usr/lib/libcrypto.so" \
+	-DFTDISUPPORT=OFF \
+	-DANDROID_NATIVE_LIBSSL="$PREFIX/lib/libssl_1_1.so" \
+	-DANDROID_NATIVE_LIBCRYPT="$PREFIX/lib/libcrypto_1_1.so" \
 	-DCMAKE_MAKE_PROGRAM="make" \
 	"$SUBSURFACE_SOURCE"
 
