@@ -195,7 +195,6 @@ void DiveEventItem::setupToolTipString(struct gasmix lastgasmix)
 			name += QString::fromUtf8(mb.buffer, mb.len);
 			free_buffer(&mb);
 		}
-		lastgasmix = mix;
 	} else if (same_string(internalEvent->name, "modechange")) {
 		name += QString(": %1").arg(gettextFromC::tr(divemode_text_ui[internalEvent->value]));
 	} else if (value) {
@@ -262,7 +261,7 @@ int DiveEventItem::depthAtTime(int time)
 {
 	QModelIndexList result = dataModel->match(dataModel->index(0, DivePlotDataModel::TIME), Qt::DisplayRole, time);
 	if (result.isEmpty()) {
-		Q_ASSERT("can't find a spot in the dataModel");
+		qWarning("can't find a spot in the dataModel");
 		hide();
 		return DEPTH_NOT_FOUND;
 	}
@@ -276,7 +275,7 @@ void DiveEventItem::recalculatePos(int speed)
 
 	QModelIndexList result = dataModel->match(dataModel->index(0, DivePlotDataModel::TIME), Qt::DisplayRole, internalEvent->time.seconds);
 	if (result.isEmpty()) {
-		Q_ASSERT("can't find a spot in the dataModel");
+		qWarning("can't find a spot in the dataModel");
 		hide();
 		return;
 	}
