@@ -1,8 +1,10 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "profile-widget/profilewidget2.h"
 #include "qt-models/diveplotdatamodel.h"
+#include "core/divecomputer.h"
 #include "core/subsurface-string.h"
 #include "core/qthelper.h"
+#include "core/picture.h"
 #include "core/profile.h"
 #include "core/settings/qPrefDisplay.h"
 #include "core/settings/qPrefTechnicalDetails.h"
@@ -24,11 +26,12 @@
 #include "desktop-widgets/diveplanner.h"
 #include "desktop-widgets/simplewidgets.h"
 #include "desktop-widgets/divepicturewidget.h"
-#include "desktop-widgets/command.h"
 #include "desktop-widgets/mainwindow.h"
+#include "commands/command.h"
 #include "core/qthelper.h"
 #include "core/gettextfromc.h"
 #include "core/imagedownloader.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 #endif
 
 #include <libdivecomputer/parser.h>
@@ -169,6 +172,8 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	connect(DivePictureModel::instance(), &DivePictureModel::rowsInserted, this, &ProfileWidget2::plotPictures);
 	connect(DivePictureModel::instance(), &DivePictureModel::picturesRemoved, this, &ProfileWidget2::removePictures);
 	connect(DivePictureModel::instance(), &DivePictureModel::modelReset, this, &ProfileWidget2::plotPictures);
+	connect(&diveListNotifier, &DiveListNotifier::cylinderEdited, this, &ProfileWidget2::profileChanged);
+	connect(&diveListNotifier, &DiveListNotifier::eventsChanged, this, &ProfileWidget2::profileChanged);
 #endif // SUBSURFACE_MOBILE
 
 #if !defined(QT_NO_DEBUG) && defined(SHOW_PLOT_INFO_TABLE)
@@ -178,6 +183,11 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 #endif
 }
 
+ProfileWidget2::~ProfileWidget2()
+{
+	free_plot_info_data(&plotInfo);
+}
+
 #ifndef SUBSURFACE_MOBILE
 void ProfileWidget2::addActionShortcut(const Qt::Key shortcut, void (ProfileWidget2::*slot)())
 {
@@ -367,14 +377,10 @@ void ProfileWidget2::setupItemOnScene()
 	timeAxis->setLinesVisible(true);
 	profileYAxis->setLinesVisible(true);
 	gasYAxis->setZValue(timeAxis->zValue() + 1);
-
-	replotEnabled = true;
 }
 
 void ProfileWidget2::replot(struct dive *d)
 {
-	if (!replotEnabled)
-		return;
 	dataModel->clear();
 	plotDive(d, true, false);
 }
@@ -560,8 +566,10 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 #endif
 	if (currentState != ADD && currentState != PLAN) {
 		if (!d) {
-			if (!current_dive)
+			if (!current_dive) {
+				setEmptyState();
 				return;
+			}
 			d = current_dive; // display the current dive
 		}
 
@@ -794,6 +802,8 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 		item->setZValue(2);
 		scene()->addItem(item);
 		eventItems.push_back(item);
+		if (event_is_gaschange(event))
+			lastgasmix = get_gasmix_from_event(&displayed_dive, event);
 		event = event->next;
 	}
 
@@ -801,7 +811,7 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 	Q_FOREACH (DiveEventItem *event, eventItems) {
 		event->setVisible(!event->shouldBeHidden());
 	}
-	QString dcText = get_dc_nickname(currentdc->model, currentdc->deviceid);
+	QString dcText = get_dc_nickname(currentdc);
 	if (dcText == "planned dive")
 		dcText = tr("Planned dive");
 	else if (dcText == "manually added dive")
@@ -839,13 +849,6 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 #endif
 }
 
-void ProfileWidget2::recalcCeiling()
-{
-#ifndef SUBSURFACE_MOBILE
-	diveCeiling->recalc();
-#endif
-}
-
 void ProfileWidget2::dateTimeChanged()
 {
 	emit dateTimeChangedItems();
@@ -1136,6 +1139,7 @@ void ProfileWidget2::setEmptyState()
 	ccrsensor3GasItem->setVisible(false);
 	ocpo2GasItem->setVisible(false);
 #ifndef SUBSURFACE_MOBILE
+	toolTipItem->clearPlotInfo();
 	toolTipItem->setVisible(false);
 	diveCeiling->setVisible(false);
 	decoModelParameters->setVisible(false);
@@ -1385,24 +1389,19 @@ bool ProfileWidget2::isPlanner()
 	return currentState == PLAN;
 }
 
-struct plot_data *ProfileWidget2::getEntryFromPos(QPointF pos)
+#if 0 // TODO::: FINISH OR DISABLE
+struct int ProfileWidget2::getEntryFromPos(QPointF pos)
 {
 	// find the time stamp corresponding to the mouse position
 	int seconds = lrint(timeAxis->valueAt(pos));
-	struct plot_data *entry = NULL;
 
 	for (int i = 0; i < plotInfo.nr; i++) {
-		entry = plotInfo.entry + i;
-		if ((int)entry->sec >= seconds)
-			break;
+		if (plotInfo.entry[i].sec >= seconds)
+			return i;
 	}
-	return entry;
-}
-
-void ProfileWidget2::setReplot(bool state)
-{
-	replotEnabled = state;
+	return plotInfo.nr - 1;
 }
+#endif
 
 #ifndef SUBSURFACE_MOBILE
 void ProfileWidget2::contextMenuEvent(QContextMenuEvent *event)
@@ -1432,10 +1431,10 @@ void ProfileWidget2::contextMenuEvent(QContextMenuEvent *event)
 				return;
 			// create menu to show when right clicking on dive computer name
 			if (dc_number > 0)
-				m.addAction(tr("Make first dive computer"), this, SLOT(makeFirstDC()));
+				m.addAction(tr("Make first dive computer"), this, &ProfileWidget2::makeFirstDC);
 			if (count_divecomputers(current_dive) > 1) {
-				m.addAction(tr("Delete this dive computer"), this, SLOT(deleteCurrentDC()));
-				m.addAction(tr("Split this dive computer into own dive"), this, SLOT(splitCurrentDC()));
+				m.addAction(tr("Delete this dive computer"), this, &ProfileWidget2::deleteCurrentDC);
+				m.addAction(tr("Split this dive computer into own dive"), this, &ProfileWidget2::splitCurrentDC);
 			}
 			m.exec(event->globalPos());
 			// don't show the regular profile context menu
@@ -1443,112 +1442,78 @@ void ProfileWidget2::contextMenuEvent(QContextMenuEvent *event)
 		}
 	}
 	// create the profile context menu
-	GasSelectionModel *model = GasSelectionModel::instance();
-	model->repopulate();
-	int rowCount = model->rowCount();
-	if (rowCount > 1) {
-		// if we have more than one gas, offer to switch to another one
-		QMenu *gasChange = m.addMenu(tr("Add gas change"));
-		for (int i = 0; i < rowCount; i++) {
-			QAction *action = new QAction(&m);
-			action->setText(model->data(model->index(i, 0), Qt::DisplayRole).toString() + QString(tr(" (cyl. %1)")).arg(i + 1));
-			connect(action, SIGNAL(triggered(bool)), this, SLOT(changeGas()));
-			action->setData(event->globalPos());
-			gasChange->addAction(action);
-		}
-	}
-	QAction *setpointAction = m.addAction(tr("Add setpoint change"), this, SLOT(addSetpointChange()));
-	setpointAction->setData(event->globalPos());
-	QAction *action = m.addAction(tr("Add bookmark"), this, SLOT(addBookmark()));
-	action->setData(event->globalPos());
-	QAction *splitAction = m.addAction(tr("Split dive into two"), this, SLOT(splitDive()));
-	splitAction->setData(event->globalPos());
-	const struct event *ev = NULL;
-	enum divemode_t divemode = UNDEF_COMP_TYPE;
 	QPointF scenePos = mapToScene(mapFromGlobal(event->globalPos()));
-	QString gas = action->text();
 	qreal sec_val = timeAxis->valueAt(scenePos);
 	int seconds = (sec_val < 0.0) ? 0 : (int)sec_val;
+	if (current_dive && current_dive->cylinders.nr > 1) {
+		// if we have more than one gas, offer to switch to another one
+		QMenu *gasChange = m.addMenu(tr("Add gas change"));
+		for (int i = 0; i < current_dive->cylinders.nr; i++)
+			gasChange->addAction(QString(current_dive->cylinders.cylinders[i].type.description) + tr(" (cyl. %1)").arg(i + 1),
+					     [this, i, seconds] { changeGas(i, seconds); });
+	}
+	m.addAction(tr("Add setpoint change"), [this, seconds]() { ProfileWidget2::addSetpointChange(seconds); });
+	m.addAction(tr("Add bookmark"), [this, seconds]() { addBookmark(seconds); });
+	m.addAction(tr("Split dive into two"), [this, seconds]() { splitDive(seconds); });
+	const struct event *ev = NULL;
+	enum divemode_t divemode = UNDEF_COMP_TYPE;
 
 	get_current_divemode(current_dc, seconds, &ev, &divemode);
 	QMenu *changeMode = m.addMenu(tr("Change divemode"));
-	if (divemode != OC) {
-		QAction *action = new QAction(&m);
-		action->setText(gettextFromC::tr(divemode_text_ui[OC]));
-		connect(action, SIGNAL(triggered(bool)), this, SLOT(addDivemodeSwitch()));
-		action->setData(event->globalPos());
-		changeMode->addAction(action);
-	}
-	if (divemode != CCR) {
-		QAction *action = new QAction(&m);
-		action->setText(gettextFromC::tr(divemode_text_ui[CCR]));
-		connect(action, SIGNAL(triggered(bool)), this, SLOT(addDivemodeSwitch()));
-		action->setData(event->globalPos());
-		changeMode->addAction(action);
-	}
-	if (divemode != PSCR) {
-		QAction *action = new QAction(&m);
-		action->setText(gettextFromC::tr(divemode_text_ui[PSCR]));
-		connect(action, SIGNAL(triggered(bool)), this, SLOT(addDivemodeSwitch()));
-		action->setData(event->globalPos());
-		changeMode->addAction(action);
-	}
+	if (divemode != OC)
+		changeMode->addAction(gettextFromC::tr(divemode_text_ui[OC]),
+				      [this, seconds](){ addDivemodeSwitch(seconds, OC); });
+	if (divemode != CCR)
+		changeMode->addAction(gettextFromC::tr(divemode_text_ui[CCR]),
+				      [this, seconds](){ addDivemodeSwitch(seconds, CCR); });
+	if (divemode != PSCR)
+		changeMode->addAction(gettextFromC::tr(divemode_text_ui[PSCR]),
+				      [this, seconds](){ addDivemodeSwitch(seconds, PSCR); });
 
 	if (same_string(current_dc->model, "manually added dive"))
 		m.addAction(tr("Edit the profile"), this, SIGNAL(editCurrentDive()));
 
 	if (DiveEventItem *item = dynamic_cast<DiveEventItem *>(sceneItem)) {
-		action = new QAction(&m);
-		action->setText(tr("Remove event"));
-		action->setData(QVariant::fromValue<void *>(item)); // so we know what to remove.
-		connect(action, SIGNAL(triggered(bool)), this, SLOT(removeEvent()));
-		m.addAction(action);
-		action = new QAction(&m);
-		action->setText(tr("Hide similar events"));
-		action->setData(QVariant::fromValue<void *>(item));
-		connect(action, SIGNAL(triggered(bool)), this, SLOT(hideEvents()));
-		m.addAction(action);
+		m.addAction(tr("Remove event"), [this,item] { removeEvent(item); });
+		m.addAction(tr("Hide similar events"), [this, item] { hideEvents(item); });
 		struct event *dcEvent = item->getEvent();
-		if (dcEvent->type == SAMPLE_EVENT_BOOKMARK) {
-			action = new QAction(&m);
-			action->setText(tr("Edit name"));
-			action->setData(QVariant::fromValue<void *>(item));
-			connect(action, SIGNAL(triggered(bool)), this, SLOT(editName()));
-			m.addAction(action);
-		}
-#if 0 // FIXME::: FINISH OR DISABLE
+		if (dcEvent->type == SAMPLE_EVENT_BOOKMARK)
+			m.addAction(tr("Edit name"), [this, item] { editName(item); });
+#if 0 // TODO::: FINISH OR DISABLE
 		QPointF scenePos = mapToScene(event->pos());
-		struct plot_data *entry = getEntryFromPos(scenePos);
+		int idx = getEntryFromPos(scenePos);
 		// this shows how to figure out if we should ask the user if they want adjust interpolated pressures
 		// at either side of a gas change
 		if (dcEvent->type == SAMPLE_EVENT_GASCHANGE || dcEvent->type == SAMPLE_EVENT_GASCHANGE2) {
 			qDebug() << "figure out if there are interpolated pressures";
-			struct plot_data *gasChangeEntry = entry;
-			struct plot_data *newGasEntry;
-			while (gasChangeEntry > plotInfo.entry) {
-				--gasChangeEntry;
-				if (gasChangeEntry->sec <= dcEvent->time.seconds)
+			int gasChangeIdx = idx;
+			while (gasChangeIdx > 0) {
+				--gasChangeIdx;
+				if (plotInfo.entry[gasChangeIdx].sec <= dcEvent->time.seconds)
 					break;
 			}
-			qDebug() << "at gas change at" << gasChangeEntry->sec << ": sensor pressure" << gasChangeEntry->pressure[0] << "interpolated" << gasChangeEntry->pressure[1];
+			const struct plot_data &gasChangeEntry = plotInfo.entry[newGasIdx];
+			qDebug() << "at gas change at" << gasChangeEntry->sec << ": sensor pressure" << get_plot_sensor_pressure(&plotInfo, newGasIdx)
+				 << "interpolated" << ;get_plot_sensor_pressure(&plotInfo, newGasIdx);
 			// now gasChangeEntry points at the gas change, that entry has the final pressure of
 			// the old tank, the next entry has the starting pressure of the next tank
-			if (gasChangeEntry + 1 <= plotInfo.entry + plotInfo.nr) {
-				newGasEntry = gasChangeEntry + 1;
+			if (gasChangeIdx < plotInfo.nr - 1) {
+				int newGasIdx = gasChangeIdx + 1;
+				const struct plot_data &newGasEntry = plotInfo.entry[newGasIdx];
 				qDebug() << "after gas change at " << newGasEntry->sec << ": sensor pressure" << newGasEntry->pressure[0] << "interpolated" << newGasEntry->pressure[1];
-				if (get_plot_sensor_pressure(gasChangeEntry) == 0 || displayed_dive.cylinder[gasChangeEntry->sensor[0]].sample_start.mbar == 0) {
+				if (get_plot_sensor_pressure(&plotInfo, gasChangeIdx) == 0 || get_cylinder(&displayed_dive, gasChangeEntry->sensor[0])->sample_start.mbar == 0) {
 					// if we have no sensorpressure or if we have no pressure from samples we can assume that
 					// we only have interpolated pressure (the pressure in the entry may be stored in the sensor
 					// pressure field if this is the first or last entry for this tank... see details in gaspressures.c
 					pressure_t pressure;
-					pressure.mbar = get_plot_interpolated_pressure(gasChangeEntry) ? : get_plot_sensor_pressure(gasChangeEntry);
+					pressure.mbar = get_plot_interpolated_pressure(&plotInfo, gasChangeIdx) ? : get_plot_sensor_pressure(&plotInfo, gasChangeIdx);
 					QAction *adjustOldPressure = m.addAction(tr("Adjust pressure of cyl. %1 (currently interpolated as %2)")
 										 .arg(gasChangeEntry->sensor[0] + 1).arg(get_pressure_string(pressure)));
 				}
-				if (get_plot_sensor_pressure(newGasEntry) == 0 || displayed_dive.cylinder[newGasEntry->sensor[0]].sample_start.mbar == 0) {
+				if (get_plot_sensor_pressure(&plotInfo, newGasIdx) == 0 || get_cylinder(&displayed_dive, newGasEntry->sensor[0])->sample_start.mbar == 0) {
 					// we only have interpolated press -- see commend above
 					pressure_t pressure;
-					pressure.mbar = get_plot_interpolated_pressure(newGasEntry) ? : get_plot_sensor_pressure(newGasEntry);
+					pressure.mbar = get_plot_interpolated_pressure(&plotInfo, newGasIdx) ? : get_plot_sensor_pressure(&plotInfo, newGasIdx);
 					QAction *adjustOldPressure = m.addAction(tr("Adjust pressure of cyl. %1 (currently interpolated as %2)")
 										 .arg(newGasEntry->sensor[0] + 1).arg(get_pressure_string(pressure)));
 				}
@@ -1563,10 +1528,8 @@ void ProfileWidget2::contextMenuEvent(QContextMenuEvent *event)
 			break;
 		}
 	}
-	if (some_hidden) {
-		action = m.addAction(tr("Unhide all events"), this, SLOT(unhideEvents()));
-		action->setData(event->globalPos());
-	}
+	if (some_hidden)
+		m.addAction(tr("Unhide all events"), this, &ProfileWidget2::unhideEvents);
 	m.exec(event->globalPos());
 }
 
@@ -1585,10 +1548,8 @@ void ProfileWidget2::makeFirstDC()
 	Command::moveDiveComputerToFront(current_dive, dc_number);
 }
 
-void ProfileWidget2::hideEvents()
+void ProfileWidget2::hideEvents(DiveEventItem *item)
 {
-	QAction *action = qobject_cast<QAction *>(sender());
-	DiveEventItem *item = static_cast<DiveEventItem *>(action->data().value<void *>());
 	struct event *event = item->getEvent();
 
 	if (QMessageBox::question(this,
@@ -1620,115 +1581,68 @@ void ProfileWidget2::unhideEvents()
 		item->show();
 }
 
-void ProfileWidget2::removeEvent()
+// The profile displays a copy of the current_dive, namely displayed_dive.
+// Therefore, the events we get are likewise copies. This function finds
+// the original event. TODO: Remove function once the profile can display
+// arbitrary dives.
+static event *find_event(const struct event *ev)
 {
-	QAction *action = qobject_cast<QAction *>(sender());
-	DiveEventItem *item = static_cast<DiveEventItem *>(action->data().value<void *>());
-	struct event *event = item->getEvent();
+	struct divecomputer *dc = current_dc;
+	if (!dc)
+		return nullptr;
+	for (struct event *act = current_dc->events; act; act = act->next) {
+		if (same_event(act, ev))
+			return act;
+	}
+	return nullptr;
+}
+
+void ProfileWidget2::removeEvent(DiveEventItem *item)
+{
+	struct event *event = find_event(item->getEvent());
+	if (!event)
+		return;
 
 	if (QMessageBox::question(this, TITLE_OR_TEXT(
 					  tr("Remove the selected event?"),
 					  tr("%1 @ %2:%3").arg(event->name).arg(event->time.seconds / 60).arg(event->time.seconds % 60, 2, 10, QChar('0'))),
-				  QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok) {
-		remove_event(event);
-		invalidate_dive_cache(current_dive);
-		mark_divelist_changed(true);
-		replot();
-	}
+				  QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok)
+		Command::removeEvent(current_dive, dc_number, event);
 }
 
-void ProfileWidget2::addBookmark()
+void ProfileWidget2::addBookmark(int seconds)
 {
-	QAction *action = qobject_cast<QAction *>(sender());
-	QPointF scenePos = mapToScene(mapFromGlobal(action->data().toPoint()));
-	add_event(current_dc, lrint(timeAxis->valueAt(scenePos)), SAMPLE_EVENT_BOOKMARK, 0, 0, "bookmark");
-	invalidate_dive_cache(current_dive);
-	mark_divelist_changed(true);
-	replot();
+	Command::addEventBookmark(current_dive, dc_number, seconds);
 }
 
-void ProfileWidget2::addDivemodeSwitch()
+void ProfileWidget2::addDivemodeSwitch(int seconds, int divemode)
 {
-	int i;
-	QAction *action = qobject_cast<QAction *>(sender());
-	QPointF scenePos = mapToScene(mapFromGlobal(action->data().toPoint()));
-	for (i = 0; i < UNDEF_COMP_TYPE; i++)
-		if (gettextFromC::tr(divemode_text_ui[i]) == action->text())
-			add_event(current_dc, lrint(timeAxis->valueAt(scenePos)), 8, 0, i,
-				QT_TRANSLATE_NOOP("gettextFromC", "modechange"));
-	invalidate_dive_cache(current_dive);
-	mark_divelist_changed(true);
-	replot();
+	Command::addEventDivemodeSwitch(current_dive, dc_number, seconds, divemode);
 }
 
-void ProfileWidget2::addSetpointChange()
+void ProfileWidget2::addSetpointChange(int seconds)
 {
-	QAction *action = qobject_cast<QAction *>(sender());
-	QPointF scenePos = mapToScene(mapFromGlobal(action->data().toPoint()));
-	SetpointDialog::instance()->setpointData(current_dc, lrint(timeAxis->valueAt(scenePos)));
-	SetpointDialog::instance()->show();
+	SetpointDialog dialog(current_dive, dc_number, seconds);
+	dialog.exec();
 }
 
-void ProfileWidget2::splitDive()
+void ProfileWidget2::splitDive(int seconds)
 {
 #ifndef SUBSURFACE_MOBILE
 	// Make sure that this is an actual dive and we're not in add mode
 	dive *d = get_dive_by_uniq_id(displayed_dive.id);
 	if (!d)
 		return;
-	QAction *action = qobject_cast<QAction *>(sender());
-	QPointF scenePos = mapToScene(mapFromGlobal(action->data().toPoint()));
-	duration_t time;
-	time.seconds = lrint(timeAxis->valueAt(scenePos));
-	Command::splitDives(d, time);
+	Command::splitDives(d, duration_t{ seconds });
 #endif
 }
 
-void ProfileWidget2::changeGas()
+void ProfileWidget2::changeGas(int tank, int seconds)
 {
-	QAction *action = qobject_cast<QAction *>(sender());
-	QPointF scenePos = mapToScene(mapFromGlobal(action->data().toPoint()));
-	QString gas = action->text();
-	gas.remove(QRegExp(" \\(.*\\)"));
+	if (!current_dive || tank < 0 || tank >= current_dive->cylinders.nr)
+		return;
 
-	// backup the things on the dataModel, since we will clear that out.
-	struct gasmix gasmix;
-	qreal sec_val = timeAxis->valueAt(scenePos);
-
-	// no gas changes before the dive starts
-	int seconds = (sec_val < 0.0) ? 0 : (int)sec_val;
-
-	// if there is a gas change at this time stamp, remove it before adding the new one
-	struct event *gasChangeEvent = current_dc->events;
-	while ((gasChangeEvent = get_next_event_mutable(gasChangeEvent, "gaschange")) != NULL) {
-		if (gasChangeEvent->time.seconds == seconds) {
-			remove_event(gasChangeEvent);
-			gasChangeEvent = current_dc->events;
-		} else {
-			gasChangeEvent = gasChangeEvent->next;
-		}
-	}
-	validate_gas(qPrintable(gas), &gasmix);
-	QRegExp rx("\\(\\D*(\\d+)");
-	int tank;
-	if (rx.indexIn(action->text()) > -1) {
-		tank = rx.cap(1).toInt() - 1; // we display the tank 1 based
-	} else {
-		qDebug() << "failed to parse tank number";
-		tank = get_gasidx(&displayed_dive, gasmix);
-	}
-	// add this both to the displayed dive and the current dive
-	add_gas_switch_event(current_dive, current_dc, seconds, tank);
-	add_gas_switch_event(&displayed_dive, get_dive_dc(&displayed_dive, dc_number), seconds, tank);
-	// this means we potentially have a new tank that is being used and needs to be shown
-	fixup_dive(&displayed_dive);
-	invalidate_dive_cache(current_dive);
-
-	// FIXME - this no longer gets written to the dive list - so we need to enableEdition() here
-
-	emit updateDiveInfo();
-	mark_divelist_changed(true);
-	replot();
+	Command::addGasSwitch(current_dive, dc_number, seconds, tank);
 }
 #endif
 
@@ -1774,11 +1688,11 @@ double ProfileWidget2::getFontPrintScale()
 }
 
 #ifndef SUBSURFACE_MOBILE
-void ProfileWidget2::editName()
+void ProfileWidget2::editName(DiveEventItem *item)
 {
-	QAction *action = qobject_cast<QAction *>(sender());
-	DiveEventItem *item = static_cast<DiveEventItem *>(action->data().value<void *>());
-	struct event *event = item->getEvent();
+	struct event *event = find_event(item->getEvent());
+	if (!event)
+		return;
 	bool ok;
 	QString newName = QInputDialog::getText(this, tr("Edit name of bookmark"),
 						tr("Custom name:"), QLineEdit::Normal,
@@ -1790,14 +1704,7 @@ void ProfileWidget2::editName()
 			lengthWarning.exec();
 			return;
 		}
-		// order is important! first update the current dive (by matching the unchanged event),
-		// then update the displayed dive (as event is part of the events on displayed dive
-		// and will be freed as part of changing the name!
-		update_event_name(current_dive, event, qPrintable(newName));
-		update_event_name(&displayed_dive, event, qPrintable(newName));
-		invalidate_dive_cache(current_dive);
-		mark_divelist_changed(true);
-		replot();
+		Command::renameEvent(current_dive, dc_number, event, qPrintable(newName));
 	}
 }
 #endif
@@ -1882,7 +1789,7 @@ void ProfileWidget2::repositionDiveHandlers()
 		QLineF line(p1, p2);
 		QPointF pos = line.pointAt(0.5);
 		gases[i]->setPos(pos);
-		gases[i]->setText(get_gas_string(displayed_dive.cylinder[datapoint.cylinderid].gasmix));
+		gases[i]->setText(get_gas_string(get_cylinder(&displayed_dive, datapoint.cylinderid)->gasmix));
 		gases[i]->setVisible(datapoint.entered &&
 				(i == 0 || gases[i]->text() != gases[i-1]->text()));
 	}
@@ -2059,14 +1966,14 @@ void ProfileWidget2::keyDeleteAction()
 	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 	int selCount = scene()->selectedItems().count();
 	if (selCount) {
-		QVector<int> selectedIndexes;
+		QVector<int> selectedIndices;
 		Q_FOREACH (QGraphicsItem *i, scene()->selectedItems()) {
 			if (DiveHandler *handler = qgraphicsitem_cast<DiveHandler *>(i)) {
-				selectedIndexes.push_back(handles.indexOf(handler));
+				selectedIndices.push_back(handles.indexOf(handler));
 				handler->hide();
 			}
 		}
-		plannerModel->removeSelectedPoints(selectedIndexes);
+		plannerModel->removeSelectedPoints(selectedIndices);
 	}
 }
 
@@ -2267,6 +2174,13 @@ void ProfileWidget2::removePictures(const QVector<QString> &fileUrls)
 	calculatePictureYPositions();
 }
 
+void ProfileWidget2::profileChanged(dive *d)
+{
+	if (!d || d->id != displayed_dive.id)
+		return; // Cylinders of a differnt dive than the shown one changed.
+	replot();
+}
+
 #endif
 
 void ProfileWidget2::dropEvent(QDropEvent *event)
