@@ -73,6 +73,7 @@ public:
 	};
 
 	ProfileWidget2(QWidget *parent = 0);
+	~ProfileWidget2();
 	void resetZoom();
 	void scale(qreal sx, qreal sy);
 	void plotDive(const struct dive *d = 0, bool force = false, bool clearPictures = false, bool instant = false);
@@ -87,7 +88,6 @@ public:
 	bool eventFilter(QObject *, QEvent *) override;
 	void clearHandlers();
 #endif
-	void recalcCeiling();
 	void setToolTipVisibile(bool visible);
 	State currentState;
 	int animSpeed;
@@ -97,7 +97,6 @@ signals:
 	void enableToolbar(bool enable);
 	void enableShortcuts();
 	void disableShortcuts(bool paste);
-	void refreshDisplay(bool recreateDivelist);
 	void updateDiveInfo();
 	void editCurrentDive();
 	void dateTimeChangedItems();
@@ -109,28 +108,16 @@ slots: // Necessary to call from QAction's signals.
 	void actionRequestedReplot(bool triggered);
 	void setEmptyState();
 	void setProfileState();
-	void setReplot(bool state);
 	void replot(dive *d = 0);
 #ifndef SUBSURFACE_MOBILE
 	void plotPictures();
 	void removePictures(const QVector<QString> &fileUrls);
 	void setPlanState();
 	void setAddState();
-	void changeGas();
-	void addSetpointChange();
-	void splitDive();
-	void addBookmark();
-	void addDivemodeSwitch();
-	void hideEvents();
-	void unhideEvents();
-	void removeEvent();
-	void editName();
-	void makeFirstDC();
-	void deleteCurrentDC();
-	void splitCurrentDC();
 	void pointInserted(const QModelIndex &parent, int start, int end);
 	void pointsRemoved(const QModelIndex &, int start, int end);
 	void updateThumbnail(QString filename, QImage thumbnail, duration_t duration);
+	void profileChanged(dive *d);
 
 	/* this is called for every move on the handlers. maybe we can speed up this a bit? */
 	void recreatePlannedDive();
@@ -147,7 +134,7 @@ slots: // Necessary to call from QAction's signals.
 	void divePlannerHandlerReleased();
 #endif
 
-protected:
+private:
 	void resizeEvent(QResizeEvent *event) override;
 #ifndef SUBSURFACE_MOBILE
 	void wheelEvent(QWheelEvent *event) override;
@@ -161,8 +148,7 @@ protected:
 	void dragEnterEvent(QDragEnterEvent *event) override;
 	void dragMoveEvent(QDragMoveEvent *event) override;
 
-
-private: /*methods*/
+	void changeGas(int tank, int seconds);
 	void fixBackgroundPos();
 	void scrollViewTo(const QPoint &pos);
 	void setupSceneAndFlags();
@@ -176,7 +162,18 @@ private: /*methods*/
 			 const double *thresholdSettingsMin, const double *thresholdSettingsMax);
 	void clearPictures();
 	void plotPicturesInternal(const struct dive *d, bool synchronous);
-private:
+	void addDivemodeSwitch(int seconds, int divemode);
+	void addBookmark(int seconds);
+	void splitDive(int seconds);
+	void addSetpointChange(int seconds);
+	void removeEvent(DiveEventItem *item);
+	void hideEvents(DiveEventItem *item);
+	void editName(DiveEventItem *item);
+	void unhideEvents();
+	void makeFirstDC();
+	void deleteCurrentDC();
+	void splitCurrentDC();
+
 	DivePlotDataModel *dataModel;
 	int zoomLevel;
 	qreal zoomFactor;
@@ -186,7 +183,6 @@ private:
 	ToolTipItem *toolTipItem;
 #endif
 	bool isPlotZoomed;
-	bool replotEnabled;
 	// All those here should probably be merged into one structure,
 	// So it's esyer to replicate for more dives later.
 	// In the meantime, keep it here.
