@@ -4,6 +4,8 @@
 #include <QAbstractTableModel>
 #include <vector>
 #include "core/divesite.h"
+#include "core/divelist.h"
+#include "core/downloadfromdcthread.h"
 
 class DiveImportedModel : public QAbstractTableModel
 {
@@ -16,12 +18,18 @@ public:
 	int rowCount(const QModelIndex& index = QModelIndex()) const;
 	QVariant data(const QModelIndex& index, int role) const;
 	QVariant headerData(int section, Qt::Orientation orientation, int role) const;
-	void setImportedDivesIndexes(int first, int last);
+	void setImportedDivesIndices(int first, int last);
 	Qt::ItemFlags flags(const QModelIndex &index) const;
 	Q_INVOKABLE void clearTable();
 	QHash<int, QByteArray> roleNames() const;
-	Q_INVOKABLE void repopulate(dive_table_t *table, dive_site_table_t *sites);
-	Q_INVOKABLE void recordDives();
+	void deleteDeselected();
+	std::pair<struct dive_table, struct dive_site_table> consumeTables(); // Returns dives and sites and resets model.
+
+	int numDives() const;
+	Q_INVOKABLE void recordDives(int flags = IMPORT_PREFER_IMPORTED | IMPORT_IS_DOWNLOADED);
+	Q_INVOKABLE void startDownload();
+
+	DownloadThread thread;
 public
 slots:
 	void changeSelected(QModelIndex clickedIndex);
@@ -29,12 +37,17 @@ slots:
 	void selectAll();
 	void selectNone();
 
+private
+slots:
+	void downloadThreadFinished();
+
+signals:
+	void downloadFinished();
+
 private:
-	int firstIndex;
-	int lastIndex;
 	std::vector<char> checkStates; // char instead of bool to avoid silly pessimization of std::vector.
-	struct dive_table *diveTable;
-	struct dive_site_table *sitesTable;
+	struct dive_table diveTable;
+	struct dive_site_table sitesTable;
 };
 
 #endif
