@@ -1,15 +1,13 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "core/units.h"
 #include "qt-models/divelocationmodel.h"
-#include "core/subsurface-qt/DiveListNotifier.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 #include "core/qthelper.h"
 #include "core/divesite.h"
 #include "core/metrics.h"
 #ifndef SUBSURFACE_MOBILE
 #include "cleanertablemodel.h" // for trashIcon() and editIcon()
-#include "desktop-widgets/mainwindow.h" // to place message box
-#include "desktop-widgets/command.h"
-#include <QMessageBox>
+#include "commands/command.h"
 #endif
 #include <QLineEdit>
 #include <QIcon>
@@ -231,7 +229,7 @@ QStringList DiveSiteSortedModel::allSiteNames() const
 		// (more precisely: the core has more sites than the model is aware of),
 		// we might get an invalid index.
 		if (idx < 0 || idx > dive_site_table.nr) {
-			fprintf(stderr, "DiveSiteSortedModel::allSiteNames(): invalid index");
+			SSRF_INFO("DiveSiteSortedModel::allSiteNames(): invalid index");
 			continue;
 		}
 		locationNames << QString(dive_site_table.dive_sites[idx]->name);
@@ -262,27 +260,6 @@ bool DiveSiteSortedModel::setData(const QModelIndex &index, const QVariant &valu
 	}
 }
 
-// TODO: Remove or edit. It doesn't make sense to call the model here, which calls the undo command,
-// which in turn calls the model.
-void DiveSiteSortedModel::remove(const QModelIndex &index)
-{
-	struct dive_site *ds = getDiveSite(index);
-	if (!ds)
-		return;
-	switch (index.column()) {
-	case LocationInformationModel::EDIT:
-		MainWindow::instance()->editDiveSite(ds);
-		break;
-	case LocationInformationModel::REMOVE:
-		if (ds->dives.nr > 0 &&
-		    QMessageBox::warning(MainWindow::instance(), tr("Delete dive site?"),
-					 tr("This dive site has %n dive(s). Do you really want to delete it?\n", "", ds->dives.nr),
-					 QMessageBox::Yes|QMessageBox::No) == QMessageBox::No)
-				return;
-		Command::deleteDiveSites(QVector<dive_site *>{ds});
-		break;
-	}
-}
 #endif // SUBSURFACE_MOBILE
 
 void DiveSiteSortedModel::setFilter(const QString &text)
@@ -308,8 +285,11 @@ GeoReferencingOptionsModel::GeoReferencingOptionsModel(QObject *parent) : QStrin
 
 bool GPSLocationInformationModel::filterAcceptsRow(int sourceRow, const QModelIndex &parent) const
 {
+	if (!has_location(&location))
+		return false;
+
 	struct dive_site *ds = sourceModel()->index(sourceRow, LocationInformationModel::DIVESITE, parent).data().value<dive_site *>();
-	if (!ds || ds == ignoreDs || ds == RECENTLY_ADDED_DIVESITE)
+	if (!ds || ds == ignoreDs || ds == RECENTLY_ADDED_DIVESITE || !has_location(&ds->location))
 		return false;
 
 	return distance <= 0 ? same_location(&ds->location, &location)
