@@ -1,14 +1,17 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "diveplannermodel.h"
+#include "core/divelist.h"
 #include "core/subsurface-string.h"
 #include "qt-models/cylindermodel.h"
 #include "core/planner.h"
 #include "qt-models/models.h"
 #include "core/device.h"
 #include "core/qthelper.h"
-#include "core/divelist.h" // for mark_divelist_changed()
 #include "core/settings/qPrefDivePlanner.h"
-#include "desktop-widgets/command.h"
+#include "core/settings/qPrefUnit.h"
+#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
+#include "commands/command.h"
+#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
 #include "core/gettextfromc.h"
 #include "core/deco.h"
 #include <QApplication>
@@ -19,9 +22,12 @@
 
 #define UNIT_FACTOR ((prefs.units.length == units::METERS) ? 1000.0 / 60.0 : feet_to_mm(1.0) / 60.0)
 
-/* TODO: Port this to CleanerTableModel to remove a bit of boilerplate and
- * use the signal warningMessage() to communicate errors to the MainWindow.
- */
+CylindersModel *DivePlannerPointsModel::cylindersModel()
+{
+	return &cylinders;
+}
+
+/* TODO: Port this to CleanerTableModel to remove a bit of boilerplate. */
 void DivePlannerPointsModel::removeSelectedPoints(const QVector<int> &rows)
 {
 	if (!rows.count())
@@ -35,7 +41,7 @@ void DivePlannerPointsModel::removeSelectedPoints(const QVector<int> &rows)
 		divepoints.remove(v2[i]);
 	}
 	endRemoveRows();
-	CylindersModel::instance()->updateTrashIcon();
+	cylinders.updateTrashIcon();
 }
 
 void DivePlannerPointsModel::createSimpleDive()
@@ -87,7 +93,7 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 	const struct event *evd = NULL;
 	enum divemode_t current_divemode = UNDEF_COMP_TYPE;
 	recalc = false;
-	CylindersModel::instance()->updateDive();
+	cylinders.updateDive(&displayed_dive);
 	duration_t lasttime = { 0 };
 	duration_t lastrecordedtime = {};
 	duration_t newtime = {};
@@ -145,6 +151,7 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 		addStop(0, d->dc.duration.seconds,cylinderid, last_sp.mbar, true, current_divemode);
 	recalc = oldRec;
 	DiveTypeSelectionModel::instance()->repopulate();
+	preserved_until = d->duration;
 	emitDataChanged();
 }
 
@@ -153,29 +160,32 @@ void DivePlannerPointsModel::loadFromDive(dive *d)
 // setup the cylinder widget accordingly
 void DivePlannerPointsModel::setupCylinders()
 {
-	int i;
+	clear_cylinder_table(&displayed_dive.cylinders);
 	if (mode == PLAN && current_dive) {
 		// take the displayed cylinders from the selected dive as starting point
-		CylindersModel::instance()->copyFromDive(current_dive);
-		copy_cylinders(current_dive, &displayed_dive, !prefs.display_unused_tanks);
+		copy_used_cylinders(current_dive, &displayed_dive, !prefs.display_unused_tanks);
 		reset_cylinders(&displayed_dive, true);
 
-		for (i = 0; i < MAX_CYLINDERS; i++)
-			if (!cylinder_none(&(displayed_dive.cylinder[i])))
-				return;		// We have at least one cylinder
+		if (displayed_dive.cylinders.nr > 0) {
+			cylinders.updateDive(&displayed_dive);
+			return;		// We have at least one cylinder
+		}
 	}
 	if (!empty_string(prefs.default_cylinder)) {
-		fill_default_cylinder(&displayed_dive, 0);
-		displayed_dive.cylinder[0].start = displayed_dive.cylinder[0].type.workingpressure;
-	}
-	if (cylinder_none(&displayed_dive.cylinder[0])) {
+		cylinder_t cyl = empty_cylinder;
+		fill_default_cylinder(&displayed_dive, &cyl);
+		cyl.start = cyl.type.workingpressure;
+		add_cylinder(&displayed_dive.cylinders, 0, cyl);
+	} else {
+		cylinder_t cyl = empty_cylinder;
 		// roughly an AL80
-		displayed_dive.cylinder[0].type.description = copy_qstring(tr("unknown"));
-		displayed_dive.cylinder[0].type.size.mliter = 11100;
-		displayed_dive.cylinder[0].type.workingpressure.mbar = 207000;
+		cyl.type.description = copy_qstring(tr("unknown"));
+		cyl.type.size.mliter = 11100;
+		cyl.type.workingpressure.mbar = 207000;
+		add_cylinder(&displayed_dive.cylinders, 0, cyl);
 	}
 	reset_cylinders(&displayed_dive, false);
-	CylindersModel::instance()->copyFromDive(&displayed_dive);
+	cylinders.updateDive(&displayed_dive);
 }
 
 // Update the dive's maximum depth.  Returns true if max. depth changed
@@ -204,11 +214,9 @@ void DivePlannerPointsModel::removeDeco()
 
 void DivePlannerPointsModel::addCylinder_clicked()
 {
-	CylindersModel::instance()->add();
+	cylinders.add();
 }
 
-
-
 void DivePlannerPointsModel::setPlanMode(Mode m)
 {
 	mode = m;
@@ -266,13 +274,13 @@ QVariant DivePlannerPointsModel::data(const QModelIndex &index, int role) const
 		case GAS:
 			/* Check if we have the same gasmix two or more times
 			 * If yes return more verbose string */
-			int same_gas = same_gasmix_cylinder(&displayed_dive.cylinder[p.cylinderid], p.cylinderid, &displayed_dive, true);
+			int same_gas = same_gasmix_cylinder(get_cylinder(&displayed_dive, p.cylinderid), p.cylinderid, &displayed_dive, true);
 			if (same_gas == -1)
-				return get_gas_string(displayed_dive.cylinder[p.cylinderid].gasmix);
+				return get_gas_string(get_cylinder(&displayed_dive, p.cylinderid)->gasmix);
 			else
-				return get_gas_string(displayed_dive.cylinder[p.cylinderid].gasmix) +
+				return get_gas_string(get_cylinder(&displayed_dive, p.cylinderid)->gasmix) +
 					QString(" (%1 %2 ").arg(tr("cyl.")).arg(p.cylinderid + 1) +
-						displayed_dive.cylinder[p.cylinderid].type.description + ")";
+						get_cylinder(&displayed_dive, p.cylinderid)->type.description + ")";
 		}
 	} else if (role == Qt::DecorationRole) {
 		switch (index.column()) {
@@ -312,7 +320,7 @@ bool DivePlannerPointsModel::setData(const QModelIndex &index, const QVariant &v
 			if (value.toInt() >= 0) {
 				p.depth = units_to_depth(value.toInt());
 				if (updateMaxDepth())
-					CylindersModel::instance()->updateBestMixes();
+					cylinders.updateBestMixes();
 			}
 			break;
 		case RUNTIME:
@@ -334,12 +342,12 @@ bool DivePlannerPointsModel::setData(const QModelIndex &index, const QVariant &v
 				p.setpoint = po2;
 		} break;
 		case GAS:
-			if (value.toInt() >= 0 && value.toInt() < MAX_CYLINDERS)
+			if (value.toInt() >= 0)
 				p.cylinderid = value.toInt();
 			/* Did we change the start (dp 0) cylinder to another cylinderid than 0? */
 			if (value.toInt() != 0 && index.row() == 0)
-				CylindersModel::instance()->moveAtFirst(value.toInt());
-			CylindersModel::instance()->updateTrashIcon();
+				cylinders.moveAtFirst(value.toInt());
+			cylinders.updateTrashIcon();
 			break;
 		case DIVEMODE:
 			if (value.toInt() < FREEDIVE) {
@@ -409,6 +417,7 @@ int DivePlannerPointsModel::rowCount(const QModelIndex&) const
 }
 
 DivePlannerPointsModel::DivePlannerPointsModel(QObject *parent) : QAbstractTableModel(parent),
+	cylinders(true),
 	mode(NOTHING),
 	recalc(false)
 {
@@ -429,6 +438,14 @@ void DivePlannerPointsModel::emitDataChanged()
 
 void DivePlannerPointsModel::setBottomSac(double sac)
 {
+// mobile delivers the same value as desktop when using
+// units:METERS
+// however when using units:CUFT mobile deliver 0-300 which
+// are really 0.00 - 3.00 so start be correcting that
+#ifdef SUBSURFACE_MOBILE
+	if (qPrefUnits::volume() == units::CUFT)
+		sac /= 100; // cuft without decimals (0 - 300)
+#endif
 	diveplan.bottomsac = units_to_sac(sac);
 	qPrefDivePlanner::set_bottomsac(diveplan.bottomsac);
 	emitDataChanged();
@@ -436,6 +453,14 @@ void DivePlannerPointsModel::setBottomSac(double sac)
 
 void DivePlannerPointsModel::setDecoSac(double sac)
 {
+// mobile delivers the same value as desktop when using
+// units:METERS
+// however when using units:CUFT mobile deliver 0-300 which
+// are really 0.00 - 3.00 so start be correcting that
+#ifdef SUBSURFACE_MOBILE
+	if (qPrefUnits::volume() == units::CUFT)
+		sac /= 100; // cuft without decimals (0 - 300)
+#endif
 	diveplan.decosac = units_to_sac(sac);
 	qPrefDivePlanner::set_decosac(diveplan.decosac);
 	emitDataChanged();
@@ -443,6 +468,12 @@ void DivePlannerPointsModel::setDecoSac(double sac)
 
 void DivePlannerPointsModel::setSacFactor(double factor)
 {
+// sacfactor is normal x.y (one decimal), however mobile
+// delivers 0 - 100 so adjust that to 0.0 - 10.0, to have
+// the same value as desktop
+#ifdef SUBSURFACE_MOBILE
+	factor /= 10.0;
+#endif
 	qPrefDivePlanner::set_sacfactor((int) round(factor * 100));
 	emitDataChanged();
 }
@@ -511,35 +542,55 @@ void DivePlannerPointsModel::setLastStop6m(bool value)
 	emitDataChanged();
 }
 
-void DivePlannerPointsModel::setAscrate75(int rate)
+void DivePlannerPointsModel::setAscrate75Display(int rate)
 {
 	qPrefDivePlanner::set_ascrate75(lrint(rate * UNIT_FACTOR));
 	emitDataChanged();
 }
+int DivePlannerPointsModel::ascrate75Display()
+{
+	return lrint((float)prefs.ascrate75 / UNIT_FACTOR);
+}
 
-void DivePlannerPointsModel::setAscrate50(int rate)
+void DivePlannerPointsModel::setAscrate50Display(int rate)
 {
 	qPrefDivePlanner::set_ascrate50(lrint(rate * UNIT_FACTOR));
 	emitDataChanged();
 }
+int DivePlannerPointsModel::ascrate50Display()
+{
+	return lrint((float)prefs.ascrate50 / UNIT_FACTOR);
+}
 
-void DivePlannerPointsModel::setAscratestops(int rate)
+void DivePlannerPointsModel::setAscratestopsDisplay(int rate)
 {
 	qPrefDivePlanner::set_ascratestops(lrint(rate * UNIT_FACTOR));
 	emitDataChanged();
 }
+int DivePlannerPointsModel::ascratestopsDisplay()
+{
+	return lrint((float)prefs.ascratestops / UNIT_FACTOR);
+}
 
-void DivePlannerPointsModel::setAscratelast6m(int rate)
+void DivePlannerPointsModel::setAscratelast6mDisplay(int rate)
 {
 	qPrefDivePlanner::set_ascratelast6m(lrint(rate * UNIT_FACTOR));
 	emitDataChanged();
 }
+int DivePlannerPointsModel::ascratelast6mDisplay()
+{
+	return lrint((float)prefs.ascratelast6m / UNIT_FACTOR);
+}
 
-void DivePlannerPointsModel::setDescrate(int rate)
+void DivePlannerPointsModel::setDescrateDisplay(int rate)
 {
 	qPrefDivePlanner::set_descrate(lrint(rate * UNIT_FACTOR));
 	emitDataChanged();
 }
+int DivePlannerPointsModel::descrateDisplay()
+{
+	return lrint((float)prefs.descrate / UNIT_FACTOR);
+}
 
 void DivePlannerPointsModel::setVerbatim(bool value)
 {
@@ -752,9 +803,9 @@ void DivePlannerPointsModel::editStop(int row, divedatapoint newData)
 	divepoints[row] = newData;
 	std::sort(divepoints.begin(), divepoints.end(), divePointsLessThan);
 	if (updateMaxDepth())
-		CylindersModel::instance()->updateBestMixes();
+		cylinders.updateBestMixes();
 	if (divepoints[0].cylinderid != old_first_cylid)
-		CylindersModel::instance()->moveAtFirst(divepoints[0].cylinderid);
+		cylinders.moveAtFirst(divepoints[0].cylinderid);
 	emitDataChanged();
 }
 
@@ -795,17 +846,20 @@ void DivePlannerPointsModel::remove(const QModelIndex &index)
 	int i;
 	int rows = rowCount();
 	if (QApplication::keyboardModifiers() & Qt::ControlModifier) {
+		preserved_until.seconds = divepoints.at(index.row()).time;
 		beginRemoveRows(QModelIndex(), index.row(), rows - 1);
 		for (i = rows - 1; i >= index.row(); i--)
 			divepoints.remove(i);
 	} else {
+		if (index.row() == rows -1)
+			preserved_until.seconds = divepoints.at(rows - 1).time;
 		beginRemoveRows(QModelIndex(), index.row(), index.row());
 		divepoints.remove(index.row());
 	}
 	endRemoveRows();
-	CylindersModel::instance()->updateTrashIcon();
+	cylinders.updateTrashIcon();
 	if (divepoints[0].cylinderid != old_first_cylid)
-		CylindersModel::instance()->moveAtFirst(divepoints[0].cylinderid);
+		cylinders.moveAtFirst(divepoints[0].cylinderid);
 }
 
 struct diveplan &DivePlannerPointsModel::getDiveplan()
@@ -860,13 +914,14 @@ void DivePlannerPointsModel::clear()
 {
 	bool oldRecalc = setRecalc(false);
 
-	CylindersModel::instance()->updateDive();
+	cylinders.updateDive(&displayed_dive);
 	if (rowCount() > 0) {
 		beginRemoveRows(QModelIndex(), 0, rowCount() - 1);
 		divepoints.clear();
 		endRemoveRows();
 	}
-	CylindersModel::instance()->clear();
+	cylinders.clear();
+	preserved_until.seconds = 0;
 	setRecalc(oldRecalc);
 }
 
@@ -891,8 +946,8 @@ void DivePlannerPointsModel::createTemporaryPlan()
 	// what does the cache do???
 	struct deco_state *cache = NULL;
 	struct divedatapoint *dp = NULL;
-	for (int i = 0; i < MAX_CYLINDERS; i++) {
-		cylinder_t *cyl = &displayed_dive.cylinder[i];
+	for (int i = 0; i < displayed_dive.cylinders.nr; i++) {
+		cylinder_t *cyl = get_cylinder(&displayed_dive, i);
 		if (cyl->depth.mm && cyl->cylinder_use != NOT_USED) {
 			dp = create_dp(0, cyl->depth.mm, i, 0);
 			if (diveplan.dp) {
@@ -919,7 +974,10 @@ void DivePlannerPointsModel::createTemporaryPlan()
 		cloneDiveplan(&diveplan, plan_copy);
 		unlock_planner();
 #ifdef VARIATIONS_IN_BACKGROUND
-		QtConcurrent::run(this, &DivePlannerPointsModel::computeVariations, plan_copy, &plan_deco_state);
+		// Since we're calling computeVariations asynchronously and plan_deco_state is allocated
+		// on the stack, it must be copied and freed by the worker-thread.
+		struct deco_state *plan_deco_state_copy = new deco_state(plan_deco_state);
+		QtConcurrent::run(this, &DivePlannerPointsModel::computeVariationsFreeDeco, plan_copy, plan_deco_state_copy);
 #else
 		computeVariations(plan_copy, &plan_deco_state);
 #endif
@@ -1004,21 +1062,25 @@ int DivePlannerPointsModel::analyzeVariations(struct decostop *min, struct decos
 	return (leftsum + rightsum) / 2;
 }
 
-void DivePlannerPointsModel::computeVariations(struct diveplan *original_plan, struct deco_state *previos_ds)
+void DivePlannerPointsModel::computeVariationsFreeDeco(struct diveplan *original_plan, struct deco_state *previous_ds)
 {
+	computeVariations(original_plan, previous_ds);
+	delete previous_ds;
+}
+
+void DivePlannerPointsModel::computeVariations(struct diveplan *original_plan, const struct deco_state *previous_ds)
+{
+	// nothing to do unless there's an original plan
+	if (!original_plan)
+		return;
 
-//	bool oldRecalc = setRecalc(false);
 	struct dive *dive = alloc_dive();
 	copy_dive(&displayed_dive, dive);
 	struct decostop original[60], deeper[60], shallower[60], shorter[60], longer[60];
 	struct deco_state *cache = NULL, *save = NULL;
 	struct diveplan plan_copy;
 	struct divedatapoint *last_segment;
-	struct deco_state ds = *previos_ds;
-
-	if (!original_plan)
-		return;
-
+	struct deco_state ds = *previous_ds;
 
 	if (in_planner() && prefs.display_variations && decoMode() != RECREATIONAL) {
 		int my_instance = ++instanceCounter;
@@ -1127,11 +1189,24 @@ void DivePlannerPointsModel::createPlan(bool replanCopy)
 		QTextDocument notesDocument;
 		notesDocument.setHtml(current_dive->notes);
 		QString oldnotes(notesDocument.toPlainText());
-		int disclaimerPosition = oldnotes.indexOf(disclaimer);
-		if (disclaimerPosition == 0)
-			oldnotes.clear();
-		else if (disclaimerPosition >= 1)
-			oldnotes.truncate(disclaimerPosition-1);
+		QString disclaimer = get_planner_disclaimer();
+		int disclaimerMid = disclaimer.indexOf("%s");
+		QString disclaimerBegin, disclaimerEnd;
+		if (disclaimerMid >= 0) {
+			disclaimerBegin = disclaimer.left(disclaimerMid);
+			disclaimerEnd = disclaimer.mid(disclaimerMid + 2);
+		} else {
+			disclaimerBegin = disclaimer;
+		}
+		int disclaimerPositionStart = oldnotes.indexOf(disclaimerBegin);
+		if (disclaimerPositionStart >= 0) {
+			if (oldnotes.indexOf(disclaimerEnd, disclaimerPositionStart) >= 0) {
+				// We found a disclaimer according to the current locale.
+				// Remove the disclaimer and anything after the disclaimer, because
+				// that's supposedly the old planner notes.
+				oldnotes = oldnotes.left(disclaimerPositionStart);
+			}
+		}
 		// Deal with line breaks
 		oldnotes.replace("\n", "<br>");
 		oldnotes.append(displayed_dive.notes);
@@ -1147,15 +1222,23 @@ void DivePlannerPointsModel::createPlan(bool replanCopy)
 	if (!current_dive || displayed_dive.id != current_dive->id) {
 		// we were planning a new dive, not re-planning an existing one
 		displayed_dive.divetrip = nullptr; // Should not be necessary, just in case!
+#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
 		Command::addDive(&displayed_dive, autogroup, true);
-	} else if (replanCopy) {
-		// we were planning an old dive and save as a new dive
-		displayed_dive.id = dive_getUniqID(); // Things will break horribly if we create dives with the same id.
-		Command::addDive(&displayed_dive, false, false);
+#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
 	} else {
-		// we were planning an old dive and rewrite the plan
-		mark_divelist_changed(true);
-		copy_dive(&displayed_dive, current_dive);
+		copy_events_until(current_dive, &displayed_dive, preserved_until.seconds);
+		if (replanCopy) {
+			// we were planning an old dive and save as a new dive
+			displayed_dive.id = dive_getUniqID(); // Things will break horribly if we create dives with the same id.
+#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
+			Command::addDive(&displayed_dive, false, false);
+#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
+		} else {
+			// we were planning an old dive and rewrite the plan
+#if not defined(SUBSURFACE_MOBILE) && not defined(SUBSURFACE_TESTING)
+			Command::replanDive(&displayed_dive);
+#endif // SUBSURFACE_MOBILE SUBSURFACE_TESTING
+		}
 	}
 
 	// Remove and clean the diveplan, so we don't delete
