@@ -21,21 +21,29 @@ public:
 	int rowCount(const QModelIndex &parent = QModelIndex()) const override;
 	Qt::ItemFlags flags(const QModelIndex &index) const override;
 	bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
+	void setTempWS(int row, weightsystem_t ws);
+	void clearTempWS();
+	void commitTempWS();
 
-	void passInData(const QModelIndex &index, const QVariant &value);
-	void add();
 	void clear();
-	void updateDive();
-	weightsystem_t *weightSystemAt(const QModelIndex &index);
-	bool changed;
+	void updateDive(dive *d);
+	weightsystem_t weightSystemAt(const QModelIndex &index) const;
+
+signals:
+	void divesEdited(int num);
 
 public
 slots:
-	void remove(const QModelIndex &index);
 	void weightsystemsReset(const QVector<dive *> &dives);
+	void weightAdded(dive *d, int pos);
+	void weightRemoved(dive *d, int pos);
+	void weightEdited(dive *d, int pos);
 
 private:
-	int rows;
+	dive *d;
+	// If we temporarily change a line because the user is selecting a weight type
+	int tempRow;
+	weightsystem_t tempWS;
 };
 
 #endif
