@@ -9,13 +9,13 @@ CURRENT_LIBCURL="curl-7_54_1"
 CURRENT_LIBUSB="v1.0.21"
 CURRENT_OPENSSL="OpenSSL_1_1_0h"
 CURRENT_LIBSSH2="libssh2-1.8.0"
-CURRENT_XSLT="v1.1.29"
+CURRENT_XSLT="v1.1.34"
 CURRENT_SQLITE="3190200"
 CURRENT_LIBXML2="v2.9.4"
 CURRENT_LIBFTDI="1.3"
-CURRENT_KIRIGAMI="8691651c1f0d09430416ba5fe2130647554b06a9"
-CURRENT_BREEZE_ICONS=""
-CURRENT_GRANTLEE="v5.0.0"
+CURRENT_KIRIGAMI="v5.62.0"
+CURRENT_BREEZE_ICONS="4daac191fb33c8c03bba8356db9767816cb8ee02"
+CURRENT_GRANTLEE="v5.1.0"
 CURRENT_MDBTOOLS="master"
 CURRENT_QT_ANDROID_CMAKE="master"
 
@@ -103,7 +103,7 @@ fi
 
 # FIX FOR ANDROID,
 if [ "$PLATFORM" == "singleAndroid" ] ; then
-	CURRENT_OPENSSL="OpenSSL_1_1_1c"
+	CURRENT_OPENSSL="OpenSSL_1_1_1d"
 # If changing the openSSL version here, make sure to change it in packaging/android/variables.sh also.
 fi
 # no curl and old libs (never version breaks)
@@ -119,7 +119,7 @@ case ${PLATFORM} in
 		PACKAGES=("${COMMON_PACKAGES[@]}" hidapi libcurl libusb openssl libssh2)
 		;;
 	ios)
-		PACKAGES=("${COMMON_PACKAGES[@]}" libxslt)
+		PACKAGES=("${COMMON_PACKAGES[@]}" libxml2 libxslt)
 		;;
 	android)
 		PACKAGES=("${COMMON_PACKAGES[@]}" libxslt sqlite libxml2 openssl libftdi1 libusb)
@@ -163,10 +163,10 @@ for package in "${PACKAGES[@]}" ; do
 			git_checkout_library libxslt $CURRENT_XSLT https://github.com/GNOME/libxslt.git
 			;;
 		breeze-icons)
-			git_checkout_library breeze-icons master https://github.com/kde/breeze-icons.git
+			git_checkout_library breeze-icons $CURRENT_BREEZE_ICONS https://github.com/kde/breeze-icons.git
 			;;
 		googlemaps)
-			git_checkout_library googlemaps master https://github.com/Subsurface-divelog/googlemaps.git
+			git_checkout_library googlemaps master https://github.com/Subsurface/googlemaps.git
 			;;
 		hidapi)
 			git_checkout_library hidapi master https://github.com/signal11/hidapi.git
@@ -188,6 +188,8 @@ for package in "${PACKAGES[@]}" ; do
 			;;
 		grantlee)
 			git_checkout_library grantlee $CURRENT_GRANTLEE https://github.com/steveire/grantlee.git
+			# this should be removed once there is a newer Grantlee release than v5.1.0
+			( cd grantlee ; git -c user.email=in@val.id -c user.name='Subsurface build automation' cherry-pick 1b4f22431ae35dfd11f07a5ae88a1b4db3de2a85 ; cd .. )
 			;;
 		mdbtools)
 			git_checkout_library mdbtools $CURRENT_MDBTOOLS https://github.com/brianb/mdbtools.git
