@@ -247,7 +247,7 @@ void TestParse::testParseNewFormat()
 	 * Set the directory location and file filter for H3 CSV files.
 	 */
 
-	dir = QString::fromLatin1(SUBSURFACE_TEST_DATA "/dives");
+	dir.setPath(QString::fromLatin1(SUBSURFACE_TEST_DATA "/dives"));
 	filter << "TestDiveSeabearH3*.csv";
 	filter << "TestDiveSeabearT1*.csv";
 	files = dir.entryList(filter, QDir::Files);
@@ -286,11 +286,6 @@ void TestParse::testParseDLD()
 	fprintf(stderr, "number of dives from DLD: %d \n", dive_table.nr);
 
 	// Compare output
-	/*
-	 * DC is not cleared from previous tests with the
-	 * clear_dive_file_data(), so we do have an additional DC nick
-	 * name field on the log.
-	 */
 	QCOMPARE(save_dives("./testdldout.ssrf"), 0);
 	FILE_COMPARE("./testdldout.ssrf",
 		     SUBSURFACE_TEST_DATA "/dives/TestDiveDivelogsDE.xml")
@@ -357,7 +352,7 @@ int TestParse::parseCSVmanual(int units, std::string file)
 	params[pnr++] = intdup(21);
 	params[pnr++] = strdup("tagsField");
 	params[pnr++] = intdup(22);
-	// Numbers are indexes of possible options
+	// Numbers are indices of possible options
 	params[pnr++] = strdup("separatorIndex");
 	params[pnr++] = intdup(0);
 	params[pnr++] = strdup("datefmt");
@@ -410,7 +405,7 @@ int TestParse::parseCSVprofile(int units, std::string file)
 	params[pnr++] = intdup(5);
 	params[pnr++] = strdup("pressureField");
 	params[pnr++] = intdup(6);
-	// Numbers are indexes of possible options
+	// Numbers are indices of possible options
 	params[pnr++] = strdup("datefmt");
 	params[pnr++] = intdup(2);
 	params[pnr++] = strdup("units");
